#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_beg_002                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP BMC Election Group (BEG)                            #
#                                                                             #
# Title             : Master state election based on clockClass value         #
#                                                                             #
# Purpose           : Verify that in a boundary clock, the Master state is    #
#                     elected based on the clockClass value, when an Announce #
#                     message with same Priority1 is received from the        #
#                     neighbour.                                              #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.3.4 Page 89                     #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                           <Configure Priority1 = X> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                     SEQ_ID = Y, PRI1 = X, CC = CC1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1,                |           #
#           |           SEQ_ID = A, PRI1 = X, CC = CC1-1]         |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |<Send Sync/ Sync & Follow-up msgs                    |           #
#           |based on clock step>                                 |           #
#           |                                                     |           #
#           |                                                     |           #
#           |                         DELAY_REQ [MSG_TYPE = 0x01, |           #
#           |                               DN = DN1, SEQ_ID = S] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1,                |           #
#           |           SEQ_ID = A, PRI1 = X, CC = CC1+1]         |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                     SEQ_ID = Y, PRI1 = X, CC = CC1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                              SYNC [MSG_TYPE = 0x00, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                          <If Clock Step = Two Step, |           #
#           |                             then receive FOLLOW_UP> |           #
#           |                                                     |           #
#           |                         FOLLOW_UP [MSG_TYPE = 0x08, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     PRI1      = Priority1                                                   #
#     CC        = ClockClass                                                  #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#     v.   Configure Priority1 as X (Default : 128).                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part I)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                                                                             #
# Step 4 : Send periodic ANNOUNCE message with clockClass value decremented   #
#          from the clockClass value of received Announce message on port T1  #
#          with following parameters.                                         #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = A                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1-1                                      #
#                                                                             #
# Step 5a: Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = B                                          #
#                                                                             #
# Step 5b: If the clock is two step, send periodic FOLLOW_UP message on port  #
#          T1 with following parameters.                                      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = B                                          #
#                                                                             #
# Step 6 : Verify that DUT transmits DELAY_REQ on the port P1 with following  #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x01                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = S                                          #
#                                                                             #
# (Part II)                                                                   #
#                                                                             #
# Step 7 : Send ANNOUNCE message with clockClass value incremented from the   #
#          clockClass value of received Announce message on port T1 with      #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = A                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1+1                                      #
#                                                                             #
# Step 8 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                                                                             #
# Step 9a: Verify that DUT transmits SYNC message on the port P1 with         #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 9b: If the clock is two step, verify that DUT transmits FOLLOW_UP      #
#          message on the port P1 with following parameters:                  #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx            Initial                         #
# 1.1          Jan/2017      Veryx            Updated ladder and steps        #
###############################################################################
# Copyright (C) 2013 - 2017 Veryx Technologies(P) Ltd.                        #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that in a boundary clock, the Master state is\
                 elected based on the clockClass value, when an Announce\
                 message with same Priority1 is received from the\
                 neighbour."


########################### START - INITIALIZATION ############################
set tee_port_1		            $::tee_port_num_1
set tee_session_id 	            $::session_id
set dut_port_1		            $::dut_port_num_1
set vlan_id		            $::ptp_vlan_id
set eth_type1		            $::eth_type1
set eth_type2		            $::eth_type2
set dest_mac                        $::dest_mac
set ann_interval                    $::ptp_announce_interval
set clock_step                      $::ptp_clock_step
set sequence_ann                    1
set sequence_sync                   16385
set sequence_foll                   16385
set sequence_del                    32768
set sequence_pdel                   49151
set src_port_general                "320"
set dst_port_general                "320"
set src_port_event                  "319"
set dst_port_event                  "319"
set unicast_flag                    0

set dut_ip                          $::dut_test_port_ip
set tee_ip                          $::tee_test_port_ip

set dut_dest_ip                     $::dut_test_port_ip
set tee_dest_ip                     $::tee_test_port_ip

set tee_dest_mac                    $::dest_mac
set dut_dest_mac                    $::dest_mac
set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval +5))) +$::ptp_user_delay]
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set arp_timeout                    $::ARP_timeout

########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#     v.   Configure Priority1 as X.                                          #
###############################################################################


STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"

    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################
STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1


if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $::ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

###############################################################################
#                                                                             #
#	PRE - REQUISITE CONFIGURATION & ASSIGNING VARIABLES                   #
#                                                                             #
###############################################################################

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

         ERRLOG -msg   "$tee_log_msg(ARP_F1)"
         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}

set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip

}



sleep $::wait_time

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
###############################################################################
### (Part1) ###
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"
### Creating filter - Receive packet ###
if {![ptp::recv_announce \
                           -session_id                 $::session_id\
                           -port_num                   $tee_port_1\
                           -dest_mac                   $tee_dest_mac\
                           -eth_type                   $eth_type1\
                           -vlan_id                    $vlan_id\
                           -type                       $eth_type2\
                           -source_ip                  $dut_ip\
                           -dest_ip                    $tee_dest_ip\
                           -src_port                   $src_port_general\
                           -dest_port                  $dst_port_general\
                           -reset_count                $::reset_count_on\
                           -message_type               $::msg_announce\
                           -domain_number              $::default_domain\
                           -timeout                    $announceReceiptTimeoutInterval\
			   -priority1		       priority1\
			   -recv_gmclockquality_class  recv_gmclockquality_class\
                           -filter_id                  $filter_id\
			   -dut_mac                    dut_mac]} {

          TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

		LOG -level 0 -msg   "$tee_log_msg(TAB)"
		LOG -level 0 -msg   "$tee_log_msg(ANN_RX_CLKCLASS) $recv_gmclockquality_class"

set clock_class  $mypktptpannounce::gmclockquality_class
 if { (($clock_class >= "1") && ($clock_class <= "5")) || ($clock_class == "8") || (($clock_class >= "9") && ($clock_class <= "10")) || (($clock_class >= "11") && ($clock_class <= "12")) || (($clock_class >= "15") && ($clock_class <= "51")) || (($clock_class >= "53") && ($clock_class <= "57")) || (($clock_class >= "59") && ($clock_class <= "67")) || (($clock_class >= "123") && ($clock_class <= "127")) || (($clock_class >= "128") && ($clock_class <= "132")) || (($clock_class >= "171") && ($clock_class <= "186")) || (($clock_class >= "188") && ($clock_class <= "192")) || (($clock_class >= "194") && ($clock_class <= "215")) || (($clock_class >= "233") && ($clock_class <= "247")) || (($clock_class >= "233") && ($clock_class <= "247")) || (($clock_class >= "249") && ($clock_class <= "250")) || ($clock_class == "251") || (($clock_class >= "252") && ($clock_class <= "254")) } {
          LOG -level 0 -msg "The Clock Class value received from DUT:$clock_class"
          TEE_CLEANUP
          TC_CLEAN_AND_ABORT -reason "The Clock Class value received from DUT is $clock_class which is an reserved value"\
                             -tc_def $tc_def
}


set recv_gmclockquality_class1 [expr $recv_gmclockquality_class - 1]

###############################################################################
# Step 4 : Send periodic ANNOUNCE message with clockClass value decremented   #
#          from the clockClass value of received Announce message on port T1  #
#          with following parameters.                                         #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = A                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1-1                                      #
###############################################################################
STEP 4 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_DECR_CLKCLASS_T1)"

        if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $tee_ip\
                           -dest_ip                 $dut_dest_ip\
                           -unicast_flag            $unicast_flag\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -message_type            $::msg_announce\
			   -default_domain          $::default_domain\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmid                    $::gmid\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $priority1\
                           -gmclockquality_class    $recv_gmclockquality_class1]} {
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def
		
				   return 0
	}


if { $clock_step == "One Step" } {
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
}
###############################################################################
# Step 5a : Send periodic SYNC message on port T1 with following parameters.  #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = B                                          #
###############################################################################
STEP 5a 
LOG -level 0 -msg "$tee_log_msg(SYN_TX_PERIOD_T1)"

if {![ptp::send_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
                           -twostep_flag            $::twostep_flag\
                           -unicast_flag            $unicast_flag\
                           -message_length          $::sync_len\
			   -domain_number           $::default_domain\
                           -control_field           $::ctrl_sync\
                           -sequence_id             $sequence_sync]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_TX_T1_F)"\
                              -tc_def $tc_def				   

		return 0
	}


###############################################################################
# Step 5b : Send periodic FOLLOW_UP message on port T1 with following         #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = B                                          #
###############################################################################
if { $clock_step == "Two Step" } {

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
STEP 5b 
LOG -level 0 -msg "$tee_log_msg(FOLL_TX_PERIOD_T1)"

        if {![ptp::send_followup\
			 -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_dest_mac\
                         -eth_type                $eth_type1\
			 -vlan_id		  $vlan_id\
			 -type 		  	  $eth_type2\
		         -source_ip               $tee_ip\
		         -dest_ip	          $dut_dest_ip\
			 -src_port		  $src_port_general\
			 -dest_port		  $dst_port_general\
		  	 -unicast_flag            $unicast_flag\
                         -message_length          $::followup_len\
			 -domain_number           $::default_domain\
                         -control_field           $::ctrl_followup\
                         -sequence_id             $sequence_sync]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_TX_T1_F)"\
                              -tc_def $tc_def				 

		return 0 
	
		}
}	
	
###############################################################################
# Step 6 : Verify that DUT transmits DELAY_REQ on the port P1 with following  #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x01                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = S                                          #
###############################################################################
STEP 6 
LOG -level 0 -msg "$tee_log_msg(DREQ_RX_P1_V)"

	if {![ptp::recv_delayreq \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
  			   -vlan_id		    $vlan_id\
		    	   -type 		    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
			   -reset_count		    $::reset_count_on\
                           -message_type            $::msg_delayreq\
                           -domain_number           $::default_domain\
			   -timeout		    $delay_req_timeout\
                           -filter_id               $filter_id]} {
 
               LOG -level 0 -msg "$tee_log_msg(DREQ_RX_P1_F)"
	       TEE_CLEANUP
               TC_CLEAN_AND_FAIL -reason "DUT does not elect master based on the clockClass value, when an Announce message with same Priority1 is received from the neighbour clock"\
                            -tc_def $tc_def

return 0
}


pltLib::cancel_send_periodic \
                          -instance       3
set recv_gmclockquality_class2 [expr $recv_gmclockquality_class + 1]


###############################################################################
# Step 7 : Send ANNOUNCE message with clockClass value incremented from the   #
#          clockClass value of received Announce message on port T1 with      #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = A                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1+1                                      #
###############################################################################
### (Part2) ###

STEP 7 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_INCR_CLKCLASS_T1)"

	if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $tee_ip\
                           -dest_ip                 $dut_dest_ip\
                           -unicast_flag            $unicast_flag\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -message_type            $::msg_announce\
			   -default_domain          $::default_domain\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $priority1\
                           -gmclockquality_class    $recv_gmclockquality_class2]} {

	       TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def
		return 0
	}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 8 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
###############################################################################
STEP 8
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"
### Creating filter - Receive packet ###
if {![ptp::recv_announce \
                           -session_id                 $::session_id\
                           -port_num                   $tee_port_1\
                           -dest_mac                   $tee_dest_mac\
                           -eth_type                   $eth_type1\
                           -vlan_id                    $vlan_id\
                           -type                       $eth_type2\
                           -source_ip                  $dut_ip\
                           -dest_ip                    $tee_dest_ip\
                           -src_port                   $src_port_general\
                           -dest_port                  $dst_port_general\
                           -reset_count                $::reset_count_on\
                           -message_type               $::msg_announce\
                           -domain_number              $::default_domain\
                           -timeout                    $announceReceiptTimeoutInterval\
                           -gmpriority1                $priority1\
                           -gmclockquality_class       $recv_gmclockquality_class\
                           -filter_id                  $filter_id]} {

# To clear the periodic send / recieve function	
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 9a: Verify that DUT transmits SYNC message on the port P1 with         #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
###############################################################################
STEP 9a 
LOG -level 0 -msg "$tee_log_msg(SYN_RX_VER)"

if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_event\
                           -dest_port               $dst_port_event\
                           -reset_count             $::reset_count_on\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function
         LOG -level 0 -msg "$tee_log_msg(SYN_RX_P1_F)"
         TEE_CLEANUP
         TC_CLEAN_AND_FAIL -reason "DUT does not elect master based on the clockClass value, when an Announce message with same Priority1 is received from the neighbour clock"\
                              -tc_def $tc_def

       return 0
       }


###############################################################################
# Step 9b: Verify that DUT transmits FOLLOW_UP message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
###############################################################################


if { $clock_step == "Two Step" } {

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
STEP 9b 
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_P1_VER)"

if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_on\
                           -message_type            $::msg_followup\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function
          LOG -level 0 -msg "$tee_log_msg(FOLL_RX_P1_F)"
          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "DUT does not elect master based on the clockClass value, when an Announce message with same Priority1 is received from the neighbour clock"\
                              -tc_def $tc_def

       return 0
       }
}

# To clear the periodic send / recieve function	
if { $clock_step == "Two Step" } {
        LOG -level 0 -msg "$tee_log_msg(SYN_FOLL_RX_P1)"
	TEE_CLEANUP
        TC_CLEAN_AND_PASS -reason "DUT elects master based on the clockClass value, when an announce message with same priority1 is received from the neighbour clock"\
                          -tc_def $tc_def

return 1
} else { 

        LOG -level 0 -msg "$tee_log_msg(SYN_RX_P1)"
        TEE_CLEANUP
        TC_CLEAN_AND_PASS -reason "DUT elects master based on the clockClass value, when an announce message with same priority1 is received from the neighbour clock"\
                          -tc_def $tc_def

return 1
}
