#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_beg_004                                  #
# Test Case Version : 1.2                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP BMC Election Group (BEG)                            #
#                                                                             #
# Title             : Master state election based on Variance value           #
#                                                                             #
# Purpose           : Verify that in a boundary clock, the Master state is    #
#                     elected based on the Variance value, when an Announce   #
#                     message with same Priority1, clockClass and             #
#                     clockAccuracy value is received from the neighbour.     #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.3.4 Page 89                     #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                           <Configure Priority1 = X> |           #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, SEQ_ID = Y,    |           #
#           |           PRI1 = X, CC = CC1, CA = CA1, VAR = VAR1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, SEQ_ID = A,    |           #
#           |      PRI1 = X, CC = CC1, CA = CA1, VAR = VAR1-1]    |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |<Send Sync/ Sync & Follow-up msgs                    |           #
#           |based on clock step>                                 |           #
#           |                                                     |           #
#           |                         DELAY_REQ [MSG_TYPE = 0x01, |           #
#           |                               DN = DN1, SEQ_ID = S] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, SEQ_ID = A,    |           #
#           |      PRI1 = X, CC = CC1, CA = CA1, VAR = VAR1+1]    |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |    ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, SEQ_ID = Y, |           #
#           |           PRI1 = X, CC = CC1, CA = CA1, VAR = VAR1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                              SYNC [MSG_TYPE = 0x00, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                          <If Clock Step = Two Step, |           #
#           |                             then receive FOLLOW_UP> |           #
#           |                                                     |           #
#           |                         FOLLOW_UP [MSG_TYPE = 0x08, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     PRI1      = Priority1                                                   #
#     CC        = ClockClass                                                  #
#     CA        = ClockAccuracy                                               #
#     VAR       = Variance                                                    #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#     v.   Configure Priority1 as X.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part I)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                  ClockAccuracy = CA1                                        #
#                  Variance      = VAR1                                       #
#                                                                             #
# Step 4 : Send periodic ANNOUNCE message with Variance value decremented     #
#          from the Variance value of received Announce message on port T1    #
#          with following parameters.                                         #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = A                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                  ClockAccuracy = CA1                                        #
#                  Variance      = VAR1-1                                     #
#                                                                             #
# Step 5a: Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = B                                          #
#                                                                             #
# Step 5b: If the clock is two step, send periodic FOLLOW_UP message on port  #
#          T1 with following parameters.                                      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = B                                          #
#                                                                             #
# Step 6 : Verify that DUT transmits DELAY_REQ on the port P1 with following  #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x01                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = S                                          #
#                                                                             #
# (Part II)                                                                   #
#                                                                             #
# Step 7 : Send ANNOUNCE message with Variance value incremented from the     #
#          Variance value of received Announce message on port T1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = A                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                  ClockAccuracy = CA1                                        #
#                  Variance      = VAR1+1                                     #
#                                                                             #
# Step 8 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                  ClockAccuracy = CA1                                        #
#                  Variance      = VAR1                                       #
#                                                                             #
# Step 9a: Verify that DUT transmits SYNC message on the port P1 with         #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 9b: If the clock is two step, verify that DUT transmits FOLLOW_UP      #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx          Initial                           #
#                                                                             #
# 1.1          Jun/2017      Veryx          Handled clock Variance check      #
#                                           (CR ID - 2450).                   #
#                                                                             #
# 1.2          Nov/2018      Veryx          Clear capture stats before Step 9b#
#                                           was removed. CR: 18275            #
###############################################################################
# Copyright (C) 2017-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that in a boundary clock, the Master state is\
                 elected based on the Variance value, when an Announce\
                 message with same Priority1, clockClass and\
                 clockAccuracy value is received from the neighbor."


########################### START - INITIALIZATION ############################
set tee_port_1		            $::tee_port_num_1
set tee_session_id 	            $::session_id
set dut_port_1		            $::dut_port_num_1
set vlan_id		            $::ptp_vlan_id
set eth_type1		            $::eth_type1
set eth_type2		            $::eth_type2
set dest_mac                        $::dest_mac
set ann_interval                    $::ptp_announce_interval
set clock_step                      $::ptp_clock_step
set sequence_ann                    1
set sequence_sync                   16385
set sequence_foll                   16385
set sequence_del                    32768
set sequence_pdel                   49151
set src_port_general                "320"
set dst_port_general                "320"
set src_port_event                  "319"
set dst_port_event                  "319"
set unicast_flag                    0

set dut_ip                          $::dut_test_port_ip
set tee_ip                          $::tee_test_port_ip

set dut_dest_ip                     $::dut_test_port_ip
set tee_dest_ip                     $::tee_test_port_ip

set tee_dest_mac                    $::dest_mac
set dut_dest_mac                    $::dest_mac
set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr 2*(pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval +5))) +$::ptp_user_delay]
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set arp_timeout                    $::ARP_timeout
########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#     v.   Configure Priority1 as X.                                          #
###############################################################################


STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"

    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################
STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1


if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $::ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

###############################################################################
#                                                                             #
#	PRE - REQUISITE CONFIGURATION & ASSIGNING VARIABLES                   #
#                                                                             #
###############################################################################

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

         ERRLOG -msg   "$tee_log_msg(ARP_F1)"
         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}

set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip

}


sleep $::wait_time

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                  ClockAccuracy = CA1                                        #
#                  Variance      = VAR1                                       #
###############################################################################

STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"
### Creating filter - Receive packet ###
if {![ptp::recv_announce \
                           -session_id                 $::session_id\
                           -port_num                   $tee_port_1\
                           -dest_mac                   $tee_dest_mac\
                           -eth_type   	               $eth_type1\
			   -vlan_id                    $vlan_id\
			   -type                       $eth_type2\
			   -source_ip                  $dut_ip\
			   -dest_ip   	               $tee_dest_ip\
			   -src_port		       $src_port_general\
			   -dest_port                  $dst_port_general\
                           -reset_count                $::reset_count_on\
                           -message_type               $::msg_announce\
                           -domain_number              $::default_domain\
                           -timeout                    $announceReceiptTimeoutInterval\
			   -priority1		       priority1\
			   -recv_gmclockquality_class  recv_gmclockquality_class\
			   -recv_gmclockquality_accu   recv_gmclockquality_accu\
			   -recv_gmclockquality_var    recv_gmclockquality_var\
                           -filter_id                  $filter_id\
			   -dut_mac                    dut_mac]} {


# To clear the periodic send / recieve function	
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

LOG -level 0 -msg   "$tee_log_msg(ANN_RX_PRIOR1) $priority1"
LOG -level 0 -msg   "$tee_log_msg(ANN_RX_CLKCLASS) $recv_gmclockquality_class"
LOG -level 0 -msg   "$tee_log_msg(ANN_RX_CLKACCU) $recv_gmclockquality_accu"
LOG -level 0 -msg   "$tee_log_msg(ANN_RX_CLKVAR) $recv_gmclockquality_var"
if { ($recv_gmclockquality_var == "0") || ($recv_gmclockquality_var == "65535") } {
  set flag 1
  LOG -level 0 -msg  "The Grandmaster Clock-Quality variance value is $recv_gmclockquality_var in DUT"
} else {
set flag 0
}
 
if { $recv_gmclockquality_var != "0" } {

set recv_gmclockquality_var1 [expr $recv_gmclockquality_var - 1]
###############################################################################
# Step 4 : Send periodic ANNOUNCE message with Variance value decremented     #
#          from the Variance value of received Announce message on port T1    #
#          with following parameters.                                         #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = A                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                  ClockAccuracy = CA1                                        #
#                  Variance      = VAR1-1                                     #
###############################################################################

STEP 4 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_DECR_VARIANCE_T1)"

	if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -unicast_flag            $unicast_flag\
                           -message_type            $::msg_announce\
			   -default_domain          $::default_domain\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $priority1\
                           -gmclockquality_class    $recv_gmclockquality_class\
			   -gmclockquality_accu     $recv_gmclockquality_accu\
			   -gmclockquality_var      $recv_gmclockquality_var1]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def
		return 0
	}

if { $clock_step == "One Step" } {
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
}
###############################################################################
# Step 5a : Send periodic SYNC message on port T1 with following parameters.  #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = B                                          #
###############################################################################
STEP 5a 
LOG -level 0 -msg "$tee_log_msg(SYN_TX_PERIOD_T1)"

if {![ptp::send_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
                           -twostep_flag            $::twostep_flag\
		  	   -unicast_flag            $unicast_flag\
			   -clock_step              $clock_step\
                           -message_length          $::sync_len\
			   -domain_number           $::default_domain\
                           -control_field           $::ctrl_sync\
                           -sequence_id             $sequence_sync]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_TX_T1_F)"\
                              -tc_def $tc_def				   

		return 0
	}


###############################################################################
# Step 5b : Send periodic FOLLOW_UP message on port T1 with following         #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = B                                          #
###############################################################################
if { $clock_step == "Two Step" } {

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
STEP 5b 
LOG -level 0 -msg "$tee_log_msg(FOLL_TX_PERIOD_T1)"

        if {![ptp::send_followup\
			 -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_dest_mac\
                         -eth_type                $eth_type1\
			 -vlan_id		  $vlan_id\
			 -type 		  	  $eth_type2\
		         -source_ip               $tee_ip\
		         -dest_ip	          $dut_dest_ip\
			 -src_port		  $src_port_general\
			 -dest_port		  $dst_port_general\
		  	 -unicast_flag            $unicast_flag\
                         -message_length          $::followup_len\
			 -domain_number           $::default_domain\
                         -control_field           $::ctrl_followup\
                         -sequence_id             $sequence_sync]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_TX_T1_F)"\
                              -tc_def $tc_def				 

		return 0 
	
		}
}	
	
###############################################################################
# Step 6 : Verify that DUT transmits DELAY_REQ on the port P1 with following  #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x01                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = S                                          #
###############################################################################
STEP 6 
LOG -level 0 -msg "$tee_log_msg(DREQ_RX_P1_V)"

	if {![ptp::recv_delayreq \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
  			   -vlan_id		    $vlan_id\
		    	   -type 		    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
			   -reset_count		    $::reset_count_on\
                           -message_type            $::msg_delayreq\
                           -domain_number           $::default_domain\
			   -timeout		    $delay_req_timeout\
                           -filter_id               $filter_id]} {
 
               LOG -level 0 -msg "$tee_log_msg(DREQ_RX_P1_F)"
	       TEE_CLEANUP
               TC_CLEAN_AND_FAIL -reason "DUT does not elect master based on the variance value, when an announce message with same Priority1,clockClass and clockAccuracy value is received from the neighbour clock"\
                                  -tc_def $tc_def

return 0
}
#To cancel send_periodic - Announce
pltLib::cancel_send_periodic \
                          -instance       3
#To cancel send_periodic - Sync
pltLib::cancel_send_periodic \
                          -instance       1
#To cancel send_periodic - Follow-up
pltLib::cancel_send_periodic \
                          -instance       2
}
if { $recv_gmclockquality_var != "65535" } {
set recv_gmclockquality_var2 [expr $recv_gmclockquality_var + 1]


###############################################################################
# Step 7 : Send ANNOUNCE message with Variance value incremented from the     #
#          Variance value of received Announce message on port T1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = A                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                  ClockAccuracy = CA1                                        #
#                  Variance      = VAR1+1                                     #
###############################################################################

STEP 7 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_INCR_VARIANCE_T1)"

	if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
	                   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -unicast_flag            $unicast_flag\
                           -message_type            $::msg_announce\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $priority1\
                           -gmclockquality_class    $recv_gmclockquality_class\
                           -gmclockquality_accu     $recv_gmclockquality_accu\
                           -gmid                    $::gmid\
			   -gmclockquality_var      $recv_gmclockquality_var2]} {

	   TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def
		return 0
	}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 8 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                  Priority1     = X                                          #
#                  ClockClass    = CC1                                        #
#                  ClockAccuracy = CA1                                        #
#                  Variance      = VAR1                                       #
###############################################################################

STEP 8
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"
### Creating filter - Receive packet ###
if {![ptp::recv_announce \
                           -session_id                 $::session_id\
                           -port_num                   $tee_port_1\
                           -dest_mac                   $tee_dest_mac\
                           -eth_type   	               $eth_type1\
			   -vlan_id                    $vlan_id\
			   -type                       $eth_type2\
			   -source_ip                  $dut_ip\
			   -dest_ip   	               $tee_dest_ip\
			   -src_port		       $src_port_general\
			   -dest_port                  $dst_port_general\
                           -gmpriority1        	       $priority1\
                           -gmclockquality_class       $recv_gmclockquality_class\
                           -gmclockquality_accu        $recv_gmclockquality_accu\
			   -gmclockquality_var         $recv_gmclockquality_var\
                           -reset_count                $::reset_count_on\
                           -message_type               $::msg_announce\
                           -domain_number              $::default_domain\
                           -timeout                    $announceReceiptTimeoutInterval\
                           -filter_id                  $filter_id]} {

# To clear the periodic send / recieve function	
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 9a: Verify that DUT transmits SYNC message on the port P1 with         #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
###############################################################################

STEP 9a 
LOG -level 0 -msg "$tee_log_msg(SYN_RX_VER)"
if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_event\
                           -dest_port               $dst_port_event\
                           -reset_count             $::reset_count_on\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function
           LOG -level 0 -msg "$tee_log_msg(SYN_RX_P1_F)"
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "DUT does not elect master based on the variance value, when an announce message with same Priority1,clockClass and clockAccuracy value is received from the neighbour clock"\
                              -tc_def $tc_def

       return 0
       }


###############################################################################
# Step 9b: Verify that DUT transmits FOLLOW_UP message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
###############################################################################


if { $clock_step == "Two Step" } {
STEP 9b 
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_P1_VER)"
if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_followup\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function
          LOG -level 0 -msg "$tee_log_msg(FOLL_RX_P1_F)"
          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "DUT does not elect master based on the variance value, when an announce message with same Priority1,clockClass and clockAccuracy value is received from the neighbour clock"\
                              -tc_def $tc_def

       return 0
       }
}

}
if { $clock_step == "Two Step" } {
        if { $flag == 1 } {
        if { $recv_gmclockquality_var == "0" } {
               LOG -level 0 -msg "$tee_log_msg(SYN_FOLL_RX_P1)"
               LOG -level 0 -msg "Election of the master by decrementing the variance value of neighbour clock as compared to the DUT is not verified as the grandmaster clock variance of the DUT is 0"
               TEE_CLEANUP
               TC_CLEAN_AND_PASS -reason "DUT elects master based on the variance value, when an announce message with same Priority1,clockClass and clockAccuracy value is received from the neighbour clock"\
                                 -tc_def $tc_def
}
        if { $recv_gmclockquality_var == "65535" } {        
               LOG -level 0 -msg "$tee_log_msg(DREQ_RX_P1)"
               LOG -level 0 -msg "Election of the master by incrementing the variance value of neighbour clock as compared to the DUT is not verified as the grandmaster clock variance of the DUT is 65535"
               TEE_CLEANUP
               TC_CLEAN_AND_PASS -reason "DUT elects master based on the variance value, when an announce message with same Priority1,clockClass and clockAccuracy value is received from the neighbour clock"\
                                 -tc_def $tc_def
}
} else {        
               LOG -level 0 -msg "$tee_log_msg(SYN_FOLL_RX_P1)"
	TEE_CLEANUP
        TC_CLEAN_AND_PASS -reason "DUT elects master based on the variance value, when an announce message with same Priority1,clockClass and clockAccuracy value is received from the neighbour clock"\
                                  -tc_def $tc_def
}
return 1
} else { 
        if { $flag == 1 } { 
        if { $recv_gmclockquality_var == "0" } {
              LOG -level 0 -msg "$tee_log_msg(SYN_RX_P1)"
              LOG -level 0 -msg "Election of the master by decrementing the variance value of neighbour clock as compared to the DUT is not verified as the grandmaster clock variance of the DUT is 0"
              TEE_CLEANUP
              TC_CLEAN_AND_PASS -reason "DUT elects master based on the variance value, when an announce message with same Priority1,clockClass and clockAccuracy value is received from the neighbour clock"\
                                -tc_def $tc_def
        } 
        if { $recv_gmclockquality_var == "65535" } {        
              LOG -level 0 -msg "$tee_log_msg(DREQ_RX_P1)"
              LOG -level 0 -msg "Election of the master by incrementing the variance value of neighbour clock as compared to the DUT is not verified as the grandmaster clock variance of the DUT is 65535"
              TEE_CLEANUP
              TC_CLEAN_AND_PASS -reason "DUT elects master based on the variance value, when an announce message with same Priority1,clockClass and clockAccuracy value is received from the neighbour clock"\
                                  -tc_def $tc_def

}
} else { 
              LOG -level 0 -msg "$tee_log_msg(SYN_RX_P1)"
	TEE_CLEANUP
        TC_CLEAN_AND_PASS -reason "DUT elects master based on the variance value, when an announce message with same Priority1,clockClass and clockAccuracy value is received from the neighbour clock"\
                                  -tc_def $tc_def
}
return 1
}
