#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mfg_001                                  #
# Test Case Version : 1.2                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Format Group (MFG)                          #
#                                                                             #
# Title             : Frame Format - UDP/IPv4 Announce message                #
#                                                                             #
# Purpose           : Verify that the boundary clock sends the UDP/IPv4       #
#                     Announce message with following frame format.           #
#                                                                             #
#                     Ethernet Header                                         #
#                       1) Source MAC                = Unicast MAC            #
#                       2) Destination MAC           = Unicast/Multicast MAC  #
#                       3) Ether Type                = 0x0800 (IP)            #
#                     IPv4 Header                                             #
#                       4) IP Protocol               = 17 (UDP)               #
#                       5) Destination IP            = Unicast IP/224.0.1.129 #
#                       6) Source IP                 = Unicast IP             #
#                       7) Checksum                  = Valid                  #
#                     UDP Header                                              #
#                       8) UDP Destination Port      = 320                    #
#                     PTP Header                                              #
#                       9) TransportSpecific(4 bits) = 0 or 1, Reserved:1-3   #
#                      10) Message Type     (4 bits) = 0x0b                   #
#                      11) Reserved Bits (1)         = 0 (4 bits)             #
#                      12) Version                   = 2 (4 bits)             #
#                      13) Message Length            = 64(2 Octets)           #
#                      14) Domain Number             = 0-255, Reserved:128-255#
#                      15) Reserved Bits (2)         = 0 (8 bits)             #
#                      16) Flag Field                = 0x0000 - 0xFFFF        #
#                          a)alternateMasterFlag     = 0 or 1                 #
#                          b)unicastFlag             = 0 or 1                 #
#                          c)PTP profile specific1   = 0 or 1                 #
#                          d)PTP profile specific2   = 0 or 1                 #
#                          e) Reserved     = 0 (Octet-0, Bit-7)               #
#                          f) leap61      = 0 or 1                            # 
#                          g) leap59      = 0 or 1                            #
#                          h)currentUtcOffsetValid  = 0 or 1                  #
#                          i)ptpTimescale           = 0 or 1                  #
#                          j)timeTraceable          = 0 or 1                  #
#                          k)frequencyTraceable     = 0 or 1                  #
#                           Note: All unsed flags are Reserved. i.e Zero.     #
#                      17) Correction Field          = 0  (8 octets)          #
#                      18) Reserved Bits (3)         = 0 (32 bits)            #
#                      19) Source Port ID            = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                      20) Sequence ID               = 0 - 65535              #
#                      21) Control Field             = 5                      #
#                      22) Log Message Interval      = 0 to 4                 #
#                      23) Origin Timestamp          = 0 - 10 octets          #
#                           a)secondsField           = 6 octets               #
#                           a)nanosecondsField       = 4 octets               #
#                      24) CurrentUTCoffset          = -32768 to 32767        #
#                      25) Reserved Bits (4)         = 0 (8 bits)             #
#                      26) Grandmaster Priority1     = 0 - 255                #
#                      27) GrandmasterClockQuality   = 4 Octets               #
#                          a) clockClass             = 0 - 255                #
#                          b) clockAccuracy          = 0x00 - 0xFF            #
#                          c) clockVariance          = 0 - 65535              #
#                      28) Grandmaster Priority2     = 0 - 255                #
#                      29) GrandmasterIdentity       = 0x0000000000000000 -   #
#                                                      0xFFFFFFFFFFFFFFFF     #
#                      30) Steps Removed             = 0 - 65535              #
#                      31) TimeSource                = 0x00 - 0xFF            #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 13.3, 13.5 Page 124 - 129 and     #
#                     Annex D                                                 #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                     <Transport Protocol - UDP/IPv4> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                               DN = DN1, SEQ_ID = X] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#     This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Network Transport Protocol as UDP/IPv4.                  #
#     v.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#                     Ethernet Header                                         #
#                       1) Source MAC                = Unicast MAC            #
#                       2) Destination MAC           = Unicast/Multicast MAC  #
#                       3) Ether Type                = 0x0800 (IP)            #
#                     IPv4 Header                                             #
#                       4) IP Protocol               = 17 (UDP)               #
#                       5) Destination IP            = Unicast IP/224.0.1.129 #
#                       6) Source IP                 = Unicast IP             #
#                       7) Checksum                  = Valid                  #
#                     UDP Header                                              #
#                       8) UDP Destination Port      = 320                    #
#                     PTP Header                                              #
#                       9) TransportSpecific(4 bits) = 0 or 1, Reserved:1-3   #
#                      10) Message Type     (4 bits) = 0x0b                   #
#                      11) Reserved Bits (1)         = 0 (4 bits)             #
#                      12) Version                   = 2 (4 bits)             #
#                      13) Message Length            = 64(2 Octets)           #
#                      14) Domain Number             = 0-255, Reserved:128-255#
#                      15) Reserved Bits (2)         = 0 (8 bits)             #
#                      16) Flag Field                = 0x0000 - 0xFFFF        #
#                          a)alternateMasterFlag     = 0 or 1                 #
#                          b)unicastFlag             = 0 or 1                 #
#                          c)PTP profile specific1   = 0 or 1                 #
#                          d)PTP profile specific2   = 0 or 1                 #
#                          e) Reserved               = 0 (Octet-0, Bit-7)     #
#                          f) leap61                 = 0 or 1                 #
#                          g) leap59                 = 0 or 1                 #
#                          h)currentUtcOffsetValid   = 0 or 1                 #
#                          i)ptpTimescale            = 0 or 1                 #
#                          j)timeTraceable           = 0 or 1                 #
#                          k)frequencyTraceable      = 0 or 1                 #
#          Note: All unsed flags are Reserved. i.e Zero.                      #
#                      17) Correction Field          = 0  (8 octets)          #
#                      18) Reserved Bits (3)         = 0 (32 bits)            #
#                      19) Source Port ID            = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                      20) Sequence ID               = 0 - 65535              #
#                      21) Control Field             = 5                      #
#                      22) Log Message Interval      = -128 to +127           #
#                      23) Origin Timestamp          = 0 - 10 octets          #
#                           a)secondsField           = 6 octets               #
#                           a)nanosecondsField       = 4 octets               #
#                      24) CurrentUTCoffset          = -32768 to 32767        #
#                      25) Reserved Bits (4)         = 0 (8 bits)             #
#                      26) Grandmaster Priority1     = 0 - 255                #
#                      27) GrandmasterClockQuality   = 4 Octets               #
#                           a) clockClass            = 0 - 255                #
#                           b) clockAccuracy         = 0x00 - 0xFF            #
#                           c) clockVariance         = 0 - 65535              #
#                      28) Grandmaster Priority2     = 0 - 255                #
#                      29) GrandmasterIdentity       = 0x0000000000000000 -   #
#                                                      0xFFFFFFFFFFFFFFFF     #
#                      30) Steps Removed             = 0 - 65535              #
#                      31) TimeSource                = 0x00 - 0xFF            #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx            Initial                         #
# 1.1          Jan/2017      Veryx            Updated Ladder & relevant steps #
# 1.2          Nov/2018      Veryx            Included timeout in Step 3.     #
#                                             (CR: 18202).                    #
###############################################################################
# Copyright (C) 2013 - 2018 Veryx Technologies(P) Ltd.                        #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock sends the UDP/IPv4  \
                 Announce message with following frame format.\
                 Ethernet Header\
                       1) Source MAC                = Unicast MAC \
                       2) Destination MAC           = Unicast                                        /Multicast MAC   \
                       3) Ether Type                = 0x0800 (IP)   \
                 IPv4 Header                                      \
                       4) IP Protocol               = 17 (UDP)        \
                       5) Destination IP            = Unicast IP                                     /224.0.1.129     \
                       6) Source IP                 = Unicast IP      \
                       7) Checksum                  = Valid         \
                 UDP Header                                       \
                       8) UDP Destination Port      = 319         \
                 PTP Header                                    \
                       4) TransportSpecific (4 bits)= 0 or 1,                                         Reserved:2-F    \
                       5) Message Type      (4 bits)= 0x0b            \
                       6) Reserved Bits (1)         = 0 (4 bits)     \
                       7) Version                   = 2 (4 bits)      \
                       8) Message Length            = 64(2 Octets)    \
                       9) Domain Number             = 0-255, Reserved:128-255                               \
                      10) Reserved Bits (2)        = 0 (8 bits)     \
                      11) Flag Field               = 0x0000 - 0xFFFF     \
                    a)alternateMasterFlag    = 0 or 1              \
                    b)unicastFlag            = 0 or 1              \
                    c)PTP profile specific1  = 0 or 1              \
                    d)PTP profile specific2  = 0 or 1              \
                    e)Reserved               = 0 (Octet-0,                                      Bit-7)             \
                    f)leap61                 = 0 or 1              \
                    g)leap59                 = 0 or 1              \
                    h)currentUtcOffsetValid  = 0 or 1              \
                    i)ptpTimescale           = 0 or 1              \
                    j)timeTraceable          = 0 or 1              \
                    k)frequencyTraceable     = 0 or 1          \
                    Note: All unsed flags are Reserved. i.e Zero.\
                      12) Correction Field         = 0  (8 octets)\
                      13) Reserved Bits (3)        = 0 (32 bits)     \
                      14) Source Port ID           = 0 - 10 octets     \
                    a)clockIdentity            = 8 octets          \
                    b)portNumber               = 2 octets      \
                      15) Sequence ID              = 0 - 65535       \
                      16) Control Field            = 5               \
                      17) Log Message Interval     = 0 to 4          \
                      18) Origin Timestamp         = 0 - 10 octets     \
                    a)secondsField             = 6 octets          \
                    b)nanosecondsField         = 4 octets        \
                      19) CurrentUTCoffset         = -32768 to 32767 \
                      20) Reserved Bits (4)        = 0 (8 bits)      \
                      21) Grandmaster Priority1    = 0 - 255         \
                      22) GrandmasterClockQuality  = 4 Octets          \
                    a)clockClass 	           = 0 - 255             \
                    b)clockAccuracy            = 0x00 - 0xFF       \
                    c)clockVariance            = 0 - 65535       \
                      23) Grandmaster Priority2    = 0 - 255      \
                      24) GrandmasterIdentity      = 0x0000000000000000 -\
                                                      0xFFFFFFFFFFFFFFFF        \
                      25) Steps Removed            = 0 - 65535      \
                      26) TimeSource               = 0x00 - 0xFF"

########################### START - INITIALIZATION ############################
set tee_port_1		 $::tee_port_num_1
set tee_session_id 	 $::session_id
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac

set dut_ip               $::dut_test_port_ip
set tee_ip               $::tee_test_port_ip
set user_delay           $::ptp_user_delay
set announce_timeout     $::ptp_announce_timeout
set announce_interval    $::ptp_announce_interval
set announce_interval    [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr 2* ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout          $::ptp_timeout
set timeout              [expr $ptp_timeout + $user_delay]
set sync_timeout         [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout        [expr  int(pow(2,( $::ptp_sync_interval +1))) +$::ptp_user_delay] 
set arp_timeout          $::ARP_timeout
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len
set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set tee_dest_mac         $::dest_mac
set dut_dest_mac         $::dest_mac
########################### END - INITIALIZATION ##############################

if { $::ptp_transport_type == "IEEE 802.3" } {

        ERRLOG -msg "$tee_log_msg(UDP)" 
        TC_ABORT    -reason "$tee_log_msg(UDP)"\
                    -tc_def $tc_def

  return 0 

}

ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_001

}       
##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###
###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Network Transport Protocol as UDP/IPv4.                  #
#     v.   Configure PTP on port P1.                                          #
###############################################################################
     
STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1]} {

       ERRLOG -msg "$dut_log_msg(INIT_SETUP1_F)"

       TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
                          -tc_def $tc_def
return 0
}

###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add ports T1 and T2 at TEE.                                        #
###############################################################################

STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1
### (Part1) ###

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
                           -vlan_id                 $vlan_id\
                           -ids                     id1]} {
        return 0
}


set filter_id [lindex $id1 end]

if {$::ptp_communication_model == "Unicast" } {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]


if { $result == 0 } {

	ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                             -tc_def $tc_def

       return 0

}

set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

}
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

sleep $::wait_time
###############################################################################
# Step 3 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#                     Ethernet Header                                         #
#                       1) Source MAC                = Unicast MAC            #
#                       2) Destination MAC           = Unicast/Multicast MAC  #
#                       3) Ether Type                = 0x0800 (IP)            #
#                     IPv4 Header                                             #
#                       4) IP Protocol               = 17 (UDP)               #
#                       5) Destination IP            = Unicast IP/224.0.1.129 #
#                       6) Source IP                 = Unicast IP             #
#                       7) Checksum                  = Valid                  #
#                     UDP Header                                              #
#                       8) UDP Destination Port      = 320                    #
#                     PTP Header                                              #
#                       9) TransportSpecific(4 bits) = 0 or 1, Reserved:1-3   #
#                      10) Message Type     (4 bits) = 0x0b                   #
#                      11) Reserved Bits (1)         = 0 (4 bits)             #
#                      12) Version                   = 2 (4 bits)             #
#                      13) Message Length            = 64(2 Octets)           #
#                      14) Domain Number             = 0-255, Reserved:128-255#
#                      15) Reserved Bits (2)         = 0 (8 bits)             #
#                      16) Flag Field                = 0x0000 - 0xFFFF        #
#                          a)alternateMasterFlag     = 0 or 1                 #
#                          b)unicastFlag             = 0 or 1                 #
#                          c)PTP profile specific1   = 0 or 1                 #
#                          d)PTP profile specific2   = 0 or 1                 #
#                          e) Reserved               = 0 (Octet-0, Bit-7)     #
#                          f) leap61                 = 0 or 1                 #
#                          g) leap59                 = 0 or 1                 #
#                          h)currentUtcOffsetValid   = 0 or 1                 #
#                          i)ptpTimescale            = 0 or 1                 #
#                          j)timeTraceable           = 0 or 1                 #
#                          k)frequencyTraceable      = 0 or 1                 #
#          Note: All unsed flags are Reserved. i.e Zero.                      #
#                      17) Correction Field          = 0  (8 octets)          #
#                      18) Reserved Bits (3)         = 0 (32 bits)            #
#                      19) Source Port ID            = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                      20) Sequence ID               = 0 - 65535              #
#                      21) Control Field             = 5                      #
#                      22) Log Message Interval      = -128 to +127           #
#                      23) Origin Timestamp          = 0 - 10 octets          #
#                           a)secondsField           = 6 octets               #
#                           a)nanosecondsField       = 4 octets               #
#                      24) CurrentUTCoffset          = -32768 to 32767        #
#                      25) Reserved Bits (4)         = 0 (8 bits)             #
#                      26) Grandmaster Priority1     = 0 - 255                #
#                      27) GrandmasterClockQuality   = 4 Octets               #
#                           a) clockClass            = 0 - 255                #
#                           b) clockAccuracy         = 0x00 - 0xFF            #
#                           c) clockVariance         = 0 - 65535              #
#                      28) Grandmaster Priority2     = 0 - 255                #
#                      29) GrandmasterIdentity       = 0x0000000000000000 -   #
#                                                      0xFFFFFFFFFFFFFFFF     #
#                      30) Steps Removed             = 0 - 65535              #
#                      31) TimeSource                = 0x00 - 0xFF            #
###############################################################################
STEP 3 
 LOG -level 0 -msg   "$tee_log_msg(ANN_RX_P1_CHECK_V) "

if {![ptp::check_announce \
                           -session_id              $tee_session_id\
                           -port_num                $tee_port_1\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id]} {
           
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_CHECK_F)"\
                              -tc_def $tc_def

       return 0
       } else {

           TEE_CLEANUP
           TC_CLEAN_AND_PASS  -reason "$tee_log_msg(ANN_RX_P1_CHECK)"\
                              -tc_def $tc_def
           
     return 1 
}

