#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mfg_007                                  #
# Test Case Version : 1.2                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Format Group (MFG)                          #
#                                                                             #
# Title             : Frame Format - UDP/IPv4 PDelay_Resp_Follow_Up message   #
#                                                                             #
# Purpose           : Verify that the boundary clock sends the UDP/IPv4       #
#                     PDelay_Resp_Follow_Up message with following frame      #
#                     format.                                                 #
#                                                                             #
#                     Ethernet Header                                         #
#                       1) Source MAC                = Unicast MAC            #
#                       2) Destination MAC           = Unicast/Multicast MAC  #
#                       3) Ether Type                = 0x0800 (IP)            #
#                     IPv4 Header                                             #
#                       4) IP Protocol               = 17 (UDP)               #
#                       5) Destination IP            = Unicast IP/224.0.0.107 #
#                       6) Source IP                 = Unicast IP             #
#                       7) Checksum                  = Valid                  #
#                     UDP Header                                              #
#                       8) UDP Destination Port      = 320                    #
#                     PTP Header                                              #
#                       9) TransportSpecific(4 bits) = 0 or 1, Reserved:1-3   #
#                      10) Message Type              = 0x0a                   #
#                      11) Reserved Bits (1)         = 0 (4 bits)             #
#                      12) Version                   = 2 (4 bits)             #
#                      13) Message Length            = 54 (2 Octets           #
#                      14) Domain Number             = 0-255, Reserved 128-255#
#                      15) Reserved Bits (2)         = 0 (8 bits)             #
#                      16) Flag Field                = 0x0000 - 0xFFFF        #
#                           a)unicastFlag            = 0 or 1                 #
#                           b)PTP profile specific1  = 0 or 1                 #
#                           c)PTP profile specific2  = 0 or 1                 #
#                           d)Reserved               = 0 (Octet-0, Bit-7)     #
#                           Note: All unsed flags are Reserved. i.e Zero.     #
#                      17) Correction Field          = 0 - 8 octets           #
#                      18) Reserved Bits (3)         = 0 (32 bits)            #
#                      19) Source Port ID            = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                      20) Sequence ID               = 0 - 65535              #
#                      21) Control Field             = 5                      #
#                      22) Log Message Interval      = 7F                     #
#                      23) responseReceiptTimestamp  = 0 - 10 octets          #
#                           a)  secondsField         = 6 octets               #
#                           b)  nanosecondsField     = 4 octets               #
#                      24) requestingPortIdentity    = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 13 Page 124 - 128, 132 and Annex D#
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 002                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                             <Clock step - two-step> |           #
#           |                 <Mechanism - Peer Delay on Port P1> |           #
#           |                     <Transport Protocol - UDP/IPv4> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                                             PRT1=Z] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN_1,PRT1=Z+1]                      |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                    SYNC [MSG_TYPE = 0x00, DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |               FOLLOW_UP [MSG_TYPE = 0x08, DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | PDELAY_REQ [MSG_TYPE = 0x02, DN = DN1]              |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                       PDELAY_RESP [MSG_TYPE = 0x03, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |             PDELAY_RESP_FOLLOW_UP [MSG_TYPE = 0x0A, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     PRT1      = Priority1                                                   #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as Two-Step.                                  #
#    iv.   Configure Network Transport Protocol as UDP/IPv4.                  #
#     v.   Configure PTP on port P1.                                          #
#    vi.   Configure mechanism as peer delay mechanism on port P1.            #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                                                                             #
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z+1                                        #
#                                                                             #
#                                                                             #
# Step 5 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 6 : Observe that DUT transmits FOLLOW_UP message on the port P1 with   #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 7 : Send PDELAY_REQ message on port T1 with following parameters.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x02                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 8 : Observe that DUT transmits PDELAY_RESP message on the port P1 with #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x03                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 9 : Verify that DUT transmits PDELAY_RESP_FOLLOW_UP message on the port#
#          P1 with following parameters.                                      #
#                                                                             #
#                     Ethernet Header                                         #
#                       1) Source MAC                = Unicast MAC            #
#                       2) Destination MAC           = Unicast/Multicast MAC  #
#                       3) Ether Type                = 0x0800 (IP)            #
#                     IPv4 Header                                             #
#                       4) IP Protocol               = 17 (UDP)               #
#                       5) Destination IP            = Unicast IP/224.0.0.107 #
#                       6) Source IP                 = Unicast IP             #
#                       7) Checksum                  = Valid                  #
#                     UDP Header                                              #
#                       8) UDP Destination Port      = 320                    #
#                     PTP Header                                              #
#                       9) TransportSpecific(4 bits) = 0 or 1, Reserved:1-3   #
#                      10) Message Type              = 0x0a                   #
#                      11) Reserved Bits (1)         = 0 (4 bits)             #
#                      12) Version                   = 2 (4 bits)             #
#                      13) Message Length            = 54 (2 Octets           #
#                      14) Domain Number             = 0-255, Reserved 128-255#
#                      15) Reserved Bits (2)         = 0 (8 bits)             #
#                      16) Flag Field                = 0x0000 - 0xFFFF        #
#                           a)unicastFlag            = 0 or 1                 #
#                           b)PTP profile specific1  = 0 or 1                 #
#                           c)PTP profile specific2  = 0 or 1                 #
#                           d)Reserved               = 0 (Octet-0, Bit-7)     #
#                           Note: All unsed flags are Reserved. i.e Zero.     #
#                      17) Correction Field          = 0 - 8 octets           #
#                      18) Reserved Bits (3)         = 0 (32 bits)            #
#                      19) Source Port ID            = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                      20) Sequence ID               = 0 - 65535              #
#                      21) Control Field             = 5                      #
#                      22) Log Message Interval      = 7F                     #
#                      23) responseReceiptTimestamp  = 0 - 10 octets          #
#                           a)  secondsField         = 6 octets               #
#                           b)  nanosecondsField     = 4 octets               #
#                      24) requestingPortIdentity    = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx            Initial                         #
# 1.1          Jun/2017      Veryx            The Unwanted display of LOG     #
#                                             print is cleared(CR ID - 2431)  #
# 1.2          Oct/2018      Veryx            All references to semlock were  #
#                                             removed. CR: 18255.             #
###############################################################################
# Copyright (C) 2017-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################
set tc_def      "Verify that the boundary clock sends the UDP/IPv4\
                 PDelay_Resp_Follow_Up message with following frame format.                        \
                 Ethernet Header                                  \
                       1) Source MAC                = Unicast MAC     \
                       2) Destination MAC           = Unicast/                                       Multicast MAC    \
                       3) Ether Type                = 0x0800 (IP) \
                 IPv4 Header                                      \
                       4) IP Protocol               = 17 (UDP)        \
                       5) Destination IP            = Unicast IP                                      /224.0.0.107    \
                       6) Source IP                 = Unicast IP      \
                       7) Checksum                  = Valid       \
                 UDP Header                                       \
                       8) UDP Destination Port      = 319         \
                 PTP Header                                       \
                       9) TransportSpecific(4 bits) = 0 or 1,                                         Reserved:1-3  \
                      10) Message Type              = 0x0a        \
                      11) Reserved Bits (1)         = 0 (4 bits)  \
                      12) Version                   = 2 (4 bits)  \
                      13) Message Length            = 54(2 Octets)\
                      14) Domain Number             = 0-255,                                      Reserved 128-255  \
                      15) Reserved Bits (2)         = 0 (8 bits)  \
                      16) Flag Field                = 0x0000 -                                              0xFFFF       \
                           a)twoStepFlag            = 0 or 1              \
                           b)unicastFlag            = 0 or 1              \
                           c)PTP profile specific1  = 0 or 1              \
                           d)PTP profile specific2  = 0 or 1              \
                           e)Reserved               = 0 (Octet-0,                                           Bit-7)  \
                           Note: All unsed flags are Reserved. i.e Zero.\
                      17) Correction Field          = 0 - 8 octets\
                      18) Reserved Bits (3)         = 0 (32 bits) \
                      19) Source Port ID            = 0 - 10octets        \
                           a)clockIdentity          = 8 octets            \
                           b)portNumber             = 2 octets    \
                      20) Sequence ID               = 0 - 65535   \
                      21) Control Field             = 5           \
                      22) Log Message Interval      = 7F          \
                      23) request Receipt Timestamp = 0 - 10octets        \
                           a)  secondsField         = 6 octets            \
                           b)  nanosecondsField     = 4 octets       \
                      24) requestingPortIdentity    = 0 - 10 octets       \
                           a)clockIdentity          = 8 octets            \
                           b)portNumber             = 2 octets "





########################### START - INITIALIZATION ############################

set tee_port_1		            $::tee_port_num_1
set tee_session_id 	            $::session_id
set dut_port_1		            $::dut_port_num_1
set vlan_id		            $::ptp_vlan_id
set eth_type1		            $::eth_type1
set eth_type2		            $::eth_type2
set dest_mac                        $::dest_mac
set ann_interval                    $::ptp_announce_interval
set clock_step                      $::ptp_clock_step
set pdelay_mac                      $::ptp_delay_mac
set sequence_ann                    1
set sequence_sync                   16385
set sequence_foll                   16385
set sequence_del                    32768
set sequence_pdel                   49151
set src_port_general                "320"
set dst_port_general                "320"
set src_port_event                  "319"
set dst_port_event                  "319"
set unicast_flag                    0

set dut_ip                          $::dut_test_port_ip
set tee_ip                          $::tee_test_port_ip

set tee_pdest_mac                   $::ptp_delay_mac
set dut_pdest_mac                   $::ptp_delay_mac
set dut_dest_ip                     $::dut_test_port_ip
set tee_dest_ip                     $::tee_test_port_ip

set tee_dest_mac                    $::dest_mac
set dut_dest_mac                    $::dest_mac
set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval +1))) +$::ptp_user_delay]
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set arp_timeout                    $::ARP_timeout
	
if { ($::ptp_transport_type != "UDP/IPv4") || ( $clock_step != "Two Step") } {


	TC_ABORT           -reason "$tee_log_msg(UDP_TWO_STEP)"
                           -tc_def $tc_def

  return 0 

}
ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_002

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN


###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as Two-Step.                                  #
#    iv.   Configure Network Transport Protocol as UDP/IPv4.                  #
#     v.   Configure PTP on port P1.                                          #
#    vi.   Configure mechanism as peer delay mechanism on port P1.            #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP2)"


    if {![ptp::dut_configure_setup_002 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

	       ERRLOG -msg "$dut_log_msg(INIT_SETUP2_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP2_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization


set tee_mac           [string toupper  $::TEEMAC_1]


if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $::ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_pdest_mac  $tee_mac
set dut_pdest_mac  $dut_mac
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set tee_pdest_ip   $::tee_test_port_ip
set dut_pdest_ip   $::dut_test_port_ip
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
set tee_pdest_ip $::multicast_pdest_ip
set dut_pdest_ip $::multicast_pdest_ip

}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
sleep $::wait_time
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
###############################################################################

STEP 3
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)" 

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_announce\
                           -domain_number           $::default_domain\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
                           -priority1               priority1\
                           -dut_mac                 dut_mac]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

LOG -level 0 -msg "$tee_log_msg(TAB)"
LOG -level 0 -msg "$tee_log_msg(ANN_RX_PRIOR1):$priority1"
set gmpriority1 [ expr $priority1 + 1 ]

###############################################################################
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z+1                                        #
###############################################################################

STEP 4
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"

        if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -gmpriority1             $gmpriority1\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $tee_ip\
                           -dest_ip                 $dut_dest_ip\
                           -unicast_flag            $unicast_flag\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -message_type            $::msg_announce\
			   -default_domain          $::default_domain\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmid                    $::gmid]} {

              TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def

                return 0
        }

###############################################################################
# Step 5 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
###############################################################################

STEP 5
LOG -level 0 -msg "$tee_log_msg(SYN_RX_OBS)"

if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_event\
                           -dest_port               $dst_port_event\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT  -reason "$tee_log_msg(SYN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

###############################################################################
# Step 6 : Observe that DUT transmits FOLLOW_UP message on the port P1 with   #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
###############################################################################

STEP 6
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_P1_OBS)"
if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_followup\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

          TEE_CLEANUP
          TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

###############################################################################
# Step 7 : Send PDELAY_REQ message on port T1 with following parameters.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x02                                 #
#                  Domain Number       = DN1                                  #
###############################################################################

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
STEP 7
LOG -level 0 -msg "$tee_log_msg(PDREQ_TX_T1)"
if {![ptp::send_pdelayreq\
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $tee_mac\
                        -dest_mac             $dut_pdest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $tee_ip\
                        -dest_ip              $dut_pdest_ip\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -unicast_flag         $unicast_flag\
                        -message_type         $::msg_pdelayreq\
                        -domain_number        $::default_domain\
                        -sequence_id          $sequence_pdel]} {

         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(PDREQ_TX_T1_F)"\
                              -tc_def $tc_def

       return 0
       }

###############################################################################
# Step 8 : Observe that DUT transmits PDELAY_RESP message on the port P1 with #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x03                                 #
#                  Domain Number       = DN1                                  #
###############################################################################

STEP 8
LOG -level 0 -msg "$tee_log_msg(PDRESP_RX_OBS)"
if {![ptp::recv_pdelayresp\
                   -session_id          $::session_id\
                   -port_num            $tee_port_1\
                   -dest_mac            $tee_pdest_mac\
                   -eth_type            $eth_type1\
                   -vlan_id             $vlan_id\
                   -type                $eth_type2\
                   -source_ip           $dut_ip\
                   -dest_ip             $tee_pdest_ip\
                   -src_port            $src_port_event\
                   -dest_port           $dst_port_event\
                   -message_type        $::msg_pdelayresp\
                   -domain_number       $::default_domain\
                   -reset_count         $::reset_count_on\
                   -filter_id           $filter_id\
                   -timeout             $response_timeout\
		   -twostep_flag        $::twostep_flag]} {

              TEE_CLEANUP
              TC_CLEAN_AND_FAIL  -reason "$tee_log_msg(PDRESP_RX_P1_F)"\
                                 -tc_def $tc_def
          return 0

        }

###############################################################################
# Step 9 : Verify that DUT transmits PDELAY_RESP_FOLLOW_UP message on the port#
#          P1 with following parameters.                                      #
#                                                                             #
#                     Ethernet Header                                         #
#                       1) Source MAC                = Unicast MAC            #
#                       2) Destination MAC           = Unicast/Multicast MAC  #
#                       3) Ether Type                = 0x0800 (IP)            #
#                     IPv4 Header                                             #
#                       4) IP Protocol               = 17 (UDP)               #
#                       5) Destination IP            = Unicast IP/224.0.0.107 #
#                       6) Source IP                 = Unicast IP             #
#                       7) Checksum                  = Valid                  #
#                     UDP Header                                              #
#                       8) UDP Destination Port      = 320                    #
#                     PTP Header                                              #
#                       9) TransportSpecific(4 bits) = 0 or 1, Reserved:1-3   #
#                      10) Message Type              = 0x0a                   #
#                      11) Reserved Bits (1)         = 0 (4 bits)             #
#                      12) Version                   = 2 (4 bits)             #
#                      13) Message Length            = 54 (2 Octets           #
#                      14) Domain Number             = 0-255, Reserved 128-255#
#                      15) Reserved Bits (2)         = 0 (8 bits)             #
#                      16) Flag Field                = 0x0000 - 0xFFFF        #
#                           a)unicastFlag            = 0 or 1                 #
#                           b)PTP profile specific1  = 0 or 1                 #
#                           c)PTP profile specific2  = 0 or 1                 #
#                           d)Reserved               = 0 (Octet-0, Bit-7)     #
#                           Note: All unsed flags are Reserved. i.e Zero.     #
#                      17) Correction Field          = 0 - 8 octets           #
#                      18) Reserved Bits (3)         = 0 (32 bits)            #
#                      19) Source Port ID            = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                      20) Sequence ID               = 0 - 65535              #
#                      21) Control Field             = 5                      #
#                      22) Log Message Interval      = 7F                     #
#                      23) responseReceiptTimestamp  = 0 - 10 octets          #
#                           a)  secondsField         = 6 octets               #
#                           b)  nanosecondsField     = 4 octets               #
#                      24) requestingPortIdentity    = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
###############################################################################

STEP 9
LOG -level 0 -msg "$tee_log_msg(PDRESP_FOLL_RX_P1_CHECK_V)"
set clk_mac $dut_mac
if {![ptp::check_pdelayresp_followup \
                           -session_id              $tee_session_id\
                           -port_num                $tee_port_1\
                           -clk_mac                 $clk_mac\
                           -timeout                 $response_timeout\
                           -filter_id               $filter_id]} {

              TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(PDRESP_FOLL_RX_P1_CHECK_F)"\
                              -tc_def $tc_def

       return 0
       } else {

              TEE_CLEANUP
           TC_CLEAN_AND_PASS  -reason "$tee_log_msg(PDRESP_FOLL_RX_P1_CHECK)"\
                              -tc_def $tc_def
           
     return 1
}
