#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mfg_010                                  #
# Test Case Version : 1.2                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Format Group (MFG)                          #
#                                                                             #
# Title             : Frame Format - IEEE 802.3 Follow_Up message             #
#                                                                             #
# Purpose           : Verify that the boundary clock sends the IEEE 802.3     #
#                     Follow_Up message with following frame format.          #
#                                                                             #
#                     Ethernet Header                                         #
#                       1) Source MAC                = Unicast MAC            #
#                       2) Destination MAC           = Unicast MAC/           #
#                                                      01:1B:19:00:00:00      #
#                       3) Ether Type                = PTPv2 over Ethernet    #
#                                                      (IEEE1588) (0x88f7)    #
#                     PTP Header                                              #
#                       4) TransportSpecific(4 bits) = 0 or 1 & Reserved: 2-F #
#                       5) Message Type              = 0x08                   #
#                       6) Reserved Bits (1)         = 0 (4 bits)             #
#                       7) Version                   = 2 (4 bits)             #
#                       8) Message Length            = 44 (2 Octets           #
#                       9) Domain Number             = 0-255, Reserved:128-255#
#                      10) Reserved Bits (2)         = 0 (8 bits)             #
#                      11) Flag Field                = 0x0000 - 0xFFFF        #
#                           a)alternateMasterFlag    = 0                      #
#                           b)unicastFlag            = 0 or 1                 #
#                           c)PTP profile specific1  = 0 or 1                 #
#                           d)PTP profile specific2  = 0 or 1                 #
#                           e) Reserved              = 0 (Octet-0, Bit-7)     #
#                           Note: All unsed flags are Reserved. i.e Zero.     #
#                      12) Correction Field          = 0 - 8 octets           #
#                      13) Reserved Bits (3)         = 0 (32 bits)            #
#                      14) Source Port ID            = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                      15) Sequence ID               = 0 - 65535              #
#                      16) Control Field             = 2                      #
#                      17) Log Message Interval      = -1 to +1[MC] or 7F[UC] #
#                      18) Precise Origin Timestamp  = 0 - 10 octets          #
#                           a)  secondsField         = 6 octets               #
#                           b)  nanosecondsField     = 4 octets               #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 13 Page 124 - 128, 130 and Annex F#
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                             <Clock step - two-step> |           #
#           |                   <Transport Protocol - IEEE 802.3> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                                             PRT1=Z] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=Z+1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                              SYNC [MSG_TYPE = 0x00, |           #
#           |                                          DN = DN1 ] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                         FOLLOW_UP [MSG_TYPE = 0x08, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     PRT1      = Priority1                                                   #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as Two-Step.                                  #
#    iv.   Configure Network Transport Protocol as IEEE 802.3.                #
#     v.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                                                                             #
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z+1                                  #
#                                                                             #
# Step 5 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 6 : Verify that DUT transmits FOLLOW_UP message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#                     Ethernet Header                                         #
#                       1) Source MAC                = Unicast MAC            #
#                       2) Destination MAC           = Unicast MAC/           #
#                                                      01:1B:19:00:00:00      #
#                       3) Ether Type                = PTPv2 over Ethernet    #
#                                                      (IEEE1588) (0x88f7)    #
#                     PTP Header                                              #
#                       4) TransportSpecific(4 bits) = 0 or 1 & Reserved: 2-F #
#                       5) Message Type              = 0x08                   #
#                       6) Reserved Bits (1)         = 0 (4 bits)             #
#                       7) Version                   = 2 (4 bits)             #
#                       8) Message Length            = 44 (2 Octets           #
#                       9) Domain Number             = 0-255, Reserved:128-255#
#                      10) Reserved Bits (2)         = 0 (8 bits)             #
#                      11) Flag Field                = 0x0000 - 0xFFFF        #
#                           a)alternateMasterFlag    = 0                      #
#                           b)unicastFlag            = 0 or 1                 #
#                           c)PTP profile specific1  = 0 or 1                 #
#                           d)PTP profile specific2  = 0 or 1                 #
#                           e) Reserved              = 0 (Octet-0, Bit-7)     #
#                           Note: All unsed flags are Reserved. i.e Zero.     #
#                      12) Correction Field          = 0 - 8 octets           #
#                      13) Reserved Bits (3)         = 0 (32 bits)            #
#                      14) Source Port ID            = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                      15) Sequence ID               = 0 - 65535              #
#                      16) Control Field             = 2                      #
#                      17) Log Message Interval      = -1 to +1[MC] or 7F[UC] #
#                      18) Precise Origin Timestamp  = 0 - 10 octets          #
#                           a)  secondsField         = 6 octets               #
#                           b)  nanosecondsField     = 4 octets               #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2017      Veryx            Initial                         #
# 1.1          Jun/2017      Veryx            The Unwanted display of LOG     #
#                                             print is cleared(CR ID - 2431)  #
# 1.2          Oct/2018      Veryx            All references to semlock were  #
#                                             removed. CR: 18255.             #
###############################################################################
# Copyright (C) 2017-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id

####################### LOCAL VARIABLES ARE DECLARED HERE #####################



set tc_def      "Verify that the boundary clock sends the IEEE 802.3\
                 Follow_up message with following frame format.                                 \
                 Ethernet Header                                \
                       1) Source MAC                = Unicast MAC     \
                       2) Destination MAC           = Unicast MAC/                                   01:1B:19:00:00:00\
                       3) Ether Type                = PTPv2 over                         Ethernet (IEEE1588) (0x88f7)\
                 PTP Header                                    \
                       4) TransportSpecific (4 bits)= 0 or 1,                                         Reserved:2-F    \
                       5) Message Type      (4 bits)= 0x08            \
                       6) Reserved Bits (1)         = 0 (4 bits)     \
                       7) Version                   = 2 (4 bits)      \
                       8) Message Length            = 44(2 Octets)    \
                       9) Domain Number             = 0-255, Reserved:128-255                               \
                      10) Reserved Bits (2)        = 0 (8 bits)     \
                      11) Flag Field               = 0x0000 - 0xFFFF     \
                    a)alternateMasterFlag    = 0                   \
                    b)unicastFlag            = 0 or 1              \
                    c)PTP profile specific1  = 0 or 1              \
                    d)PTP profile specific2  = 0 or 1              \
                    e)Reserved               = 0 (Octet-0,                                      Bit-7)         \
                    Note: All unsed flags are Reserved. i.e Zero.\
                      12) Correction Field         = 0 - 8 octets)\
                      13) Reserved Bits (3)        = 0 (32 bits)     \
                      14) Source Port ID           = 0 - 10 octets     \
                    a)clockIdentity            = 8 octets          \
                    b)portNumber               = 2 octets      \
                      15) Sequence ID              = 0 - 65535       \
                      16) Control Field            = 2               \
                      17) Log Message Interval     = -1 to +1 MC                                    or 7F UC       \
                      18) Origin Timestamp         = 0 - 10 octets     \
                    a)secondsField             = 6 octets          \
                    b)nanosecondsField         = 4 octets"

########################### START - INITIALIZATION ############################
set tee_port_1		 $::tee_port_num_1
set tee_session_id 	 $::session_id
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set unicast_flag   	 0
set src_port_general     "320"
set dst_port_general     "320"
set src_port_event       "319"
set dst_port_event       "319"
set sequence_ann         1
set sequence_sync        16385
set sequence_foll        16385
set sequence_del         49151
set unicast_flag         0
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set dut_ip             $::dut_test_port_ip
set tee_ip             $::tee_test_port_ip

set clock_step         $::ptp_clock_step
set dut_dest_ip        $::dut_test_port_ip
set tee_dest_ip        $::tee_test_port_ip

set tee_dest_mac       $::dest_mac
set dut_dest_mac       $::dest_mac
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set arp_timeout        $::ARP_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout       $::ptp_timeout
set timeout           [expr $ptp_timeout + $user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout [expr  int(pow(2,( $::ptp_sync_interval +5))) +$::ptp_user_delay] 
set pdelay_timeout [expr int(pow(2,($::ptp_pdelay_req_interval)))+ $user_delay]


########################### END - INITIALIZATION ##############################
if { ($::ptp_transport_type == "UDP/IPv4") || ($clock_step == "One Step")} {

        LOG -level 0 -msg   "$tee_log_msg(IEEE802_TWO_STEP)"

	TC_ABORT -reason "$tee_log_msg(IEEE802_TWO_STEP)"
                           -tc_def $tc_def

  return 0 

}

ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as Two-Step.                                  #
#    iv.   Configure Network Transport Protocol as IEEE 802.3.                #
#     v.   Configure PTP on port P1.                                          #
###############################################################################
### (Initial Part) ###
STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

	       ERRLOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1
### (Part1) ###

### Creating filter - Receive packet ###
if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }

set filter_id [lindex $id1 end]

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

	set dut_mac      "dont_care"
	set tee_dest_ip  $::multicast_dest_ip
	set dut_dest_ip  $::multicast_dest_ip
	
} 
 
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
sleep $::wait_time
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN_1                                       #
#                  Priority1     = X                                          #
###############################################################################

STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)" 
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -priority1		    priority1\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
            TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)."\
                              -tc_def $tc_def

       return 0
       }

LOG -level 0 -msg   "$tee_log_msg(ANN_RX_PRIOR1):$priority1"
set recv_priority1 [expr $priority1 + 1 ]

###############################################################################
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN_1                                       #
#                  Priority1     = X+1                                        #
###############################################################################


STEP 4
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"
        if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -message_length          $ann_len\
                           -control_field           $ctrl_announce\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $recv_priority1]} {
                    TEE_CLEANUP
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ANN_TX_T1_F)"\
                                         -tc_def $tc_def
		return 0
	}

	
###############################################################################
# Step 5 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
###############################################################################

STEP 5 
LOG -level 0 -msg "$tee_log_msg(SYN_RX_OBS)"
if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
	                   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip   	            $tee_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_sync\
                           -domain_number           $domain_no\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {
                  TEE_CLEANUP
                  TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_RX_P1_F)."\
                                     -tc_def $tc_def

       return 0
       }	
	
###############################################################################	
# Step 6 : Verify that DUT transmits FOLLOW_UP message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#                     Ethernet Header                                         #
#                       1) Source MAC                = Unicast MAC            #
#                       2) Destination MAC           = Unicast MAC/           #
#                                                      01:1B:19:00:00:00      #
#                       3) Ether Type                = PTPv2 over Ethernet    #
#                                                      (IEEE1588) (0x88f7)    #
#                     PTP Header                                              #
#                       4) TransportSpecific(4 bits) = 0 or 1 & Reserved: 2-F #
#                       5) Message Type              = 0x08                   #
#                       6) Reserved Bits (1)         = 0 (4 bits)             #
#                       7) Version                   = 2 (4 bits)             #
#                       8) Message Length            = 44 (2 Octets           #
#                       9) Domain Number             = 0-255, Reserved:128-255#
#                      10) Reserved Bits (2)         = 0 (8 bits)             #
#                      11) Flag Field                = 0x0000 - 0xFFFF        #
#                           a)alternateMasterFlag    = 0                      #
#                           b)unicastFlag            = 0 or 1                 #
#                           c)PTP profile specific1  = 0 or 1                 #
#                           d)PTP profile specific2  = 0 or 1                 #
#                           e) Reserved              = 0 (Octet-0, Bit-7)     #
#                           Note: All unsed flags are Reserved. i.e Zero.     #
#                      12) Correction Field          = 0 - 8 octets           #
#                      13) Reserved Bits (3)         = 0 (32 bits)            #
#                      14) Source Port ID            = 0 - 10 octets          #
#                           a)clockIdentity          = 8 octets               #
#                           b)portNumber             = 2 octets               #
#                      15) Sequence ID               = 0 - 65535              #
#                      16) Control Field             = 2                      #
#                      17) Log Message Interval      = -1 to +1[MC] or 7F[UC] #
#                      18) Precise Origin Timestamp  = 0 - 10 octets          #
#                           a)  secondsField         = 6 octets               #
#                           b)  nanosecondsField     = 4 octets               #
#                                                                             #
###############################################################################	
	

STEP 6
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_P1_CHECK_V)"

if {![ptp::check_followup \
                      -session_id              $::session_id\
                      -timeout                 $timeout\
                      -port_num                $tee_port_1\
		              -filter_id               $filter_id]} {

                  TEE_CLEANUP
                  TC_CLEAN_AND_FAIL -reason "$tee_log_msg(FOLL_RX_P1_CHECK_F)."\
                                    -tc_def $tc_def

       return 0 
       }
TEE_CLEANUP
TC_CLEAN_AND_PASS  -reason "$tee_log_msg(FOLL_RX_P1_CHECK)"\
                   -tc_def $tc_def
           
return 1

