#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mhg_010                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Handling Group (MHG)                        #
#                                                                             #
# Title             : Ignoring Delay_Req message received on same issued port #
#                                                                             #
# Purpose           : Verify that the boundary clock ignores the Delay_Req    #
#                     message received on the port from which it was issued.  #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.5.2.2 Page 94                   #
#                                                                             #
# Conformance Type  : MAY                                                     #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                                                     |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                  DN = DN1,PRT1 = X] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=X-1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |<Send Sync/ Sync & Follow-up msgs                    |           #
#           |based on clock step>                                 |           #
#           |                                                     |           #
#           |                DELAY_REQ [MSG_TYPE = 0x01,DN = DN1, |           #
#           |                      SRC_PORT_IDEN = E, SEQ_ID = Y] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | DELAY_REQ [MSG_TYPE = 0x01, DN = DN1,               |           #
#           |    SRC_PORT_IDEN = E, SEQ_ID = Y]                   |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |               DELAY_RESP [MSG_TYPE = 0x09, DN = DN1]|           #
#        T1 |                         XX----------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE      = Message Type                                            #
#     DN            = Domain Number                                           #
#     PRT1          = Priority1                                               #
#     SEQ_ID        = Sequence ID                                             #
#     SRC_PORT_IDEN = Source Port Identity                                    #
#     REQ_PORT_IDEN = Requesting Port Identity                                #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = X                                    #
#                                                                             #
# Step 4 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = X - 1                                #
#                  Sequence ID         = A                                    #
#                                                                             #
# Step 5 : Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = B                                    #
#                                                                             #
# Step 6 : If the clock is two step, send periodic FOLLOW_UP message on port  #
#          T1 with following parameters:                                      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = B                                    #
#                                                                             #
# Step 7 : Observe that DUT transmits DELAY_REQ message on the port P1 with   #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = Y                                    #
#                  Source Port Identity= E                                    #
#                                                                             #
# Step 8 : Send the received DELAY_REQ message on port T1 with following      #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = Y                                    #
#                  Source Port Identity= E                                    #
#                                                                             #
# Step 9 : Verify that DUT does not transmit DELAY_RESP message on the port   #
#          P1 with following parameters.                                      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type             = 0x09                            #
#                  Domain Number            = DN1                             #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx            Initial                         #
# 1.1          Jan/2017      Veryx            Updated Ladder diagram and steps#
###############################################################################
# Copyright (C) 2013 - 2017 Veryx Technologies(P) Ltd.                        #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock transmits Delay_Req message\
                 on receiving Sync and Follow_Up on its slave port."


########################### START - INITIALIZATION ############################
set tee_port_1		 $::tee_port_num_1
set tee_session_id 	 $::session_id
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set clock_step           $::ptp_clock_step
set sequence_ann         1
set sequence_sync        16385
set unicast_flag         0

set src_port_general     "320"
set dst_port_general     "320"
set src_port_event       "319"
set dst_port_event       "319"
set dut_ip             $::dut_test_port_ip
set tee_ip             $::tee_test_port_ip
set dut_dest_ip        $::dut_test_port_ip
set tee_dest_ip        $::tee_test_port_ip
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set tee_dest_mac       $::dest_mac
set dut_dest_mac       $::dest_mac
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set arp_timeout        $::ARP_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout       $::ptp_timeout
set response_timeout           [expr $ptp_timeout + $user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout [expr  int(pow(2,( $::ptp_sync_interval +5))) +$::ptp_user_delay] 
set pdelay_timeout [expr int(pow(2,($::ptp_pdelay_req_interval)))+ $user_delay]

ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {


		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
## Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

### (Part1) ###
if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }
set filter_id [lindex $id1 end]

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

         ERRLOG -msg   "$tee_log_msg(ARP_F1)"  

         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1
} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip

}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
sleep $::wait_time
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = X                                    #
###############################################################################
### (Part1) ###
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"
### Creating filter - Receive packet ###
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -priority1		    priority1\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {

# To clear the periodic send / recieve function	
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

		LOG -level 0 -msg   "$tee_log_msg(TAB)"
		LOG -level 0 -msg   "$tee_log_msg(ANN_RX_PRIOR1) $priority1"
set recv_priority1 [expr $priority1 - 1 ]



###############################################################################
# Step 4 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = X - 1                                #
#                  Sequence ID         = A                                    #
###############################################################################

STEP 4 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_PERIOD_T1)"
        if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -message_length          $ann_len\
                           -control_field           $ctrl_announce\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $recv_priority1]} {
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                                         -tc_def $tc_def         
		
		return 0
	}

###############################################################################
# Step 5 : Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = B                                    #
###############################################################################
STEP 5 
LOG -level 0 -msg   "$tee_log_msg(SYN_TX_PERIOD_T1)"
if {![ptp::send_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $tee_ip\
                           -dest_ip                 $dut_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port               $dst_port_event\
                           -unicast_flag            $unicast_flag\
                           -message_length          $sync_len\
                           -domain_number           $domain_no\
                           -twostep_flag            $::twostep_flag\
                           -control_field           $ctrl_sync\
                           -message_type            $msg_sync\
                           -sequence_id             $sequence_sync]} {
                    TEE_CLEANUP
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(SYN_TX_T1_F)"\
                                         -tc_def $tc_def
		return 0
	}

###############################################################################
# Step 6 : If the clock is two step, send periodic FOLLOW_UP message on port  #
#          T1 with following parameters:                                      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = B                                    #
###############################################################################
if { $clock_step == "Two Step" } {
STEP 6 
LOG -level 0 -msg   "$tee_log_msg(FOLL_TX_PERIOD_T1)"
        if {![ptp::send_followup\
                         -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_dest_mac\
                         -eth_type                $eth_type1\
                         -vlan_id                 $vlan_id\
                         -type                    $eth_type2\
                         -source_ip               $tee_ip\
                         -dest_ip                 $dut_dest_ip\
			 -src_port		  $src_port_general\
		         -dest_port               $dst_port_general\
                         -unicast_flag            $unicast_flag\
                         -domain_number           $domain_no\
                         -message_length          $followup_len\
                         -control_field           $ctrl_followup\
                         -message_type            $msg_followup\
                         -sequence_id             $sequence_sync]} {
                    TEE_CLEANUP
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(FOLL_TX_T1_F)"\
                                         -tc_def $tc_def

	return 0 
	
		}


}
###############################################################################
# Step 7 : Observe that DUT transmits DELAY_REQ message on the port P1 with   #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = Y                                    #
#                  Source Port Identity= E                                    #
###############################################################################
STEP 7 
LOG -level 0 -msg   "$tee_log_msg(DREQ_RX_P1_O)"
        if {![ptp::recv_delayreq \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port               $dst_port_event\
                           -reset_count             $reset_count_off\
                           -message_type            $msg_delayreq\
                           -message_length          $delayreq_len\
                           -control_field           $ctrl_delayreq\
                           -domain_number           $domain_no\
                           -timeout                 $delay_timeout\
                           -sportid                 sportid\
                           -seq_id                  seq_id\
                           -clock_id                clock_id\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function	
               TEE_CLEANUP
               TC_CLEAN_AND_ABORT -reason "$tee_log_msg(DREQ_RX_P1_F)"\
                                  -tc_def $tc_def

return 0
}
set seq_del $seq_id
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 8 : Send the received DELAY_REQ message on port T1 with following      #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = Y                                    #
#                  Source Port Identity= E                                    #
###############################################################################
STEP 8 
LOG -level 0 -msg   "Send the received DELAY_REQ message on port T1\n" 
if {![ptp::send_delay_req\
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $tee_mac\
                        -dest_mac             $dut_dest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $tee_ip\
                        -dest_ip              $dut_dest_ip\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -unicast_flag         $unicast_flag\
                        -sequence_id          $seq_del\
                        -message_type         $msg_delayreq\
                        -clockIdentity        $clock_id\
                        -domain_number        $domain_no\
                        -src_port_identity    $sportid]} {
                    TEE_CLEANUP
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(DREQ_TX_T1_F)"\
                                         -tc_def $tc_def

         
       return 0

       }
###############################################################################
# Step 9 : Verify that DUT does not transmit DELAY_RESP message on the port   #
#          P1 with following parameters.                                      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x09                                 #
#                  Domain Number       = DN1                                  #
###############################################################################
STEP 9 
LOG -level 0 -msg "Verify that DUT does not transmit DELAY_RESP message on the port P1\n" 
     if {![ptp::recv_delayresp \
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $dut_mac\
                        -dest_mac             $tee_dest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $dut_ip\
                        -dest_ip              $tee_dest_ip\
                        -src_port             $src_port_general\
                        -dest_port            $dst_port_general\
                        -message_type         $msg_delayresp\
                        -domain_number        $domain_no\
                        -timeout              $response_timeout\
                        -filter_id            $filter_id\
			-control_field        $ctrl_delayresp]} {
# To clear the periodic send / recieve function	
              LOG -level 0 -msg " DUT does not transmit delay response message on the port P1\n" 
	      TEE_CLEANUP
              TC_CLEAN_AND_PASS  -reason "DUT ignores delay request message received on the port from\
                                          which it was issued."\
				 -tc_def $tc_def
          return 1
	} else { 

# To clear the periodic send / recieve function	

         LOG -level 0 -msg " DUT transmits delay response message on the port P1\n" 
         TEE_CLEANUP
         TC_CLEAN_AND_FAIL  -reason "DUT does not ignore delay request message received on the port from\
                                     which it was issued"\
                                 -tc_def $tc_def
          return 0
	
	}

