#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mhg_013                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Handling Group (MHG)                        #
#                                                                             #
# Title             : Terminating multicast Pdelay_Resp message               #
#                                                                             #
# Purpose           : Verify that the boundary clock terminates all multicast #
#                     Pdelay_Resp messages received on the port.              #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.5.1 Page 94                     #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 004                                                     #
# Test Topology     : 2                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                                <Configure PRT1 = Z> |           #
#           |                     <Transmission Type - Multicast> |           #
#           |                 <Mechanism - Peer Delay on Port P1> |           #
#           |                                                     |           #
#           |                                        <Enable PTP> | P1,P2     #
#           |                                                     |           #
#           |                                                     |           #
#           | ANNOUNCE [DEST_MAC = MC1, MSG_TYPE = 0x0B,          |           #
#           |           DN = DN1, PRT1 = Z-1]                     |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |<Send Sync/ Sync & Follow-up msgs                    |           #
#           |based on clock step on Port P1>                      |           #
#           |                                                     |           #
#           |                                                     |           #
#           |        PDELAY_REQ [DEST_MAC = MC1, MSG_TYPE = 0x02, |           #
#           |                         DN = DN1, SRC_PORT_IDEN = E]|           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | PDELAY_RESP [DEST_MAC = MC1, DN = DN1,              |           #
#           |             MSG_TYPE=0x03,SRC_PORT_IDEN=F,          |           #
#           |             REQ_PORT_IDEN = E]                      |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |              PDELAY_RESP [DEST_MAC = MC1, DN = DN1, |           #
#           |                      MSG_TYPE=0x03,SRC_PORT_IDEN=F] |           #
#        T2 |                        XX-----------------------<<--| P2        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#                                                                             #
#     DEST_MAC      = Destination MAC                                         #
#     MSG_TYPE      = Message Type                                            #
#     DN            = Domain Number                                           #
#     PRT1          = Priority1                                               #
#     SRC_PORT_IDEN = Source Port Identity                                    #
#     REQ_PORT_IDEN = Requesting Port Identity                                #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#     This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1, P2 is operational.                        #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Priority1 as Z (Default : 128).                          #
#     v.   Configure transmission type as Multicast.                          #
#    vi.   Configure delay mechanism as peer delay mechanism on port P1.      #
#   vii.   Enable PTP on port P1 and P2.                                      #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1, T2 at TEE.                                            #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Send periodic multicast ANNOUNCE message on port T1 with following #
#          parameters.                                                        #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x0B                                #
#                  Domain Number        = DN1                                 #
#                  Priority1            = Z-1                                 #
#                                                                             #
# Step 4a: Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x00                                #
#                  Domain Number        = DN1                                 #
#                                                                             #
# Step 4b: If the clock is two step clock, send periodic FOLLOW_UP message    #
#          on port T1 with following parameters:                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x08                                #
#                  Domain Number        = DN1                                 #
#                                                                             #
# Step 5 : Observe that DUT transmits multicast PDELAY_REQ message on the port#
#          P1 with following parameters.                                      #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x02                                #
#                  Domain Number        = DN1                                 #
#                  Source Port Identity = E                                   #
#                                                                             #
# Step 6 : Send multicast PDELAY_RESP message on port T1 with following       #
#          parameters.                                                        #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC          = MC1                             #
#              PTP Header                                                     #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
#                  Source Port Identity     = F                               #
#                  Requesting Port Identity = E                               #
#                                                                             #
# Step 7 : Verify that DUT does not forward the PDELAY_RESP received on port  #
#          P1 to the port P2 with the following parameters.                   #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x03                                #
#                  Domain Number        = DN1                                 #
#                  Source Port Identity = F                                   #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jun/2017      Veryx         Initial                            #
# 1.1          Oct/2018      Veryx         Port list that was passed in       #
#                                          Step 2 was updated (CR: 18266).    #
#                                          Updated the wait timeout for       #
#                                          pDelay request (CR: 18275).        #
###############################################################################
# Copyright (C) 2017-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock terminates all multicast\
                 Pdelay_Resp messages received on the port."


########################### START - INITIALIZATION ############################
set tee_port_1		 $::tee_port_num_1
set tee_port_2		 $::tee_port_num_2
set tee_session_id 	 $::session_id
set dut_port             $::dut_port_list_2
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set clock_step           $::ptp_clock_step
set pdelay_mac           $::ptp_delay_mac
set sequence_ann         1
set sequence_sync        16385
set sequence_foll        16385
set sequence_pdelreq     49151
set unicast_flag         0
set src_port_general     "320"
set dst_port_general     "320"
set src_port_event       "319"
set dst_port_event       "319"
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set dut_ip               $::dut_test_port_ip
set tee_ip               $::tee_test_port_ip

set tee_pdest_mac        $::ptp_delay_mac
set dut_pdest_mac        $::ptp_delay_mac
set tee_dest_ip          $::multicast_dest_ip
set dut_dest_ip          $::multicast_dest_ip

set tee_pdest_ip       $::multicast_pdest_ip
set tee_dest_mac       $::dest_mac
set dut_dest_mac       $::dest_mac
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set arp_timeout        $::ARP_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout        $::ptp_timeout
set response_timeout   [expr $ptp_timeout + $user_delay]
set sync_timeout       [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout      [expr  int(pow(2,( $::ptp_sync_interval + 5))) +$::ptp_user_delay] 
set pdelay_timeout     [expr (2 * int(pow(2,($::ptp_pdelay_req_interval)))) + $user_delay] 

if {$::ptp_communication_model == "Unicast"} {

	LOG -level 0 -msg   "$tee_log_msg(MC)"

         TC_ABORT   -reason "$tee_log_msg(MC)"\
                    -tc_def $tc_def

       return 0

}
ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_004
}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1, P2 is operational.                        #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Priority1 as Z (Default : 128).                          #
#     v.   Configure transmission type as Multicast.                          #
#    vi.   Configure delay mechanism as peer delay mechanism on port P1.      #
#   vii.   Enable PTP on port P1 and P2.                                      #
###############################################################################

STEP 1 

LOG -level 0 -msg "$dut_log_msg(INIT_SETUP4)"

    if {![ptp::dut_configure_setup_004 \
                       -port_list	   $dut_port]} {

                LOG -level 0 -msg "$dut_log_msg(INIT_SETUP5_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP5_F)"\
				   -tc_def $tc_def
		return 0
	}



###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1, T2 at TEE.                                            #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

if {![ptp::tee_configure_setup_002 \
                -port_list        $tee_port_2]} {

      ERRLOG -msg "$::tee_log_msg(INIT_SETUP2_F)"

      TC_ABORT -reason "$::tee_log_msg(INIT_SETUP2_F)"\

     }



set tee_mac            $::TEEMAC_1


if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }
if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_2\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id2]} {
        return 0
        }
set filter_id [lindex $id1 end]
set filter_id1 [lindex $id2 end]


pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_2
sleep $::wait_time
set gmpriority1 [expr $::ptp_dut_clock_priority1 - 1]
###############################################################################
# Step 3 : Send periodic multicast ANNOUNCE message on port T1 with following #
#          parameters.                                                        #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x0B                                #
#                  Domain Number        = DN1                                 #
#                  Priority1            = Z-1                                 #
###############################################################################

STEP 3 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_PERIOD_MC_T1)"

        if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -gmpriority1             $gmpriority1\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $tee_ip\
                           -dest_ip                 $dut_dest_ip\
                           -unicast_flag            $unicast_flag\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -message_type            $::msg_announce\
			   -domain_number           $::default_domain\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmid                    $::gmid]} {

	       TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def

                return 0
        }
if { $clock_step == "One Step" } {
pltLib::reset_all_stats\
        -session_id         $::session_id\
        -port_num           $tee_port_1
}
###############################################################################
# Step 4a: Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x00                                #
#                  Domain Number        = DN1                                 #
###############################################################################

STEP 4a
LOG -level 0 -msg   "$tee_log_msg(SYN_TX_PERIOD_MC_T1)"

if {![ptp::send_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
                           -message_type            $::msg_sync\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
		  	   -unicast_flag            $unicast_flag\
                           -twostep_flag            $::twostep_flag\
                           -message_length          $::sync_len\
			   -domain_number           $::default_domain\
                           -control_field           $::ctrl_sync\
                           -sequence_id             $sequence_sync]} {

                    TEE_CLEANUP
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(SYN_TX_T1_F)"\
                              -tc_def $tc_def				   

		return 0
	}

###############################################################################
# Step 4b: If the clock is two step clock, send periodic FOLLOW_UP message    #
#          on port T1 with following parameters:                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x08                                #
#                  Domain Number        = DN1                                 #
###############################################################################

if { $clock_step == "Two Step" } {
pltLib::reset_all_stats\
        -session_id         $::session_id\
        -port_num           $tee_port_1
STEP 4b 
LOG -level 0 -msg   "$tee_log_msg(FOLL_TX_PERIOD_MC_T1)"

        if {![ptp::send_followup\
			 -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_dest_mac\
                         -eth_type                $eth_type1\
			 -vlan_id		  $vlan_id\
			 -type 		  	  $eth_type2\
                         -message_type            $::msg_followup\
		         -source_ip               $tee_ip\
		         -dest_ip	          $dut_dest_ip\
			 -src_port		  $src_port_general\
			 -dest_port		  $dst_port_general\
		  	 -unicast_flag            $unicast_flag\
                         -message_length          $::followup_len\
			 -domain_number           $::default_domain\
                         -control_field           $::ctrl_followup\
                         -sequence_id             $sequence_sync]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_TX_T1_F)"\
                                         -tc_def $tc_def				 

		return 0 
	
		}

}

###############################################################################
# Step 5 : Observe that DUT transmits multicast PDELAY_REQ message on the port#
#          P1 with following parameters.                                      #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x02                                #
#                  Domain Number        = DN1                                 #
#                  Source Port Identity = E                                   #
###############################################################################

STEP 5
LOG -level 0 -msg "$tee_log_msg(PDREQ_RX_MC_P1_OBS)"
if {![ptp::recv_pdelayreq\
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -dest_mac             $tee_pdest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $dut_ip\
                        -dest_ip              $tee_pdest_ip\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -message_type         $::msg_pdelayreq\
                        -seq_id               seq_id\
                        -sportid              sportid\
                        -clock_id             clock_id\
                        -timeout              $pdelay_timeout\
                        -domain_number        $::default_domain\
                        -filter_id            $filter_id]} {

	  TEE_CLEANUP	
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(PDREQ_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 6 : Send multicast PDELAY_RESP message on port T1 with following       #
#          parameters.                                                        #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC          = MC1                             #
#              PTP Header                                                     #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
#                  Source Port Identity     = F                               #
#                  Requesting Port Identity = E                               #
###############################################################################
STEP 6
LOG -level 0 -msg "$tee_log_msg(PDRESP_TX_MC_T1)"

if {![ptp::send_pdelayresp\
			 -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_pdest_mac\
                         -eth_type                $eth_type1\
			 -vlan_id		  $vlan_id\
			 -type 		          $eth_type2\
                         -message_type            $::msg_pdelayresp\
		         -source_ip               $tee_ip\
		         -dest_ip	          $tee_pdest_ip\
			 -src_port		  $src_port_event\
			 -dest_port		  $dst_port_event\
		  	 -unicast_flag            $unicast_flag\
                         -twostep_flag            $::twostep_flag\
                         -message_length          $::pdelayresp_len\
			 -domain_number           $::default_domain\
                         -control_field           $::ctrl_pdelayresp\
			 -get_src_port_identity   srcprtid\
			 -reqngport_id            $sportid\
                         -reqngport_idnty         $clock_id\
                         -sequence_id             $seq_id]} {
	           
                   TEE_CLEANUP
		   TC_CLEAN_AND_ABORT -reason "$tee_log_msg(PDRESP_TX_T1_F)"\
                              -tc_def $tc_def
		return 0 
	
		}

if { $::clock_Identity_format == "EUI-64 constructed from a EUI-48" } {
set clockidP1 [clock_identity $tee_mac]
set clockidP1 [string tolower $clockidP1]
} else {
set clockidP1 [clock_identity_noneui $tee_mac]
set clockidP1 [string tolower $clockidP1]
}

###############################################################################
# Step 7 : Verify that DUT does not forward the PDELAY_RESP received on port  #
#          P1 to the port P2 with the following parameters.                   #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC      = MC1                                 #
#              PTP Header                                                     #
#                  Message Type         = 0x03                                #
#                  Domain Number        = DN1                                 #
#                  Source Port Identity = F                                   #
###############################################################################

STEP 7
LOG -level 0 -msg "Verify that DUT does not forward the PDELAY_RESP received on port p1 to the port p2"

if {![ptp::recv_pdelayresp\
                   -session_id          $::session_id\
                   -port_num            $tee_port_2\
                   -dest_mac            $tee_pdest_mac\
                   -eth_type            $eth_type1\
                   -vlan_id             $vlan_id\
                   -type                $eth_type2\
                   -source_ip           $dut_ip\
                   -dest_ip             $tee_pdest_ip\
                   -src_port            $src_port_event\
                   -dest_port           $dst_port_event\
                   -message_type        $::msg_pdelayresp\
                   -src_port_identity   $srcprtid\
                   -clockIdentity       $clockidP1\
                   -domain_number       $::default_domain\
                   -filter_id           $filter_id1\
                   -timeout             $response_timeout]} {

           LOG -level 0 -msg   "DUT does not forward pdelay response message received on port P1 to port P2\n"
	   TEE_CLEANUP
           TC_CLEAN_AND_PASS -reason "DUT terminates multicast pdelay response messages received on the port P1"\
                             -tc_def $tc_def

return 1
} else {
          LOG -level 0 -msg   "DUT forwards pdelay response message received on port P1 to port P2\n"
          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "DUT does not terminate multicast pdelay response message received on port P1"\
                            -tc_def $tc_def

return 0
}
