#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mhg_015                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Handling Group (MHG)                        #
#                                                                             #
# Title             : Ignoring Pdelay_Req message received on same issued port#
#                                                                             #
# Purpose           : Verify that the boundary clock ignores the Pdelay_Req   #
#                     message received on the port from which it was issued.  #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.5.2.2 Page 94                   #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 002                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                                <Configure PRT1 = Z> |           #
#           |                 <Mechanism - Peer Delay on Port P1> |           #
#           |                                        <Enable PTP> | P1        #
#           |                                                     |           #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=Z-1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |<Send Sync/ Sync & Follow-up msgs                    |           #
#           |based on clock step>                                 |           #
#           |                                                     |           #
#           |               PDELAY_REQ [MSG_TYPE = 0x02,DN = DN1, |           #
#           |                      SRC_PORT_IDEN = E, SEQ_ID = Y] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | PDELAY_REQ [MSG_TYPE = 0x02, DN = DN1,              |           #
#           |    SRC_PORT_IDEN = E, SEQ_ID = Y]                   |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |              PDELAY_RESP [MSG_TYPE = 0x03, DN = DN1]|           #
#        T1 |                         XX----------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE      = Message Type                                            #
#     DN            = Domain Number                                           #
#     PRT1          = Priority1                                               #
#     SEQ_ID        = Sequence ID                                             #
#     SRC_PORT_IDEN = Source Port Identity                                    #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#     This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock Step as One-step/ two-step.                        #
#    iv.   Configure Priority1 as Z (Default : 128).                          #
#     v.   Configure delay mechanism as peer delay mechanism on port P1.      #
#    vi.   Enable PTP on port P1.                                             #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z - 1                                #
#                                                                             #
# Step 4 : Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 5 : If the clock is two step, send periodic FOLLOW_UP message on port  #
#          T1 with following parameters:                                      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 6 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x02                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = Y                                    #
#                  Source Port Identity= E                                    #
#                                                                             #
# Step 7 : Send prior received PDELAY_REQ message on port T1 with following   #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x02                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = Y                                    #
#                  Source Port Identity= E                                    #
#                                                                             #
# Step 8 : Verify that DUT ignores the received PDELAY_REQ message(from which #
#          it was issued) and does not transmit PDELAY_RESP message on the    #
#          port P1 with following parameters.                                 #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jun/2017      Veryx         Initial                            #
# 1.1          Nov/2018      Veryx         Updated the wait timeout for       #
#                                          pDelay request (CR: 18275).        #
###############################################################################
# Copyright (C) 2017-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock ignores the Pdelay_Req\
                 message received on the port from which it was issued."


########################### START - INITIALIZATION ############################
set tee_port_1		 $::tee_port_num_1
set tee_session_id 	 $::session_id
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set clock_step           $::ptp_clock_step
set pdelay_mac           $::ptp_delay_mac
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set twostep_flag         $::twostep_flag 
set sequence_ann         1
set sequence_sync        16385
set sequence_foll        16385
set sequence_pdelreq     49151
set unicast_flag         0
set src_port_general     "320"
set dst_port_general     "320"
set src_port_event       "319"
set dst_port_event       "319"
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout        $::ptp_timeout
set response_timeout   [expr $ptp_timeout + $user_delay]
set sync_timeout       [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set arp_timeout        $::ARP_timeout
set pdelay_timeout     [expr (2 * int(pow(2,($::ptp_pdelay_req_interval)))) + $user_delay] 
set dut_ip             $::dut_test_port_ip
set tee_ip             $::tee_test_port_ip

set dut_dest_ip        $::dut_test_port_ip
set tee_dest_ip        $::tee_test_port_ip
set tee_pdest_mac      $::ptp_delay_mac
set dut_pdest_mac      $::ptp_delay_mac

set tee_dest_mac       $::dest_mac
set dut_dest_mac       $::dest_mac

########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_002
}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock Step as One-step/ two-step.                        #
#    iv.   Configure Priority1 as Z (Default : 128).                          #
#     v.   Configure delay mechanism as peer delay mechanism on port P1.      #
#    vi.   Enable PTP on port P1.                                             #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP2)"


    if {![ptp::dut_configure_setup_002 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP2_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP2_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################
STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1


if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

###############################################################################
#                                                                             #
#	PRE - REQUISITE CONFIGURATION & ASSIGNING VARIABLES                   #
#                                                                             #
###############################################################################

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

         ERRLOG -msg "$tee_log_msg(ARP_F1)"

        TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}

set tee_pdest_mac  $tee_mac
set dut_pdest_mac  $dut_mac
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set tee_pdest_ip   $::tee_test_port_ip
set dut_pdest_ip   $::dut_test_port_ip
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
set tee_pdest_ip $::multicast_pdest_ip
set dut_pdest_ip $::multicast_pdest_ip
}



pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1


set gm_priority1 [expr $::ptp_dut_clock_priority1 - 1 ]
###############################################################################
# Step 3 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z - 1                                #
###############################################################################
### (Part1) ###
sleep $::wait_time
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"
if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
			   -domain_number           $::default_domain\
                           -message_length          $ann_len\
                           -gmid                    $::gmid\
                           -control_field           $ctrl_announce\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $gm_priority1]} {
	            TEE_CLEANUP	
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ANN_TX_T1_F)"\
                                         -tc_def $tc_def

		return 0
	}
if { $clock_step == "One Step" } {
pltLib::reset_all_stats\
        -session_id         $::session_id\
        -port_num           $tee_port_1
}
###############################################################################
# Step 4a: Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
###############################################################################
	
STEP 4a 
LOG -level 0 -msg "$tee_log_msg(SYN_TX_PERIOD_T1)"

if {![ptp::send_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
		  	   -unicast_flag            $unicast_flag\
			   -twostep_flag            $::twostep_flag\
                           -message_type            $msg_sync\
                           -message_length          $::sync_len\
			   -domain_number           $::default_domain\
                           -control_field           $::ctrl_sync\
                           -sequence_id             $sequence_sync]} {

           TEE_CLEANUP 
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_TX_T1_F)"\
                              -tc_def $tc_def				   

		return 0
	}

###############################################################################
# Step 4b: If the clock is two step clock, send periodic FOLLOW_UP message    #
#          on port T1 with following parameters:                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
###############################################################################

if { $clock_step == "Two Step" } {
pltLib::reset_all_stats\
        -session_id         $::session_id\
        -port_num           $tee_port_1
STEP 4b 
LOG -level 0 -msg "$tee_log_msg(FOLL_TX_PERIOD_T1)"

        if {![ptp::send_followup\
			 -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_dest_mac\
                         -eth_type                $eth_type1\
			 -vlan_id		  $vlan_id\
			 -type 		  	  $eth_type2\
		         -source_ip               $tee_ip\
		         -dest_ip	          $dut_dest_ip\
			 -src_port		  $src_port_general\
			 -dest_port		  $dst_port_general\
                         -message_type            $msg_followup\
		  	 -unicast_flag            $unicast_flag\
                         -message_length          $::followup_len\
			 -domain_number           $::default_domain\
                         -control_field           $::ctrl_followup\
                         -sequence_id             $sequence_sync]} {

           TEE_CLEANUP 
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_TX_T1_F)"\
                              -tc_def $tc_def				 

		return 0 
	
		}
}
		
###############################################################################		
# Step 5 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x02                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = Y                                    #
#                  Source Port Identity= E                                    #
###############################################################################

STEP 5
LOG -level 0 -msg "$tee_log_msg(PDREQ_RX_OBS)"

if {![ptp::recv_pdelayreq \
                           -session_id                 $::session_id\
                           -port_num                   $tee_port_1\
                           -dest_mac                   $tee_pdest_mac\
                           -eth_type   	               $eth_type1\
			   -vlan_id                    $vlan_id\
			   -type                       $eth_type2\
			   -source_ip                  $dut_ip\
			   -dest_ip   	               $tee_pdest_ip\
			   -src_port		       $src_port_event\
			   -dest_port                  $dst_port_event\
			   -reset_count		       $reset_count_off\
                           -message_type               $msg_pdelayreq\
			   -domain_number              $domain_no\
			   -timeout                    $pdelay_timeout\
                           -filter_id                  $filter_id\
                           -sportid                    sportid\
                           -clock_id                   clock_id\
                           -seq_id                     seq_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(PDREQ_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }


set  seq_id1 $seq_id
set  sportid1 $sportid
set  clock_id1 $clock_id  

pltLib::reset_all_stats\
        -session_id         $::session_id\
        -port_num           $tee_port_1
###############################################################################
# Step 6 : Send prior received PDELAY_REQ message on port T1 with following   #
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x02                                 #
#                  Domain Number       = DN1                                  #
#                  Sequence ID         = Y                                    #
#                  Source Port Identity= E                                    #
###############################################################################

STEP 6 
LOG -level 0 -msg "$tee_log_msg(PDREQ_TX_T1)"

if {![ptp::send_pdelayreq\
                        -session_id            $::session_id\
                        -port_num              $tee_port_1\
                        -source_mac            $tee_mac\
                        -dest_mac              $dut_pdest_mac\
                        -eth_type              $eth_type1\
                        -vlan_id               $vlan_id\
                        -type                  $eth_type2\
                        -source_ip             $tee_ip\
                        -dest_ip               $dut_pdest_ip\
                        -src_port              $src_port_event\
                        -dest_port             $dst_port_event\
                        -unicast_flag          $unicast_flag\
                        -message_type          $::msg_pdelayreq\
                        -domain_number         $::default_domain\
                        -src_port_identity     $sportid1\
                        -clockIdentity         $clock_id1\
                        -sequence_id           $seq_id1]} {

         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(PDREQ_TX_T1_F)"\
                              -tc_def $tc_def

       return 0
       }

#Here this send pdelayreq msg is not periodic as prior to recv_pdelayreq, so we are killing the instance after sending 1 packet .
pltLib::cancel_send_periodic \
                          -instance       6
###############################################################################
# Step 7 : Verify that DUT ignores the received PDELAY_REQ message(from which #
#          it was issued) and does not transmit PDELAY_RESP message on the    #
#          port P1 with following parameters.                                 #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
###############################################################################

STEP 7
LOG -level 0 -msg "$tee_log_msg(PDREQ_IGNOR_NO_PDRESP_RX_P1_V)"

if {![ptp::recv_pdelayresp\
                   -session_id          $::session_id\
                   -port_num            $tee_port_1\
                   -dest_mac            $tee_pdest_mac\
                   -eth_type            $eth_type1\
                   -vlan_id             $vlan_id\
                   -type                $eth_type2\
                   -source_ip           $dut_ip\
                   -dest_ip             $tee_pdest_ip\
                   -src_port            $src_port_event\
                   -dest_port           $dst_port_event\
                   -message_type        $::msg_pdelayresp\
                   -domain_number       $::default_domain\
                   -timeout             $response_timeout\
                   -filter_id           $filter_id]} {


	   LOG -level 0 -msg "$tee_log_msg(PDRESP_RX_P1_F)"

           TEE_CLEANUP

           TC_CLEAN_AND_PASS  -reason "DUT ignores the received PDELAY_REQ message and does not transmit PDELAY_RESP on the port P1"\
                              -tc_def $tc_def

       return 1
       }

	   LOG -level 0 -msg "$tee_log_msg(PDRESP_RX_P1_P)"

           TEE_CLEANUP
           TC_CLEAN_AND_FAIL  -reason "DUT does not ignore the received PDELAY_REQ message and transmits PDELAY_RESP on the port P1"\
                              -tc_def $tc_def
           return 0
