#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mtg_005                                  #
# Test Case Version : 1.0                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Transaction Group (MTG)                     #
#                                                                             #
# Title             : alternateMasterFlag FALSE - Delay Request-Response      #
#                     Mechanism                                               #
#                                                                             #
# Purpose           : Verify that the boundary clock port supporting Delay    #
#                     Request-Response mechanism acting as master has         #
#                     alternateMasterFlag set to FALSE in Announce,Sync,      #
#                     Follow_Up,Delay_Resp PTP messsages.                     # 
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 7.3.8.2 Page 46                   #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                   <Clock mode - one step/ two-step> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                   SEQ_ID = X,PRT1 = Z, AM_FLAG = 0] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=Z+1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                    SYNC [MSG_TYPE = 0x00, DN = DN1, |           #
#           |                                        AM_FLAG = 0] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                          <If Clock Step = Two Step, |           #
#           |                             then receive FOLLOW_UP> |           #
#           |                                                     |           #
#           |               FOLLOW_UP [MSG_TYPE = 0x08, DN = DN1, |           #
#           |                                        AM_FLAG = 0] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | DELAY_REQ [MSG_TYPE = 0x01, DN = DN1]               |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |              DELAY_RESP [MSG_TYPE = 0x09, DN = DN1, |           #
#           |                                        AM_FLAG = 0] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     PRT1      = Priority1                                                   #
#     AM_FLAG   = AlternateMasterFlag                                         #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                  AlternateMasterFlag = 0                                    #
#                                                                             #
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z+1                                        #
#                                                                             #
# Step 5a: Verify  that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                  AlternateMasterFlag = 0                                    #
#                                                                             #
# Step 5b:If clock is two step clock, verify that DUT transmits FOLLOW_UP     #
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                  AlternateMasterFlag = 0                                    #
#                                                                             #
# Step 6 : Send DELAY_REQ message on port T1 with following parameters.       #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 7 : Verify that DUT transmits DELAY_RESP message on the port P1 with   #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x09                                 #
#                  Domain Number       = DN1                                  #
#                  AlternateMasterFlag = 0                                    #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx            Initial                         #
###############################################################################
# Copyright (C) 2017 Veryx Technologies(P) Ltd.                               #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock port supporting Delay\
                 Request-Response mechanism acting as master has\
		 alternateMasterFlag set to FALSE in Announce,Sync,\
		 Follow_Up,Delay_Resp PTP messsages."


########################### START - INITIALIZATION ############################
set tee_port_1                     $::tee_port_num_1
set tee_session_id                 $::session_id
set dut_port_1                     $::dut_port_num_1
set vlan_id                        $::ptp_vlan_id
set eth_type1                      $::eth_type1
set eth_type2                      $::eth_type2
set dest_mac                       $::dest_mac
set ann_interval                   $::ptp_announce_interval
set clock_step                     $::ptp_clock_step
set sequence_ann                   1
set sequence_sync                  16385
set sequence_foll                  16385
set sequence_del                   32768
set src_port_general               "320"
set dst_port_general               "320"
set src_port_event                 "319"
set dst_port_event                 "319"
set unicast_flag                   0

set dut_ip                         $::dut_test_port_ip
set tee_ip                         $::tee_test_port_ip

set dut_dest_ip                    $::dut_test_port_ip
set tee_dest_ip                    $::tee_test_port_ip

set tee_dest_mac                   $::dest_mac
set dut_dest_mac                   $::dest_mac

set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval + 5))) +$::ptp_user_delay]
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set arp_timeout                    $::ARP_timeout
	
ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN


###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization


set tee_mac           [string toupper  $::TEEMAC_1]


if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $::ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

         ERRLOG -msg "$tee_log_msg(ARP_F1)"

        TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip

}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 3 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                  AlternateMasterFlag = 0                                    #
###############################################################################

sleep $::wait_time
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_ALTMAS_FALSE_VER)"

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $::msg_announce\
                           -domain_number           $::default_domain\
			   -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
                           -alternatemaster         alternatemaster\
			   -priority1               priority1\
			   -dut_mac                 dut_mac]} {

        TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }
		LOG -level 0 -msg   "$tee_log_msg(TAB)"
		LOG -level 0 -msg   "$tee_log_msg(ANN_RX_PRIOR1) $priority1"

set gmpriority1 [ expr $priority1 + 1 ]
if { $alternatemaster != "0" } {
        TEE_CLEANUP
	TC_CLEAN_AND_FAIL -reason "DUT doesn't sends Announce message with Alternate master flag set to false"\
                              -tc_def $tc_def
	return 0
} else {
	 LOG -level 0 -msg "DUT sends announce message with Alternate master flag set to false"

 }

###############################################################################
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z+1                                        #
###############################################################################
 
STEP 4 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_PERIOD_T1)"

	if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
			   -gmpriority1             $gmpriority1\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
		  	   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $::msg_announce\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmid                    $::gmid]} {

        TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def				   

		return 0
	}
###############################################################################
# Step 5a: Verify that DUT transmits SYNC message on the port P1 with         #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                  AlternateMasterFlag = 0                                    #
###############################################################################
	
STEP 5a
LOG -level 0 -msg "$tee_log_msg(SYN_RX_ALTMAS_FLSE_VER)"

if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_event\
                           -dest_port               $dst_port_event\
                           -reset_count             $::reset_count_off\
                           -seq_id                  seq_id\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -alternatemaster         alternatemaster\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def

       return 0
       }

if { $alternatemaster != "0" } {

        TEE_CLEANUP
	TC_CLEAN_AND_FAIL -reason "DUT does not send Sync message with Alternate master flag set to false on Port P1"\
                              -tc_def $tc_def
	return 0
} else {
	 LOG -level 0 -msg "DUT sends Sync message with Alternate master flag set to false on Port P1"

 }
set seq_id1 $seq_id
###############################################################################
# Step 5b:If clock is two step clock, verify that DUT transmits FOLLOW_UP     #
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                  AlternateMasterFlag = 0                                    #
###############################################################################

if { $clock_step == "Two Step" } {

STEP 5b
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_ALTMAS_FLSE_VER)"

if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_followup\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -sequence_id             $seq_id1\
                           -sportid                 sportid\
                           -alternatemaster         alternatemaster\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function
          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(FOLL_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

if { $alternatemaster != "0" } {

        TEE_CLEANUP
	TC_CLEAN_AND_FAIL -reason "DUT does not send Follow up message with Alternate master flag set to false on Port P1"\
                              -tc_def $tc_def
	return 0
} else {
	 LOG -level 0 -msg "DUT sends Follow up message with Alternate master flag set to false on Port P1"

 }
}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 6 : Send DELAY_REQ message on port T1 with following parameters.       #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
###############################################################################


STEP 6 
LOG -level 0 -msg "$tee_log_msg(DREQ_TX_T1)"

        if {![ptp::send_delay_req\
                         -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_dest_mac\
                         -eth_type                $eth_type1\
                         -vlan_id                 $vlan_id\
                         -type                    $eth_type2\
                         -source_ip               $tee_ip\
                         -dest_ip                 $dut_dest_ip\
                         -unicast_flag            $unicast_flag\
                         -src_port                $src_port_event\
                         -dest_port               $dst_port_event\
                         -sequence_id             $sequence_del\
                         -domain_number           $::default_domain\
                         -message_type            $::msg_delayreq]} {
           
        TEE_CLEANUP
		   TC_CLEAN_AND_ABORT -reason "$tee_log_msg(DREQ_TX_T1_F)"\
                              -tc_def $tc_def
                return 0

                }

###############################################################################
# Step 7 : Verify that DUT transmits DELAY_RESP message on the port P1 with   #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x09                                 #
#                  Domain Number       = DN1                                  #
#                  AlternateMasterFlag = 0                                    #
###############################################################################

STEP 7 "Verify that DUT transmits DELAY_RESP message on the port P1"

if {![ptp::recv_delayresp \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -message_type            $::msg_delayresp\
                           -domain_number           $::default_domain\
                           -timeout                 $response_timeout\
                           -filter_id               $filter_id\
                           -alternatemaster         alternatemaster]} {

# To clear the periodic send / recieve function
          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "DUT does not send delay_resp message on port P1."\
                              -tc_def $tc_def

       return 0
       }


if { $alternatemaster != "0" } {
        TEE_CLEANUP
	TC_CLEAN_AND_FAIL -reason "DUT does not send delay response message with alternate master flag set to false on Port P1"\
                              -tc_def $tc_def
	return 0
} else {
        LOG -level 0 -msg "DUT sends delay response message with alternate master flag set to false on Port P1\n"
}
 
       TEE_CLEANUP
        TC_CLEAN_AND_PASS -reason  "DUT send announce,sync/sync, follow up and delay response message with Alternate master flag set to false when it is in master state"\
                                  -tc_def $tc_def
        return 1
 

