#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mtg_007                                  #
# Test Case Version : 1.2                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Transaction Group (MTG)                     #
#                                                                             #
# Title             : Sync Message Transmission Interval                      #
#                                                                             #
# Purpose           : Verify that the boundary clock transmits Sync messages  #
#                     for every sync interval.                                #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 7.7.2.3 Page 62                   #
#                                                                             #
# Conformance Type  : MAY                                                     #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                  DN = DN1,PRT1 = X] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=X+1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                    SYNC [MSG_TYPE = 0x00, DN = DN1, |           #
#           |                         SEQ_ID = Y, Timestamp = Ta] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                    SYNC [MSG_TYPE = 0x00, DN = DN1, |           #
#           |                       SEQ_ID = Y+1, Timestamp = Tb] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                    SYNC [MSG_TYPE = 0x00, DN = DN1, |           #
#           |                       SEQ_ID = Y+2, Timestamp = Tc] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |             <Check ((Tb - Ta) + (Tc - Tb))/2 = SI > |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     PRT1      = Priority1                                                   #
#     SEQ_ID    = Sequence ID                                                 #
#     SI        = Sync Interval                                               #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = X                                          #
#                                                                             #
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = X+1                                        #
#                                                                             #
# Step 5 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters and capture the timestamp(Ta) at which the    #
#          message was received.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                                                                             #
# Step 6 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters and capture the timestamp(Tb)  at which the   #
#          message was received.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y+1                                        #
#                                                                             #
# Step 7 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters and capture the timestamp(Tc)  at which the   #
#          message was received.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y+2                                        #
#                                                                             #
# Step 8 : Verify that DUT transmits SYNC message for every Sync interval.    #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx         Initial                            #
# 1.1          Jan/2017      Veryx         Updated Ladder and Steps           #
# 1.2          Nov/2018      Veryx         Updated the periodicity calculation#
#                                          (CR: 18275)                        #
###############################################################################
# Copyright (C) 2013 - 2018 Veryx Technologies(P) Ltd.                        #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id

####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set Time1 [expr (pow(2,$::ptp_sync_interval))]

set low_Time1  [expr $Time1 - ($Time1 * (30.0/100))]
set high_Time1 [expr $Time1 + ($Time1 * (30.0/100))]


set tc_def      "Verify that the boundary clock transmits Sync \
                 messages for every sync interval."


########################### START - INITIALIZATION ############################
set tee_port_1                     $::tee_port_num_1
set tee_session_id                 $::session_id
set dut_port_1                     $::dut_port_num_1
set vlan_id                        $::ptp_vlan_id
set eth_type1                      $::eth_type1
set eth_type2                      $::eth_type2
set dest_mac                       $::dest_mac
set ann_interval                   $::ptp_announce_interval
set clock_step                     $::ptp_clock_step
set sequence_ann                   1
set sequence_sync                  16385
set sequence_foll                  16385
set src_port_general               "320"
set dst_port_general               "320"
set src_port_event                 "319"
set dst_port_event                 "319"
set unicast_flag                   0

set dut_ip                         $::dut_test_port_ip
set tee_ip                         $::tee_test_port_ip

set dut_dest_ip                    $::dut_test_port_ip
set tee_dest_ip                    $::tee_test_port_ip

set tee_dest_mac                   $::dest_mac
set dut_dest_mac                   $::dest_mac

set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval + 5))) +$::ptp_user_delay]
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set arp_timeout                    $::ARP_timeout

ptp::dut_Initialization
########################### END - INITIALIZATION ##############################

proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################
### (Initial Part) ###
STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1
### (Part1) ###

### Creating filter - Receive packet ###
if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $::ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }

set filter_id [lindex $id1 end]

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

         ERRLOG -msg "$tee_log_msg(ARP_F1)"

        TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

	set dut_mac      "dont_care"
	set tee_dest_ip  $::multicast_dest_ip
	set dut_dest_ip  $::multicast_dest_ip
	
} 


pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z                                          #
###############################################################################

sleep $::wait_time
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_announce\
                           -domain_number           $::default_domain\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
                           -priority1               priority1\
                           -dut_mac                 dut_mac]} {

        TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }
		LOG -level 0 -msg   "$tee_log_msg(TAB)"
		LOG -level 0 -msg   "$tee_log_msg(ANN_RX_PRIOR1) $priority1"

set gmpriority1 [ expr $priority1 + 1 ]

###############################################################################
# Step 4 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = A                                          #
#                  Priority1     = Z+1                                        #
###############################################################################

STEP 4 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_PERIOD_T1)"

        if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -gmpriority1             $gmpriority1\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $tee_ip\
                           -dest_ip                 $dut_dest_ip\
                           -unicast_flag            $unicast_flag\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -message_type            $::msg_announce\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmid                    $::gmid]} {

        TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def

                return 0
        }

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 5 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
###############################################################################
STEP 5 
LOG -level 0 -msg   "$tee_log_msg(SYN_RX_OBS)"

if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_event\
                           -dest_port               $dst_port_event\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -rx_timestamp_ns         Ta\
			   -seq_id             	    seq1_id\
                           -filter_id               $filter_id]} {

       TEE_CLEANUP
       TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_RX_P1_F)"\
                          -tc_def $tc_def

       return 0
       }
LOG -level 0 -msg "$tee_log_msg(SYN_RX_SEQID) $seq1_id"

set Ta  [format %.2f [expr $Ta/1000000000.0]]

LOG -level 0 -msg "Sync message from DUT is received at (Ta): $Ta secs\n"

###############################################################################
# Step 6 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y+1                                        #
###############################################################################

STEP 6  
LOG -level 0 -msg   "$tee_log_msg(SYN_RX_OBS)"

if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_event\
                           -dest_port               $dst_port_event\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -rx_timestamp_ns         Tb\
			   -seq_id             	    seq2_id\
                           -filter_id               $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

set Tb  [format %.2f [expr $Tb/1000000000.0]]

LOG -level 0 -msg "Sync message from DUT is received at (Tb): $Tb secs\n"

###############################################################################
# Step 7 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y+2                                        #
###############################################################################
STEP 7 
LOG -level 0 -msg   "$tee_log_msg(SYN_RX_OBS)"

if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_event\
                           -dest_port               $dst_port_event\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -rx_timestamp_ns         Tc\
			   -seq_id             	    seq3_id\
                           -filter_id               $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

LOG -level 0 -msg "DUT sends Sync message with Sequence ID :  $seq3_id\n"

set Tc  [format %.2f [expr $Tc/1000000000.0]]

LOG -level 0 -msg "Sync message from DUT is received at (Tc): $Tc secs\n"

set t1 [format %.2f [expr $Tb - $Ta]]
set t2 [format %.2f [expr $Tc - $Tb]]

###############################################################################
# Step 8 : Verify that DUT transmits SYNC message for every Sync interval.    #
###############################################################################
STEP 8
LOG -level 0 -msg "Verify that DUT transmits SYNC message for every Sync interval\n"

	LOG -level 0 -msg "DUT sends first and second SYNC message at an interval of $t1 secs\n"
	LOG -level 0 -msg "DUT sends second and third SYNC message at an interval of $t2 secs\n"

        set periodicity [ expr ($t1 + $t2)/2]

	LOG -level 0 -msg "Expected range: $low_Time1 - $high_Time1 secs, Observed average value: $periodicity secs\n"

	if { ($seq2_id == [expr $seq1_id + 1]) && ($seq3_id == [expr $seq2_id + 1])} {
	LOG -level 0 -msg "DUT sends SYNC messages with incremental Sequence ID\n"
	set seq_result 1
	   } else {
	LOG -level 0 -msg "DUT does not send SYNC messages with incremental Sequence ID\n"
	set seq_result 0
	}

	if {($periodicity >= $low_Time1 && $periodicity <= $high_Time1)} {
	 	LOG -level 0 -msg "DUT sends SYNC messages with average periodicity within +/- 30% of the configured Sync Interval\n"
		set interval_result 1
	} else {
		LOG -level 0 -msg "DUT does not send SYNC messages with average periodicity within +/- 30% of the configured Sync Interval\n"
                 
                 
		set interval_result 0
	}


if { $seq_result && $interval_result } {
# To clear the periodic send / receive function	
           TEE_CLEANUP
           TC_CLEAN_AND_PASS -reason "DUT transmits SYNC message for every Sync interval"\
                              -tc_def $tc_def

           return 1
   } else {
# To clear the periodic send / receive function	
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "DUT does not transmit SYNC message for every Sync interval"\
                              -tc_def $tc_def

           return 0
	
}
