#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mtg_011                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Transaction Group (MTG)                     #
#                                                                             #
# Title             : Unicast Transmission of Sync and Follow_Up messages     #
#                                                                             #
# Purpose           : Verify that boundary clock configured for unicast       #
#                     transmission shall send follow_up message to same       #
#                     unicast address as the associated sync message.         #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.5.10 Page 104                   #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                             <Clock step - two-step> |           #
#           |                       <Transmission Type - Unicast> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                 DN = DN1,PRT1 = Z]  |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=Z+1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |              SYNC [DEST MAC = UC1, MSG_TYPE = 0x00, |           #
#           |                               DN = DN1, SEQ_ID = X] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |           FOLLOW_UP [DEST_MAC = UC1, MSG_TYPE = 0x08|           #
#           |                               DN = DN1, SEQ_ID = X] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     SRC MAC   = Source MAC                                                  #
#     DEST MAC  = Destination MAC                                             #
#     MSG_TYPE  = Message Type                                                #
#     PRT1      = Priority1                                                   #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     UC        = Unicast                                                     #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as two-step.                                  #
#    iv.   Configure transmission type as Unicast.                            #
#     v.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                                                                             #
# Step 4 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z+1                                  #
#                                                                             #
# Step 5 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC = UC1                                      #
#              PTP Header                                                     #
#                  Message Type    = 0x00                                     #
#                  Domain Number   = DN1                                      #
#                  Sequence Id     = X                                        #
#                                                                             #
# Step 6 : Verify that DUT transmits FOLLOW_UP message to the same unicast    #
#          destination MAC  as the associated SYNC message on the port P1     #
#          with following parameters.                                         #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC = UC1                                      #
#              PTP Header                                                     #
#                  Message Type    = 0x08                                     #
#                  Domain Number   = DN1                                      #
#                  Sequence Id     = X                                        #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx            Initial                         #
#                                                                             #
# 1.1          Jun/2017      Veryx            ptp_ethtype updated             #
#                                             (CR ID - 2428).                 #
###############################################################################
# Copyright (C) 2017 Veryx Technologies(P) Ltd.                               #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that boundary clock configured for unicast\
                 transmission shall send follow_up message to same\
                 unicast address as the associated sync message."


########################### START - INITIALIZATION ############################
set tee_port_1		            $::tee_port_num_1
set tee_session_id 	            $::session_id
set dut_port_1		            $::dut_port_num_1
set vlan_id		            $::ptp_vlan_id
set eth_type1		            $::eth_type1
set eth_type2		            $::eth_type2
set dest_mac                        $::dest_mac
set clock_step                      $::ptp_clock_step
set sequence_ann                    1
set sequence_sync                   16385
set sequence_foll                   16385
set sequence_del                    32768
set sequence_pdel                   49151
set unicast_flag                    1
set src_port_general                "320"
set dst_port_general                "320"
set src_port_event                  "319"
set dst_port_event                  "319"
set dut_ip                          $::dut_test_port_ip
set tee_ip                          $::tee_test_port_ip
set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval + 5))) +$::ptp_user_delay]
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set arp_timeout                    $::ARP_timeout


if { ( $::ptp_communication_model != "Unicast" ) || ( $clock_step != "Two Step" ) } {



	TC_ABORT -reason "$tee_log_msg(UC_TWO_STEP)"
                           -tc_def $tc_def
	return 0 
	}
########################### END - INITIALIZATION ##############################

ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_001

}

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock mode as Two-Step.                                  #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list          $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
                                   -tc_def $tc_def
                return 0
        }


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add ports T1 and T2 at TEE.                                        #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }

set filter_id [lindex $id1 end]

if {![pltLib::clear_rx_info \
                -session_id              $session_id\
                -port_num                $tee_port_1]} {
                return 0
        }

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

         ERRLOG -msg "$tee_log_msg(ARP_F1)"
   
        TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}


set filter_id [lindex $id1 end]
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC = UC1                                      #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
###############################################################################

sleep $::wait_time
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_ip\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count             $::reset_count_on\
                           -message_type            $::msg_announce\
                           -domain_number           $::default_domain\
			   -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
			   -ucast_flag              ucast_flag\
                           -priority1               priority1\
                           -dut_mac                 dut_mac]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }


		LOG -level 0 -msg   "$tee_log_msg(TAB)"
		LOG -level 0 -msg   "$tee_log_msg(ANN_RX_PRIOR1) $priority1"
set gmpriority1 [ expr $priority1 + 1 ]

###############################################################################
# Step 4 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC = UC2                                      #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z+1                                  #
###############################################################################

STEP 4 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_PERIOD_T1)"

	if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
			   -dest_mac                $dut_mac\
			   -gmpriority1             $gmpriority1\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_ip\
		  	   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $::msg_announce\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmid                    $::gmid]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def

		return 0
	}

###############################################################################
# Step 5 : Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC = UC1                                      #
#              PTP Header                                                     #
#                  Message Type    = 0x00                                     #
#                  Domain Number   = DN1                                      #
#                  Sequence Id     = X                                        #
###############################################################################

STEP 5 
LOG -level 0 -msg   "$tee_log_msg(SYN_RX_OBS)"

if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip		    $dut_ip\
			   -dest_ip		    $tee_ip\
			   -src_port                $src_port_event\
			   -dest_port               $dst_port_event\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
			   -seq_id                  seq_id\
			   -dmac                    dmac1\
                           -timeout                 $sync_timeout\
			   -unicast_flag            $unicast_flag\
                           -filter_id               $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

###############################################################################
# Step 6 : Verify that DUT transmits FOLLOW_UP message to the same unicast    #
#          destination MAC  as the associated SYNC message on the port P1     #
#          with following parameters.                                         #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC = UC1                                      #
#              PTP Header                                                     #
#                  Message Type    = 0x08                                     #
#                  Domain Number   = DN1                                      #
#                  Sequence Id     = X                                        #
###############################################################################

STEP 6  
LOG -level 0 -msg "Verify that DUT transmits FOLLOW_UP message to the same unicast\
                  destination MAC  as the associated SYNC message on the port P1"

if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip		    $dut_ip\
			   -dest_ip		    $tee_ip\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_followup\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
			   -unicast_flag            $unicast_flag\
			   -sequence_id             $seq_id\
			   -dmac                    dmac2\
                           -filter_id               $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }


if { $dmac1 == $dmac2 } {
           TEE_CLEANUP
	TC_CLEAN_AND_PASS    -reason " DUT transmits FOLLOW_UP message to the same unicast destination MAC\
                                       as the associated SYNC message on the port P1"\
			     -tc_def $tc_def
	return 1
} else {
           TEE_CLEANUP
	TC_CLEAN_AND_FAIL    -reason "DUT does not transmit FOLLOW_UP message to the same unicast destination MAC\
                                       as the associated SYNC message on the port P1 "\
			     -tc_def $tc_def
        return 0
}
