#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mtg_013                                  #
# Test Case Version : 1.2                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Transaction Group (MTG)                     #
#                                                                             #
# Title             : SequenceId field, SourcePortIdentity and Domain number  #
#                     of PDelay_Req and Pdelay_Resp message                   #
#                                                                             #
# Purpose           : Verify that the sequenceId field,                       #
#                     requestingSourcePortIdentity and domain number of       #
#                     Pdelay_Resp message shall match the sequenceId,         #
#                     SourcePortIdentity and domain number field of the       #
#                     associated Pdelay_Req message.                          #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 11.4.3 Page 114, Page 115         #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 002                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                 <Mechanism - Peer Delay on Port P1> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                  DN = DN1,PRT1 = Z] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=Z+1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                              SYNC [MSG_TYPE = 0x00, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                          <If Clock Step = Two Step, |           #
#           |                             then receive FOLLOW_UP> |           #
#           |                                                     |           #
#           |                         FOLLOW_UP [MSG_TYPE = 0x08, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | PDELAY_REQ [MSG_TYPE = 0x02,                        |           #
#           |        DN = DN1,SRC_PORT_IDEN = E, SEQ_ID = D]      |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                       PDELAY_RESP [MSG_TYPE = 0x03, |           #
#           |            DN = DN1, SEQ_ID = D, REQ_PORT_IDEN = E] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE      = Message Type                                            #
#     DN            = Domain Number                                           #
#     SEQ_ID        = Sequence ID                                             #
#     PRT1          = Priority1                                               #
#     SRC_PORT_IDEN = Source Port Identity                                    #
#     REQ_PORT_IDEN = Requesting Port Identity                                #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#     v.   Configure mechanism as peer delay mechanism on port P1.            #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z                                          #
#                                                                             #
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z+1                                        #
#                                                                             #
# Step 5a: Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 5b:If the clock is two step clock, observe that DUT transmits FOLLOW_UP#
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 6 : Send PDELAY_REQ message on port T1 with following parameters.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type         = 0x02                                #
#                  Domain Number        = DN1                                 #
#                  Sequence Id          = D                                   #
#                  Source Port Identity = E                                   #
#                                                                             #
# Step 7 : Verify that DUT transmits PDELAY_RESP with domain number, sequence #
#          id and requesting port identity that matches the domain number,    #
#          sequence id and sourcePortIdentity respectively of the associated  #
#          PDELAY_REQ message on the port P1 and with following parameters:   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
#                  Sequence Id              = D                               #
#                  Requesting Port Identity = E                               #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx        Initial                             #
# 1.1          Jun/2017      Veryx        The Unicast pdelay destination      #
#                                         IP is updated(CR ID - 2427)         #
# 1.2          Nov/2018      Veryx        Restricted the number of Delay Req  #
#                                         messages sent in Step 6 to 1        #
#                                         (CR: 18275).                        #
###############################################################################
# Copyright (C) 2017-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
#######################  LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      " Verify that the sequenceId field,requestingSourcePortIdentity\
                  and domain number of Pdelay_Resp message shall match the\
		  sequenceId,SourcePortIdentity and domain number field of the\
		  associated Pdelay_Req message."


########################### START - INITIALIZATION ############################
set tee_port_1		 $::tee_port_num_1
set tee_session_id 	 $::session_id
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set clock_step           $::ptp_clock_step
set pdelay_mac           $::ptp_delay_mac
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set sequence_ann         1
set sequence_sync        16385
set sequence_foll        16385
set sequence_pdelreq     49151
set unicast_flag         0
set src_port_general     "320"
set dst_port_general     "320"
set src_port_event       "319"
set dst_port_event       "319"
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout        $::ptp_timeout
set timeout            [expr $ptp_timeout + $user_delay]
set sync_timeout       [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout      [expr  int(pow(2,( $::ptp_sync_interval + 5))) +$::ptp_user_delay] 
set pdelay_timeout     [expr int(pow(2,( $::ptp_pdelay_req_interval))) + $user_delay ]
set arp_timeout        $::ARP_timeout
set dut_ip             $::dut_test_port_ip
set tee_ip             $::tee_test_port_ip

set tee_pdest_mac                   $::ptp_delay_mac
set dut_pdest_mac                   $::ptp_delay_mac
set dut_dest_ip        $::dut_test_port_ip
set tee_dest_ip        $::tee_test_port_ip

set tee_dest_mac       $::dest_mac
set dut_dest_mac       $::dest_mac
set times              1 
########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_002
}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#     v.   Configure mechanism as peer delay mechanism on port P1.            #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP2)"


    if {![ptp::dut_configure_setup_002 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP2_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP2_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################
STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

###############################################################################
#                                                                             #
#	PRE - REQUISITE CONFIGURATION & ASSIGNING VARIABLES                   #
#                                                                             #
###############################################################################

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

         ERRLOG -msg "$tee_log_msg(ARP_F1)"

        TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_pdest_mac  $tee_mac
set dut_pdest_mac  $dut_mac
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set tee_pdest_ip   $::tee_test_port_ip
set dut_pdest_ip   $::dut_test_port_ip
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
set tee_pdest_ip $::multicast_pdest_ip
set dut_pdest_ip $::multicast_pdest_ip
set unicast_flag   0

}


pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

sleep $::wait_time
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z                                          #
###############################################################################
### (Part1) ###
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"
### Creating filter - Receive packet ###
 if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -priority1		    priority1\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
           TEE_CLEANUP
	   TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def
       return 0
       }

		LOG -level 0 -msg   "$tee_log_msg(TAB)"
                LOG -level 0 -msg   "$tee_log_msg(ANN_RX_PRIOR1) $priority1"
set recv_priority1 [expr $priority1 +1 ]

###############################################################################
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z+1                                        #
###############################################################################

STEP 4 
LOG -level 0 -msg   "$tee_log_msg(ANN_TX_PERIOD_T1)"

       if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -message_length          $ann_len\
                           -control_field           $ctrl_announce\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $recv_priority1]} {
        TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                                         -tc_def $tc_def

		return 0
	}

###############################################################################
# Step 5a: Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        
###############################################################################
STEP 5a
LOG -level 0 -msg   "$tee_log_msg(SYN_RX_OBS)"

 if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
	                   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip   	            $tee_dest_ip\
			   -unicast_flag            $unicast_flag\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_sync\
                           -domain_number           $domain_no\
                           -seq_id                  seq_id\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {
# To clear the periodic send / recieve function	
           
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_RX_P1_F)"\
                               -tc_def $tc_def

       return 0
       }

###############################################################################
# Step 5b:If the clock is two step clock, observe that DUT transmits FOLLOW_UP#
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
###############################################################################
if { $clock_step == "Two Step" } {
STEP 5b 
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_P1_OBS)"

      if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac		    $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
	                   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip   	            $tee_dest_ip\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_followup\
                           -domain_number           $domain_no\
                           -timeout                 $sync_timeout\
                           -sequence_id             $seq_id\
			   -reset_count		    $reset_count_off\
                           -filter_id               $filter_id]} {
	
# To clear the periodic send / recieve function	
          
           TEE_CLEANUP
          TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_RX_P1_F)"\
                             -tc_def $tc_def

       return 0
       }
}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 6 : Send PDELAY_REQ message on port T1 with following parameters.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type         = 0x02                                #
#                  Domain Number        = DN1                                 #
#                  Sequence Id          = D                                   #
#                  Source Port Identity = E                                   #
###############################################################################
STEP 6 
LOG -level 0 -msg "$tee_log_msg(PDREQ_TX_T1)"

if {![ptp::send_pdelayreq\
                        -session_id            $::session_id\
                        -port_num              $tee_port_1\
                        -source_mac            $tee_mac\
                        -dest_mac              $dut_pdest_mac\
                        -eth_type              $eth_type1\
                        -vlan_id               $vlan_id\
                        -type                  $eth_type2\
                        -source_ip             $tee_ip\
                        -dest_ip               $dut_pdest_ip\
                        -src_port              $src_port_event\
                        -dest_port             $dst_port_event\
                        -unicast_flag          $unicast_flag\
                        -message_type          $msg_pdelayreq\
                        -domain_number         $domain_no\
                        -sequence_id           $sequence_pdelreq\
                        -frame_count           $times \
		             	-get_src_port_identity  port_id]} {
          
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT -reason "$tee_log_msg(PDREQ_TX_T1_F)"\
                           -tc_def $tc_def
        return 0
       }

set seq_pdelayreq  $sequence_pdelreq 
set sportid_pdelayreq  $port_id 
if { $::clock_Identity_format == "EUI-64 constructed from a EUI-48" } {
set clockid_pdelayreq [clock_identity $tee_mac]
} else {
set clockid_pdelayreq [clock_identity_noneui $tee_mac]
}
LOG -level 0 -msg   "Send pdelay request message with source port identity : $sportid_pdelayreq"
LOG -level 0 -msg   "Send pdelay request message with clock identity : $clockid_pdelayreq"
            

###############################################################################
# Step 7 : Verify that DUT transmits PDELAY_RESP with domain number, sequence #
#          id and requesting port identity that matches the domain number,    #
#          sequence id and sourcePortIdentity respectively of the associated  #
#          PDELAY_REQ message on the port P1 and with following parameters:   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
#                  Sequence Id              = D                               #
#                  Requesting Port Identity = E                               #
#                                                                             #
###############################################################################
STEP 7 
LOG -level 0 -msg " Verify that DUT transmits PDELAY_RESP with domain number,\
                    sequence id and requesting port identity that matches the\
                    domain number, sequence id and sourcePortIdentity respectively\
                    of the associated PDELAY_REQ message on the port P1"

            if {![ptp::recv_pdelayresp\
                   -session_id          $::session_id\
                   -port_num            $tee_port_1\
                   -dest_mac            $dut_pdest_mac\
                   -eth_type            $eth_type1\
                   -vlan_id             $vlan_id\
                   -type                $eth_type2\
                   -source_ip           $dut_ip\
                   -dest_ip             $tee_pdest_ip\
                   -src_port            $src_port_event\
                   -dest_port           $dst_port_event\
                   -unicast_flag        $unicast_flag\
                   -message_type        $msg_pdelayresp\
                   -domain_number       $domain_no\
                   -sequence_id         $seq_pdelayreq\
		   -req_spid            req_spid\
                   -req_clock_id        req_clock_id\
                   -filter_id           $filter_id\
                   -timeout             $timeout]} {

         TEE_CLEANUP
         TC_CLEAN_AND_FAIL -reason "$tee_log_msg(PDRESP_RX_P1_F)"\
                           -tc_def $tc_def

       return 0
       }
set req_sportid_pdelayresp  $req_spid
set req_clockid_pdelayresp  $req_clock_id  
LOG -level 0 -msg   "DUT transmits pdelay response message with source port id value : $req_sportid_pdelayresp"

if { ($sportid_pdelayreq == $req_sportid_pdelayresp) && [string match -nocase $clockid_pdelayreq $req_clockid_pdelayresp] } {
          TEE_CLEANUP
          TC_CLEAN_AND_PASS -reason "The domain no, sequence id and req source port id of PDelay_Resp\
                                     message matches with domain no, sequence id and source port id\
                                     of PDelay_Req message respectively"\
                            -tc_def $tc_def
 
              
     return 1
} elseif { ($sportid_pdelayreq != $req_sportid_pdelayresp) && [string match -nocase $clockid_pdelayreq $req_clockid_pdelayresp] } {

          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "The req source port number of pdelay response message does not match\
                                     with source port number of pdelay request"\
                            -tc_def $tc_def
return 0

} elseif { ($sportid_pdelayreq == $req_sportid_pdelayresp) && ![string match -nocase $clockid_pdelayreq $req_clockid_pdelayresp] } {

          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "The req clock id of pdelay response message does not match\
                                     with clock id of pdelay request"\
                            -tc_def $tc_def
return 0
} else {
 
          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "The domain no, sequence id and req source port id of PDelay_Resp\
                                     message does not match with domain no, sequence id and source port id\
                                     of PDelay_Req message respectively"\
                            -tc_def $tc_def
return 0
}
