#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_mtg_016                                  #
# Test Case Version : 1.2                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Message Transaction Group (MTG)                     #
#                                                                             #
# Title             : Pdelay_Req Message Transmission Interval                #
#                                                                             #
# Purpose           : Verify that the boundary clock transmits Pdelay_Req     #
#                     messages for every pdelay_request interval.             #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 7.7.2.5 Page 62, 9.5.13.2 Page 106#                                    
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 002                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                           <Configure Priority1 = X> |           #
#           |                                        <Enable PTP> | P1        #
#           |                            <Mechanism - Peer Delay> | P1        #
#           |                                                     |           #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=X-1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |<Send Sync/ Sync & Follow-up msgs                    |           #
#           |based on clock step>                                 |           #
#           |                                                     |           #
#           |              PDELAY_REQ [MSG_TYPE = 0x02, DN = DN1, |           #
#           |                         SEQ_ID = Y, Timestamp = Ta] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |              PDELAY_REQ [MSG_TYPE = 0x02, DN = DN1, |           #
#           |                       SEQ_ID = Y+1, Timestamp = Tb] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |              PDELAY_REQ [MSG_TYPE = 0x02, DN = DN1, |           #
#           |                       SEQ_ID = Y+2, Timestamp = Tc] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |             <Check ((Tb - Ta) + (Tc - Tb))/2 = PI > |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     PRT1      = Priority1                                                   #
#     SEQ_ID    = Sequence ID                                                 #
#     PI        = Pdelay Request Interval                                     #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure Priority1 as X (Default : 128).                          #
#     v.   Enable PTP on port P1.                                             #
#    vi.   Configure mechanism as peer delay mechanism on port P1.            #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Send periodic ANNOUNCE message on port T1 with following           #
#          parameters:                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = X-1                                        #
#                                                                             #
# Step 4a: Send periodic SYNC message on port T1 with following parameters:   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 4b: If the clock is two step clock, send periodic FOLLOW_UP message    #
#          on port T1 with following parameters:                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 5 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters and capture the timestamp(Ta) at which the    #
#          message was received:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x02                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                                                                             #
# Step 6 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters and capture the timestamp(Tb) at which the    #
#          message was received:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x02                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y+1                                        #
#                                                                             #
# Step 7 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters and capture the timestamp(Tc) at which the    #
#          message was received:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x02                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y+2                                        #
#                                                                             #
#                                                                             #
# Step 8 : Verify that DUT transmits PDELAY_REQ message for every Pdelay      #
#          request interval.(Verify Tb - Ta = PI and Tc - Tb = PI)            #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jun/2017    Veryx           Initial                            #
# 1.2          Nov/2018    Veryx           Updated the wait timeout for       #
#                                          pDelay request (CR: 18275).        #
#                                          Updated the periodicity calculation#
#                                          (CR: 18275)                        #
###############################################################################
# Copyright (C) 2017-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id

####################### LOCAL VARIABLES ARE DECLARED HERE #####################
set Time1 [expr (pow(2,$::ptp_pdelay_req_interval))]

set low_Time1  [expr $Time1 - ($Time1 * (30.0/100))]
set high_Time1 [expr $Time1 + ($Time1 * (30.0/100))]


set tc_def     "Verify that the boundary clock transmits Pdelay_Req\
                messages for every pdelay_request interval."
########################### START - INITIALIZATION ############################
set tee_port_1                     $::tee_port_num_1
set tee_session_id                 $::session_id
set dut_port_1                     $::dut_port_num_1
set vlan_id                        $::ptp_vlan_id
set eth_type1                      $::eth_type1
set eth_type2                      $::eth_type2
set dest_mac                       $::dest_mac
set ann_interval                   $::ptp_announce_interval
set clock_step                     $::ptp_clock_step
set sequence_ann                   1
set sequence_sync                  16385
set sequence_foll                  16385
set src_port_general               "320"
set dst_port_general               "320"
set src_port_event                 "319"
set dst_port_event                 "319"
set unicast_flag                   0
set domain_no                      $::default_domain
set dut_ip                         $::dut_test_port_ip
set tee_ip                         $::tee_test_port_ip
set dut_dest_ip                    $::dut_test_port_ip
set tee_dest_ip                    $::tee_test_port_ip
set tee_dest_mac                   $::dest_mac
set dut_dest_mac                   $::dest_mac
set tee_pdest_mac                  $::ptp_delay_mac
set dut_pdest_mac                  $::ptp_delay_mac
set user_delay                     $::ptp_user_delay
set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval + 5))) +$::ptp_user_delay]
set pdelay_timeout                 [expr (2 * int(pow(2,($::ptp_pdelay_req_interval)))) + $user_delay] 
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set arp_timeout                    $::ARP_timeout

ptp::dut_Initialization
########################### END - INITIALIZATION ##############################

proc cleanup { } {

ptp::dut_cleanup_setup_002

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure Priority1 as X (Default : 128).                          #
#     v.   Enable PTP on port P1.                                             #
#    vi.   Configure mechanism as peer delay mechanism on port P1.            #
###############################################################################
### (Initial Part) ###
STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP2)"


    if {![ptp::dut_configure_setup_002 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP2_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP2_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1
### (Part1) ###

### Creating filter - Receive packet ###
if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $::ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }

set filter_id [lindex $id1 end]

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

         ERRLOG -msg "$tee_log_msg(ARP_F1)"

        TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_pdest_mac  $tee_mac
set dut_pdest_mac  $dut_mac
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set tee_pdest_ip   $::tee_test_port_ip
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
set tee_pdest_ip $::multicast_pdest_ip

}


pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
set gm_priority1 [expr $::ptp_dut_clock_priority1 - 1 ]
sleep $::wait_time
###############################################################################
# Step 3 : Send periodic ANNOUNCE message on port T1 with following           #
#          parameters:                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = X-1                                        #
###############################################################################
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"
if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $::msg_announce\
			   -domain_number           $::default_domain\
                           -message_length          $::ann_len\
                           -gmid                    $::gmid\
                           -control_field           $::ctrl_announce\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $gm_priority1]} {
	            TEE_CLEANUP	
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ANN_TX_T1_F)"\
                                         -tc_def $tc_def

		return 0
	}
if { $clock_step == "One Step" } {
pltLib::reset_all_stats\
        -session_id         $::session_id\
        -port_num           $tee_port_1
}
###############################################################################
# Step 4a: Send periodic SYNC message on port T1 with following parameters:   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
###############################################################################
	
STEP 4a 
LOG -level 0 -msg "$tee_log_msg(SYN_TX_PERIOD_T1)"

if {![ptp::send_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
		  	   -unicast_flag            $unicast_flag\
                           -message_type            $::msg_sync\
			   -twostep_flag            $::twostep_flag\
                           -message_length          $::sync_len\
			   -domain_number           $::default_domain\
                           -control_field           $::ctrl_sync\
                           -sequence_id             $sequence_sync]} {

           TEE_CLEANUP 
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_TX_T1_F)"\
                              -tc_def $tc_def				   

		return 0
	}

###############################################################################
# Step 4b: If the clock is two step clock, send periodic FOLLOW_UP message    #
#          on port T1 with following parameters:                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
###############################################################################

if { $clock_step == "Two Step" } {
pltLib::reset_all_stats\
        -session_id         $::session_id\
        -port_num           $tee_port_1
STEP 4b 
LOG -level 0 -msg "$tee_log_msg(FOLL_TX_PERIOD_T1)"

        if {![ptp::send_followup\
			 -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_dest_mac\
                         -eth_type                $eth_type1\
			 -vlan_id		  $vlan_id\
			 -type 		  	  $eth_type2\
		         -source_ip               $tee_ip\
		         -dest_ip	          $dut_dest_ip\
			 -src_port		  $src_port_general\
			 -dest_port		  $dst_port_general\
		  	 -unicast_flag            $unicast_flag\
                         -message_type            $::msg_followup\
                         -message_length          $::followup_len\
			 -domain_number           $::default_domain\
                         -control_field           $::ctrl_followup\
                         -sequence_id             $sequence_sync]} {

           TEE_CLEANUP 
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_TX_T1_F)"\
                              -tc_def $tc_def				 

		return 0 
	
		}
}
###############################################################################		
# Step 5 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters and capture the timestamp(Ta) at which the    #
#          message was received:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x02                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
###############################################################################

STEP 5
LOG -level 0 -msg "$tee_log_msg(PDREQ_RX_OBS)"

if {![ptp::recv_pdelayreq \
                           -session_id                 $::session_id\
                           -port_num                   $tee_port_1\
                           -dest_mac                   $tee_pdest_mac\
                           -eth_type   	               $eth_type1\
		             	   -vlan_id                    $vlan_id\
			               -type                       $eth_type2\
		             	   -source_ip                  $dut_ip\
	            		   -dest_ip   	               $tee_pdest_ip\
		            	   -src_port		           $src_port_event\
		            	   -dest_port                  $dst_port_event\
		            	   -reset_count		           $::reset_count_off\
                           -message_type               $::msg_pdelayreq\
			               -domain_number              $domain_no\
			               -timeout                    $pdelay_timeout\
                           -rx_timestamp_ns            Ta\
			               -seq_id             	       seq1_id\
                           -filter_id                  $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(PDREQ_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

LOG -level 0 -msg "$tee_log_msg(PDREQ_RX_P1_SEQ) $seq1_id"
set seq2_id [expr $seq1_id + 1]

set Ta  [format %.2f [expr $Ta/1000000000.0]]

LOG -level 0 -msg "pDelay Request message from DUT is received at (Ta):  $Ta secs\n"

###############################################################################		
# Step 6 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters and capture the timestamp(Tb) at which the    #
#          message was received:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x02                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y+1                                        #
###############################################################################

STEP 6
LOG -level 0 -msg "$tee_log_msg(PDREQ_RX_OBS)"

if {![ptp::recv_pdelayreq \
                           -session_id                 $::session_id\
                           -port_num                   $tee_port_1\
                           -dest_mac                   $tee_pdest_mac\
                           -eth_type   	               $eth_type1\
			   -vlan_id                    $vlan_id\
			   -type                       $eth_type2\
			   -source_ip                  $dut_ip\
			   -dest_ip   	               $tee_pdest_ip\
			   -src_port		       $src_port_event\
			   -dest_port                  $dst_port_event\
			   -reset_count		       $::reset_count_off\
                           -message_type               $::msg_pdelayreq\
			   -domain_number              $domain_no\
			   -timeout                    $pdelay_timeout\
                           -rx_timestamp_ns            Tb\
			   -sequence_id        	       $seq2_id\
                           -filter_id                  $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(PDREQ_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

set seq3_id [expr $seq1_id + 2]
set Tb      [format %.2f [expr $Tb/1000000000.0]]

LOG -level 0 -msg "pDelay Request message from DUT is received at (Tb):  $Tb secs\n"

###############################################################################
# Step 7 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters and capture the timestamp(Tc) at which the    #
#          message was received:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x02                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y+2                                        #
###############################################################################
STEP 7
LOG -level 0 -msg "$tee_log_msg(PDREQ_RX_OBS)"

if {![ptp::recv_pdelayreq \
                           -session_id                 $::session_id\
                           -port_num                   $tee_port_1\
                           -dest_mac                   $tee_pdest_mac\
                           -eth_type   	               $eth_type1\
			   -vlan_id                    $vlan_id\
			   -type                       $eth_type2\
			   -source_ip                  $dut_ip\
			   -dest_ip   	               $tee_pdest_ip\
			   -src_port		       $src_port_event\
			   -dest_port                  $dst_port_event\
			   -reset_count		       $::reset_count_off\
                           -message_type               $::msg_pdelayreq\
			   -domain_number              $domain_no\
			   -timeout                    $pdelay_timeout\
                           -rx_timestamp_ns            Tc\
			   -sequence_id        	       $seq3_id\
                           -filter_id                  $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(PDREQ_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

set Tc  [format %.2f [expr $Tc/1000000000.0]]
LOG -level 0 -msg "pDelay Request message from DUT is received at (Tc):  $Tc secs\n"

set t1 [format %.2f [expr $Tb - $Ta]]
set t2 [format %.2f [expr $Tc - $Tb]]
###############################################################################
# Step 8 : Verify that DUT transmits PDELAY_REQ message for every Pdelay      #
#          request interval.(Verify Tb - Ta = PI and Tc - Tb = PI)            #
###############################################################################
STEP 8
LOG -level 0 -msg "Verify that DUT transmits PDELAY_REQ  message for every Pdelay request interval\n"

	LOG -level 0 -msg "DUT sends first and second PDELAY_REQ message at an interval of $t1 secs\n"
	LOG -level 0 -msg "DUT sends second and third PDELAY_REQ message at an interval of $t2 secs\n"

        set periodicity [ expr (($t1 + $t2)/2)]

	LOG -level 0 -msg "Expected range: $low_Time1 - $high_Time1 secs, Observed average value: $periodicity secs\n"
	if {($periodicity >= $low_Time1 && $periodicity <= $high_Time1)} {
	 	LOG -level 0 -msg "DUT sends PdelayReq messages with average periodicity within +/- 30% of the configured Pdelay request Interval\n"
		set interval_result 1
	} else {
		LOG -level 0 -msg "DUT does not send PdelayReq messages with average periodicity within +/- 30% of the configured Pdelay request Interval\n"
		set interval_result 0
	}

if { $interval_result } {
           TEE_CLEANUP
           TC_CLEAN_AND_PASS -reason "DUT transmits PdelayReq message for every Pdelay request interval"\
                              -tc_def $tc_def

           return 1
   } else {
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "DUT does not transmit PdelayReq message for every Pdelay request interval"\
                              -tc_def $tc_def

           return 0
	
}

