#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_pcg_001                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Configuration Group (PCG)                           #
#                                                                             #
# Title             : Configuration of PTP on multiple ports                  #
#                                                                             #
# Purpose           : Verify that the boundary clock allows to configure PTP  #
#                     on multiple ports.                                      #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 6.5.3 Para 1 Page 20              #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 003                                                     #
# Test Topology     : 2                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                   <Clock mode - one step/ two-step> |           #
#           |                                                     |           #
#           |                      <Enable PTP on Port P1 and P2> | P1,P2     #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                               DN = DN1, SEQ_ID = X] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                               DN = DN1, SEQ_ID = Y] |           #
#        T2 |-------------------------------------------------<<--| P2        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#     This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT ports P1 and P2 is operational.                    #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Enable PTP on Port P1 and P2.                                      #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add ports T1 and T2 at TEE.                                        #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  Sequence ID      = X                                       #
#                                                                             #
# Step 4 : Verify that DUT transmits ANNOUNCE message on the port P2 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  Sequence ID      = Y                                       #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx            Initial                         #
# 1.1          Oct/2018      Veryx            The value of the DUT IP Address #
#                                             on interface I2 was corrected.  #
#                                             (CR: 18256).                    #
###############################################################################
# Copyright (C) 2013-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id

####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock allows to configure PTP\
                 on multiple ports."

#ptp::Initialization
########################### START - INITIALIZATION ############################

set tee_port_1           $::tee_port_num_1
set tee_port_2           $::tee_port_num_2
set tee_session_id       $::session_id
set dut_port             $::dut_port_list_2
set dut_port_1           $::dut_port_num_1
set vlan_id              $::ptp_vlan_id
set eth_type1            $::eth_type1
set eth_type2            $::eth_type2
set dest_mac             $::dest_mac
set clock_step           $::ptp_clock_step
set pdelay_mac           $::ptp_delay_mac
set sequence_ann         1
set sequence_sync        16385
set sequence_foll        16385
set sequence_del         49151
set unicast_flag         0
set src_port_general     "320"
set dst_port_general     "320"
set src_port_event       "319"
set dst_port_event       "319"
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set dut_ip               $::dut_test_port_ip
set tee_ip               $::tee_test_port_ip

set dut_dest_ip          $::dut_test_port_ip
set tee_dest_ip          $::tee_test_port_ip
set dut_test_port_ip2    $env(dut_TEST_ADDRESS2)
set dut_address          $env(TEST_DEVICE_IPV4_ADDRESS) 
set tee_dest_mac         $::dest_mac
set dut_dest_mac         $::dest_mac
set dut_dest_mac2        $::dest_mac
set tee_dest_mac2        $::dest_mac
set user_delay           $::ptp_user_delay
set announce_timeout     $::ptp_announce_timeout
set arp_timeout          $::ARP_timeout
set announce_interval    $::ptp_announce_interval
set announce_interval    [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout          $::ptp_timeout
set timeout              [expr $ptp_timeout + $user_delay]
set sync_timeout         [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout        [expr  int(pow(2,( $::ptp_sync_interval +1))) +$::ptp_user_delay] 
set pdelay_timeout       [expr int(pow(2,($::ptp_pdelay_req_interval)))+ $user_delay]


########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_003

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT ports P1 and P2 is operational.                    #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Enable PTP on Port P1 and P2.                                      #
###############################################################################

STEP 1
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP3)"


    if {![ptp::dut_configure_setup_003 \
                       -port_list      $dut_port]} {

            ERRLOG -msg "$dut_log_msg(INIT_SETUP3_F)"

            TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP3_F)"\
                           -tc_def $tc_def
            return 0
    }


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add ports T1 and T2 at TEE.                                        #
###############################################################################
STEP 2 "$tee_log_msg(INIT)"
ptp::Initialization

if {![ptp::tee_configure_setup_002 \
                -port_list        $tee_port_2]} {

      ERRLOG -msg "$::tee_log_msg(INIT_SETUP2_F)"

      TC_ABORT -reason "$::tee_log_msg(INIT_SETUP2_F)"\

     }
ptp::get_port_macaddress\
            -session_id         $session_id\
            -port_num           $tee_port_2\
            -mac_address        TEEMAC_2 

set tee_mac            $::TEEMAC_1
set tee_mac2           $TEEMAC_2

if { $env(DUT_DEVICE_TYPE) == "L2 Switch"} {
     if {![tee_get_lower_address    -ip_address $dut_test_port_ip2 \
                                    -ret_address tee_test_port_ip2] } {
    puts " \n################################################################"
    puts    "#ERROR: Unable to get tee_test_port_ip2 from $dut_test_port_ip2  "
    puts    "################################################################"

    TC_ABORT -reason "Second Test interface IP allocation error"
    return 0
}
     set dut_test_port_ip  $dut_address
} else { 
   if {![tee_get_lower_address    -ip_address $dut_test_port_ip2 \
                                  -ret_address tee_test_port_ip2] } {

    puts " \n################################################################"
    puts    "#ERROR: Unable to get tee_test_port_ip2 from $dut_test_port_ip2  "
    puts    "################################################################"

    TC_ABORT -reason "Second Test interface IP allocation error"
    return 0
}
}
set tee_ip2             $tee_test_port_ip2
set dut_ip2             $dut_test_port_ip2
set dut_dest_ip2        $dut_test_port_ip2
set tee_dest_ip2        $tee_test_port_ip2

if {![pltLib::create_filter_ptp_pkt\
                           -port_num          $tee_port_1\
                           -ether_type        $ptp_ethtype\
                           -vlan_id           $vlan_id\
                           -ids               id1]} {
        return 0
}
if {![pltLib::create_filter_ptp_pkt\
                           -port_num          $tee_port_2\
                           -ether_type        $ptp_ethtype\
                           -vlan_id           $vlan_id\
                           -ids               id2]} {
        return 0
}
set filter_id [lindex $id1 end]
set filter_id1 [lindex $id2 end]
###############################################################################
#                                                                             #
#     PRE - REQUISITE CONFIGURATION & ASSIGNING VARIABLES                     #
#                                                                             #
###############################################################################


if {$::ptp_communication_model == "Unicast"} {
set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
            -session_id       $session_id\
            -port_num         $tee_port_1\
            -tee_mac          $tee_mac\
            -tee_ip           $tee_ip\
            -dut_ip           $dut_ip\
            -ethertype        $eth_type1\
            -vlan_id          $vlan_id\
            -type             $eth_type2\
            -filter_id        $filter_id\
            -dut_mac          dut_mac\
            -timeout          $arp_timeout]

if { $result == 0 } {

      ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                             -tc_def $tc_def

       return 0

}

set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1



set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
            -session_id       $session_id\
            -port_num         $tee_port_2\
            -tee_mac          $tee_mac2\
            -tee_ip           $tee_ip2\
            -dut_ip           $dut_ip2\
            -ethertype        $eth_type1\
            -vlan_id          $vlan_id\
            -type             $eth_type2\
            -filter_id        $filter_id1\
            -dut_mac          dut_mac2\
            -timeout          $arp_timeout]

if { $result == 0 } {
      ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                             -tc_def $tc_def

       return 0

}

set tee_dest_mac2   $tee_mac2
set dut_dest_mac2   $dut_mac2
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
set dut_dest_ip2 $::multicast_dest_ip
set tee_dest_ip2 $::multicast_dest_ip
set dut_ip2      $dut_test_port_ip2
set tee_ip2      $tee_test_port_ip2

}



pltLib::reset_all_stats\
      -session_id     $::session_id\
      -port_num       $tee_port_1

pltLib::reset_all_stats\
      -session_id     $::session_id\
      -port_num       $tee_port_2


###############################################################################
# Step 3 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  Sequence ID      = X                                       #
###############################################################################
 
sleep $::wait_time
STEP 3
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_V)"
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
# To clear the periodic send / receive function 
          TEE_CLEANUP
          TC_CLEAN_AND_FAIL  -reason "$tee_log_msg(ANN_RX_P1_F)"\
                             -tc_def $tc_def

       return 0
       }
###############################################################################
# Step 4 : Verify that DUT transmits ANNOUNCE message on the port P2 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  Sequence ID      = Y                                       #
###############################################################################
 
STEP 4
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P2_V)"
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_2\
                           -dest_mac                $tee_dest_mac2\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip2\
                           -dest_ip                 $tee_dest_ip2\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id1\
                           -dut_mac                 dut_mac]} {

# To clear the periodic send / receive function 
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P2_F)"\
                              -tc_def $tc_def

       return 0
       } else {

# To clear the periodic send / receive function 
           TEE_CLEANUP
           TC_CLEAN_AND_PASS -reason "$tee_log_msg(ANN_RX_P1P2)"\
                              -tc_def $tc_def
return 1
}
