#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_pcg_002                                  #
# Test Case Version : 1.0                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Configuration Group (PCG)                           #
#                                                                             #
# Title             : Configuration support for domain number                 #
#                                                                             #
# Purpose           : Verify that the boundary clock supports to configure    #
#                     domain number to any value in range 0 to 127.           #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 7.1 Para 2 Page 41                #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                   <Clock step - one step/ two-step> |           #
#           |                                                     |           #
#           |                      <Configure Domain Number - X1> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                            DN = X1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                     <Configure Domain Number -  X2> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                            DN = X2] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                     <Configure Domain Number -  X3> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                            DN = X3] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                    !<Configure Domain Number =  N1> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                            DN = N1] |           #
#        T1 |                         XX----------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                    !<Configure Domain Number =  N2> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                            DN = N2] |           #
#        T1 |                         XX----------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Configured Domain Number:                                               #
#        0 <= Xn <= 127                                                       #
#        Nn < 0 or Nn > 255                                                   #
#        Reserved: 128 to 255.                                                #
#                                                                             #
#     This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part I)                                                                    #
#                                                                             #
# Step 3 : Configure Domain Number as X1 on Port P1 in DUT. (X1 = 0)          #
#                                                                             #
# Step 4 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = X1                                      #
#                                                                             #
# Step 5 : Configure Domain Number as X2 on Port P1 in DUT. (X2 = 50)         #
#                                                                             #
# Step 6 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = X2                                      #
#                                                                             #
# Step 7 : Configure Domain Number as X4 on Port P1 in DUT. (X4 = 127)        #
#                                                                             #
# Step 8 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = X4                                      #
#                                                                             #
# Step 9a: Verify that DUT does not allow to configure Domain Number as N1    #
#           on Port P1 in DUT. (N1 = -1)                                      #
#                                                                             #
# Step 9b: Verify that DUT does not transmit ANNOUNCE message on the port P1  #
#           with Domain Number same as the configured Domain Number(N1).      #
#                                                                             #
# Step 10a: Verify that DUT does not allows to configure Domain Number as N2  #
#           on Port P1 in DUT. (N2 = 256)                                     #
#                                                                             #
# Step 10b: Verify that DUT does not transmit ANNOUNCE message on the port P1 #
#           with Domain Number same as the configured Domain Number(N2).      #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx            Initial                         #
###############################################################################
# Copyright (C) 2013 Veryx Technologies(P) Ltd.                               #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id

####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock supports to configure\
                     domain number to any value in range 0 to 127."

#ptp::Initialization
########################### START - INITIALIZATION ############################

set tee_port_1		 $::tee_port_num_1
set tee_session_id 	 $::session_id
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set clock_step           $::ptp_clock_step
set pdelay_mac           $::ptp_delay_mac
set sequence_ann         1
set sequence_sync        16385
set sequence_foll        16385
set sequence_del         49151
set unicast_flag         0
set src_port_general     "320"
set dst_port_general     "320"
set src_port_event       "319"
set dst_port_event       "319"
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set dut_ip             $::dut_test_port_ip
set tee_ip             $::tee_test_port_ip

set dut_dest_ip        $::dut_test_port_ip
set tee_dest_ip        $::tee_test_port_ip

set tee_dest_mac       $::dest_mac
set dut_dest_mac       $::dest_mac
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set arp_timeout        $::ARP_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout       $::ptp_timeout
set timeout           [expr $ptp_timeout + $user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout [expr  int(pow(2,( $::ptp_sync_interval +1))) +$::ptp_user_delay] 
set pdelay_timeout [expr int(pow(2,($::ptp_pdelay_req_interval)))+ $user_delay]
set domain_0 "0"
set domain_50 "50"
set domain_127 "127"
set domain_minus1 "-1"
set domain_256 "256"
set sig_uni "1"
set sig_seqid "22"
set sig_srcprtid "3"
set sig_logmsg "1"
set sig_trgtprtidty "ffffffffffffffff"
set sig_trgprtid "3"
set sig_frame "10"
set sig_tlv "4"
set sig_ann "b0"
set sig_len "6"

########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_001
set result [ptp::dut_set_domain\
                -session_handler  $::dut_session_handler\
                -clock_mode       $::ptp_dut_clock_mode\
                -domain           $::default_domain]

if { $result == 0 } {

           LOG -level 0 -msg "Unable to set Domain number ($::default_domain])"

        }

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

	       ERRLOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################
STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1


if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

###############################################################################
#                                                                             #
#	PRE - REQUISITE CONFIGURATION & ASSIGNING VARIABLES                   #
#                                                                             #
###############################################################################

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}

set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip

}



pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

set session_handler $::dut_session_handler

set clock_mode             $::ptp_dut_clock_mode
	
if { $::ptp_communication_model == "Unicast" } {
pltLib::cancel_send_periodic\
		-instance   9
}
###############################################################################
# Step 3 : Configure Domain Number as X1 on Port P1 in DUT. (X1 = 0)          #
###############################################################################
STEP 3 
LOG -level 0 -msg "$dut_log_msg(DMN_NO_0)"

if {![ptp::dut_set_domain\
                      -session_handler     $session_handler\
                     -clock_mode          $clock_mode\
                     -domain              $domain_0 ]} {
                  TEE_CLEANUP
                  TC_CLEAN_AND_ABORT   -reason "$dut_log_msg(DMN_NO_0_F)"\
                                       -tc_def $tc_def

           return 0
   }

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

if { $::ptp_communication_model == "Unicast" } {
    LOG -level 1 -msg "Signalling message for requesting Announce Message with domain number 0"
        if {![ptp::send_signalling_message\
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $tee_ip\
                           -dest_ip                 $dut_ip\
                           -domain_number           $domain_0\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -unicast_flag            $sig_uni\
                           -sequence_id             $sig_seqid\
                           -src_port_identity       $sig_srcprtid\
                           -logmsg_interval         $sig_logmsg\
                           -targetPortIdentity      $sig_trgtprtidty\
                           -targetPortId            $sig_trgprtid\
                           -frame_interval          $sig_frame\
                           -tlvType                 $sig_tlv\
                           -signal_message_type     $sig_ann\
                           -lengthField             $sig_len]} {

                   return 0
        }
}
###############################################################################
# Step 4 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = X1                                      #
###############################################################################
 
sleep $::wait_time
STEP 4 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN_V)"
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -domain_number           $domain_0\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {

# To clear the periodic send / recieve function	
                  TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                             -tc_def $tc_def
return 0
}
      LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN0)"
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

if { $::ptp_communication_model == "Unicast" } {
pltLib::cancel_send_periodic\
		-instance   9
}
###############################################################################
# Step 5 : Configure Domain Number as X2 on Port P1 in DUT. (X2 = 50)         #
###############################################################################
STEP 5
LOG -level 0 -msg "$dut_log_msg(DMN_NO_50)"

if {![ptp::dut_set_domain\
                      -session_handler     $session_handler\
                     -clock_mode          $clock_mode\
                     -domain              $domain_50 ]} {
                  TEE_CLEANUP
                  TC_CLEAN_AND_ABORT   -reason "$dut_log_msg(DMN_NO_50_F)"\
                                       -tc_def $tc_def

           return 0
   }

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

if { $::ptp_communication_model == "Unicast" } {
    LOG -level 1 -msg "Signalling message for requesting Announce Message with domain number 50"
        if {![ptp::send_signalling_message\
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $tee_ip\
                           -dest_ip                 $dut_ip\
                           -domain_number           $domain_50\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -unicast_flag            $sig_uni\
                           -sequence_id             $sig_seqid\
                           -src_port_identity       $sig_srcprtid\
                           -logmsg_interval         $sig_logmsg\
                           -targetPortIdentity      $sig_trgtprtidty\
                           -targetPortId            $sig_trgprtid\
                           -frame_interval          $sig_frame\
                           -tlvType                 $sig_tlv\
                           -signal_message_type     $sig_ann\
                           -lengthField             $sig_len]} {

                   return 0
        }
}
###############################################################################
# Step 6 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = X2                                      #
###############################################################################
STEP 6 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN_V)"

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -domain_number           $domain_50\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function	
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                             -tc_def $tc_def
return 0
}
      LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN50)"

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1


if { $::ptp_communication_model == "Unicast" } {
pltLib::cancel_send_periodic\
		-instance   9
}
###############################################################################
# Step 7 : Configure Domain Number as X4 on Port P1 in DUT. (X4 = 127)        #
###############################################################################
STEP 7 
LOG -level 0 -msg "$dut_log_msg(DMN_NO_127)"


if {![ptp::dut_set_domain\
                      -session_handler     $session_handler\
                     -clock_mode          $clock_mode\
                     -domain              $domain_127 ]} {

                  TEE_CLEANUP
                  TC_CLEAN_AND_ABORT   -reason "$dut_log_msg(DMN_NO_127_F)"\
                                       -tc_def $tc_def

           return 0
   }

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
if { $::ptp_communication_model == "Unicast" } {
  LOG -level 1 -msg "Signalling message for requesting Announce Message with domain number 127"
        if {![ptp::send_signalling_message\
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $tee_ip\
                           -dest_ip                 $dut_ip\
                           -domain_number           $domain_127\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -unicast_flag            $sig_uni\
                           -sequence_id             $sig_seqid\
                           -src_port_identity       $sig_srcprtid\
                           -logmsg_interval         $sig_logmsg\
                           -targetPortIdentity      $sig_trgtprtidty\
                           -targetPortId            $sig_trgprtid\
                           -frame_interval          $sig_frame\
                           -tlvType                 $sig_tlv\
                           -signal_message_type     $sig_ann\
                           -lengthField             $sig_len]} {

                   return 0
        }
}
###############################################################################
# Step 8 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = X4                                      #
###############################################################################

STEP 8
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN_V)"
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -domain_number           $domain_127\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id]} {
           
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                             -tc_def $tc_def
return 0
}
      LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN127)"

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1


###############################################################################
# Step 9a: Verify that DUT does not allows to configure Domain Number as N1  #
#           on Port P1 in DUT. (N1 = -1)                                      #
###############################################################################
   
STEP 9a
LOG -level 0 -msg "$dut_log_msg(DMN_NO_MINUS1_V)"
 
if {![ptp::dut_set_domain\
                      -session_handler     $session_handler\
                     -clock_mode          $clock_mode\
                     -domain              $domain_minus1 ]} {

                  LOG -level 0 -msg "$dut_log_msg(DMN_NO_MINUS1_F)"

   } else {
	
          
                  LOG -level 0 -msg "$dut_log_msg(DMN_NO_MINUS1_S)"
                              

}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 9b: Verify that DUT does not transmit ANNOUNCE message on the port P1 #
#           with Domain Number same as the configured Domain Number(N1).      #
###############################################################################

STEP 9b
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN_MINUS1_V)"
if {![ptp::recv_announce\
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -domain_number           $domain_127\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id]} {
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                             -tc_def $tc_def
return 0
}
           LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN_S)"
               
###############################################################################
# Step 10a: Verify that DUT does not allows to configure Domain Number as N2  #
#           on Port P1 in DUT. (N2 = 256)                                     #
###############################################################################

STEP 10a
LOG -level 0 -msg "$dut_log_msg(DMN_NO_256_V)"

if {![ptp::dut_set_domain\
                      -session_handler    $session_handler\
                     -clock_mode          $clock_mode\
                     -domain              $domain_256 ]} {

                   LOG -level 0 -msg "$dut_log_msg(DMN_NO_256_F)"

   } else {
	           LOG -level 0 -msg "$dut_log_msg(DMN_NO_256_S)"
                              
}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

     
###############################################################################
# Step 10b: Verify that DUT does not transmit ANNOUNCE message on the port P1 #
#           with Domain Number same as the configured Domain Number(N2).      #
#                                                                             #
###############################################################################
 
  
STEP 10b
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN_256_V)" 

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -domain_number           $domain_127\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"
                             -tc_def $tc_def
return 0
} 
 
           LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_DMN_S)"
 
           TEE_CLEANUP
           TC_CLEAN_AND_PASS -reason "$tee_log_msg(ANN_RX_P1_DMN_S)"\
                             -tc_def $tc_def
           return 1                      
               

