#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_pcg_004                                  #
# Test Case Version : 1.2                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Configuration Group (PCG)                           #
#                                                                             #
# Title             : unicastFlag TRUE - Peer Delay Mechanism                 #
#                                                                             #
# Purpose           : Verify that the boundary clock configured for unicast   #
#                     transmission has the unicastFlag field as TRUE in       #
#                     Peer Delay messages.                                    #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 7.3.8.1 Page 46                   #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 002                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                 <Mechanism - Peer Delay on Port P1> |           #
#           |                       <Transmission Type - Unicast> |           #
#           |                                                     |           #
#           |          ANNOUNCE [DEST_MAC = UC1, MSG_TYPE = 0x0B, |           #
#           |                      DN = DN1, PRT1=Z, UN_FLAG = 1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [DEST_MAC = UC2, MSG_TYPE = 0x0B,          |           #
#           |           DN = DN_1,PRT1=Z+1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |              SYNC [DEST_MAC = UC1, MSG_TYPE = 0x00, |           #
#           |                              DN = DN1, UN_FLAG = 1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                          <If Clock Step = Two Step, |           #
#           |                             then receive FOLLOW_UP> |           #
#           |                                                     |           #
#           |         FOLLOW_UP [DEST_MAC = UC1, MSG_TYPE = 0x08, |           #
#           |                              DN = DN1, UN_FLAG = 1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                        PDELAY_REQ [MSG_TYPE = 0x02, |           #
#           |                              DN = DN1, UN_FLAG = 1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | PDELAY_REQ [DEST_MAC = UC2, MSG_TYPE = 0x02,        |           #
#           |              DN = DN1]                              |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                        PDELAY_RESP [DEST_MAC = UC1, |           #
#           |             MSG_TYPE = 0x03, DN = DN1, UC_FLAG = 1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                          <If Clock Step = Two Step, |           #
#           |                 then receive PDELAY_RESP_FOLLOW_UP> |           #
#           |                                                     |           #
#           |              PDELAY_RESP_FOLLOW_UP [DEST_MAC = UC1, |           #
#           |             MSG_TYPE = 0x0A, DN = DN1, UC_FLAG = 1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     UC_FLAG   = UnicastFlag                                                 #
#     PRT1      = Priority1                                                   #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Transmission Type as Unicast.                            #
#     v.   Configure PTP on port P1.                                          #
#    vi.   Configure mechanism as peer delay mechanism on port P1.            #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                  UnicastFlag         = 1                                    #
#                                                                             #
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z+1                                  #
#                                                                             #
#                                                                             #
# Step 5a: Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
#                                                                             #
# Step 5b:If clock is two step clock, observe that DUT transmits FOLLOW_UP    #
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
#                                                                             #
# Step 6 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters :                                             #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type              = 0x02                           #
#                  Domain Number             = DN1                            #
#                  UnicastFlag               = 1                              #
#                                                                             #
# Step 7 : Send PDELAY_REQ message on port T1 with following parameters.      #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x02                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 8a: Verify that DUT transmits PDELAY_RESP message on the port P1 with  #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x03                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
#                                                                             #
# Step 8b: If the clock is two step clock, Verify that DUT transmits          #
#          PDELAY_RESP_FOLLOW_UP message on port P1 with following parameters.#
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0A                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx            Initial                         #
# 1.1          Jun/2017      Veryx            1.The Unicast pdelay destination#
#                                               IP is updated(CR ID - 2427).  #
#                                             2.ptp_ethtype updated           #
#                                               (CR ID - 2428).               #
# 1.2          Nov/2018      Veryx            Updated the wait timeout for    #
#                                             pDelay request (CR: 18275).     #
###############################################################################
# Copyright (C) 2017-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
#######################  LOCAL VARIABLES ARE DECLARED HERE #####################
set tc_def      "Verify that the boundary clock configured as unicast\
                 transmission has the unicastFlag field as TRUE in all\
                 Peer Delay messages."

if { $::ptp_communication_model != "Unicast" } {

	LOG -level 0 -msg   "$tee_log_msg(UC)"
         TC_ABORT   -reason "$tee_log_msg(UC)"\
                           -tc_def $tc_def
	return 0 
	}


########################### START - INITIALIZATION ############################

set tee_port_1		            $::tee_port_num_1
set tee_session_id 	            $::session_id
set dut_port_1		            $::dut_port_num_1
set vlan_id		            $::ptp_vlan_id
set eth_type1		            $::eth_type1
set eth_type2		            $::eth_type2
set dest_mac                        $::dest_mac
set ann_interval                    $::ptp_announce_interval
set clock_step                      $::ptp_clock_step
set pdelay_mac                      $::ptp_delay_mac
set sequence_ann                    1
set correction_field                0
set sequence_sync                   16385
set sequence_del                    32768
set sequence_pdel                   49151
set src_port_general                "320"
set dst_port_general                "320"
set src_port_event                  "319"
set dst_port_event                  "319"
set unicast_flag                    1
set tee_pdest_mac                   $::ptp_delay_mac
set dut_pdest_mac                   $::ptp_delay_mac

set dut_ip                          $::dut_test_port_ip
set tee_ip                          $::tee_test_port_ip

set dut_dest_ip                     $::dut_test_port_ip
set tee_dest_ip                     $::tee_test_port_ip

set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval +5))) +$::ptp_user_delay]
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set pdelay_timeout                 [expr (2 * int(pow(2,($::ptp_pdelay_req_interval)))) + $::ptp_user_delay] 
set arp_timeout                    $::ARP_timeout

########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_002

}

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Transmission Type as Unicast.                            #
#     v.   Configure PTP on port P1.                                          #
#    vi.   Configure mechanism as peer delay mechanism on port P1.            #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP2)"


    if {![ptp::dut_configure_setup_002 \
                       -port_list          $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

	       ERRLOG -msg "$dut_log_msg(INIT_SETUP2_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP2_F)"\
                                   -tc_def $tc_def
                return 0
        }


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $::ptp_ethtype\
			   -vlan_id                 $vlan_id\
                           -ids                     id1]} {
        return 0
        }

set filter_id [lindex $id1 end]

if {![pltLib::clear_rx_info \
                -session_id              $session_id\
                -port_num                $tee_port_1]} {
                return 0
        }

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0



}

set tee_pdest_mac  $tee_mac
set dut_pdest_mac  $dut_mac
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set tee_pdest_ip   $::tee_test_port_ip
set dut_pdest_ip   $::dut_test_port_ip
set unicast_flag   1

if {![pltLib::clear_rx_info \
                -session_id              $session_id\
                -port_num                $tee_port_1]} {
                return 0
        }
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                  UnicastFlag         = 1                                    #
###############################################################################
sleep $::wait_time
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_UC_TRUE_OBS)"
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_announce\
                           -domain_number           $::default_domain\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
			   -ucast_flag              ucast_flag\
                           -priority1               priority1\
                           -dut_mac                 dut_mac]} {

	  TEE_CLEANUP	
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

if { $ucast_flag != 1 } {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_UC_P1_TRUE_F)"\
                            -tc_def $tc_def

       return 0 
       }
LOG -level 0 -msg   "$tee_log_msg(ANN_RX_UC) $ucast_flag"

set gmpriority1 [ expr $priority1 + 1 ]

###############################################################################
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z+1                                        #
###############################################################################

STEP 4
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"
	if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
			   -dest_mac                $dut_mac\
			   -gmpriority1             $gmpriority1\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
                           -unicast_flag            $unicast_flag\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
			   -domain_number           $::default_domain\
                           -message_type            $::msg_announce\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmid                    $::gmid]} {

	  TEE_CLEANUP	
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def
		return 0
	}

###############################################################################
# Step 5a: Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
###############################################################################

STEP 5a
LOG -level 0 -msg "$tee_log_msg(SYN_RX_UC_TRUE_OBS)"
if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip		    $dut_ip\
			   -dest_ip		    $tee_ip\
                           -src_port             $src_port_event\
                           -dest_port            $dst_port_event\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
			   -ucast_flag              ucast_flag\
                           -filter_id               $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT  -reason "$tee_log_msg(SYN_RX_P1_F)"\
                              -tc_def $tc_def
       return 0
       }
if { $ucast_flag != 1 } {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(SYN_RX_UC_P1_TRUE_F)"\
                            -tc_def $tc_def

       return 0 
       }
LOG -level 0 -msg   "$tee_log_msg(SYN_RX_UC) $ucast_flag"

###############################################################################
# Step 5b:If clock is two step clock, observe that DUT transmits FOLLOW_UP    #
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
###############################################################################

if { $clock_step == "Two Step" } {
STEP 5b
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_UC_TRUE_OBS)"
if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip		    $dut_ip\
			   -dest_ip		    $tee_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_off\
			   -ucast_flag              ucast_flag\
                           -message_type            $::msg_followup\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

          TEE_CLEANUP
          TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_RX_P1_F)"\
                              -tc_def $tc_def
       return 0
}

if { $ucast_flag != 1 } {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(FOLL_RX_UC_P1_TRUE_F)"\
                            -tc_def $tc_def

       return 0 
       }

LOG -level 0 -msg   "$tee_log_msg(FOLL_RX_UC) $ucast_flag"
}
###############################################################################
# Step 6 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters :                                             #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type              = 0x02                           #
#                  Domain Number             = DN1                            #
#                  UnicastFlag               = 1                              #
###############################################################################

STEP 6
LOG -level 0 -msg "$tee_log_msg(PDREQ_RX_UC_TRUE_OBS)"

if {![ptp::recv_pdelayreq\
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -dest_mac             $tee_pdest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $dut_ip\
                        -dest_ip              $tee_pdest_ip\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -message_type         $::msg_pdelayreq\
                        -timeout              $pdelay_timeout\
			-ucast_flag           ucast_flag\
                        -domain_number        $::default_domain\
                        -filter_id            $filter_id]} {

	  TEE_CLEANUP	
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(PDREQ_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }
if { $ucast_flag != 1 } {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(PDREQ_RX_UC_TRUE_P1_F)"\
                            -tc_def $tc_def

       return 0 
       }

LOG -level 0 -msg   "$tee_log_msg(PDREQ_RX_UC) $ucast_flag"

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 7 : Send PDELAY_REQ message on port T1 with following parameters.      #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x02                                 #
#                  Domain Number       = DN1                                  #
###############################################################################


STEP 7
LOG -level 0 -msg "$tee_log_msg(PDREQ_TX_T1)"
if {![ptp::send_pdelayreq\
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $tee_mac\
                        -dest_mac             $dut_pdest_mac\
                        -eth_type             $eth_type1\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $tee_ip\
                        -dest_ip              $dut_pdest_ip\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -unicast_flag         $unicast_flag\
                        -message_type         $::msg_pdelayreq\
                        -domain_number        $::default_domain\
                        -sequence_id          $sequence_pdel]} {

         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(PDREQ_TX_T1_F)"\
                              -tc_def $tc_def

       return 0
       }

###############################################################################
# Step 8a: Verify that DUT transmits PDELAY_RESP message on the port P1 with  #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x03                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
###############################################################################
STEP 8a
LOG -level 0 -msg "$tee_log_msg(PDRESP_RX_UC_TRUE_V)"

if {![ptp::recv_pdelayresp\
                   -session_id          $::session_id\
                   -port_num            $tee_port_1\
                   -dest_mac            $tee_pdest_mac\
                   -eth_type            $eth_type1\
                   -vlan_id             $vlan_id\
                   -type                $eth_type2\
                   -source_ip           $dut_ip\
                   -dest_ip             $tee_pdest_ip\
                   -src_port            $src_port_event\
                   -dest_port           $dst_port_event\
                   -message_type        $::msg_pdelayresp\
                   -domain_number       $::default_domain\
                   -filter_id           $filter_id\
		   -ucast_flag          ucast_flag\
                   -timeout             $response_timeout]} {

# To clear the periodic send / recieve function
              TEE_CLEANUP
              TC_CLEAN_AND_FAIL  -reason "$tee_log_msg(PDRESP_RX_P1_F)"\
                                 -tc_def $tc_def
          return 0

       } else {
LOG -level 0 -msg   "$tee_log_msg(PDRESP_RX_UC) $ucast_flag"

if { $ucast_flag != 1} {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(PDRESP_RX_UC_TRUE_P1_F)"\
                              -tc_def $tc_def

       return 0 
       }
}
###############################################################################
# Step 8b: If the clock is two step clock, Verify that DUT transmits          #
#          PDELAY_RESP_FOLLOW_UP message on port P1 with following parameters.#
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0A                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
###############################################################################
if { $clock_step == "Two Step" } {
STEP 8b
LOG -level 0 -msg "$tee_log_msg(PDRESP_FOLL_RX_UC_TRUE_V)"
       

if {![ptp::recv_pdelayresp_followup\
                   -session_id          $::session_id\
                   -port_num            $tee_port_1\
                   -dest_mac            $tee_pdest_mac\
                   -eth_type            $eth_type1\
                   -vlan_id             $vlan_id\
                   -type                $eth_type2\
                   -source_ip           $dut_ip\
                   -dest_ip             $tee_pdest_ip\
                   -src_port            $src_port_general\
                   -dest_port           $dst_port_general\
		   -ucast_flag          ucast_flag\
                   -message_type        $::msg_pdelayrespfoll\
                   -domain_number       $::default_domain\
                   -filter_id           $filter_id\
                   -timeout             $response_timeout]} {

# To clear the periodic send / recieve function
              TEE_CLEANUP
              TC_CLEAN_AND_FAIL  -reason "$tee_log_msg(PDRESP_FOLL_RX_P1_F)"\
                                 -tc_def $tc_def
          return 0

        }

if { $ucast_flag != 1} {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(PDRESP_RX_UC_TRUE_P1_F)"\
                              -tc_def $tc_def

       return 0 
       }
LOG -level 0 -msg   "$tee_log_msg(PDRESP_FOLL_RX_UC) $ucast_flag"
}
	  TEE_CLEANUP	
          TC_CLEAN_AND_PASS   -reason "DUT configured for unicast transmissions has the unicastFlag field as True in Peer Delay messages"\
                              -tc_def $tc_def

       return 1 
 
