#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_pcg_005                                  #
# Test Case Version : 1.0                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Configuration Group (PCG)                           #
#                                                                             #
# Title             : unicastFlag FALSE - Delay Request-Response - Master     #
#                                                                             #
# Purpose           : Verify that the boundary clock configured for multicast #
#                     transmission in Master has the unicastFlag field as     #
#                     FALSE in Delay Request-Response messages.               #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 13.3.2.6 Table-20 Page 126        #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                   <Clock step - one step/ two-step> |           #
#           |                     <Transmission Type - Multicast> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                                PRT1=Z, UN_FLAG = 0] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN_1,PRT1=Z+1]                      |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                    SYNC [MSG_TYPE = 0x00, DN = DN1, |           #
#           |                                        UN_FLAG = 0] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                          <If Clock Step = Two Step, |           #
#           |                             then receive FOLLOW_UP> |           #
#           |                                                     |           #
#           |               FOLLOW_UP [MSG_TYPE = 0x08, DN = DN1, |           #
#           |                                        UN_FLAG = 0] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           | DELAY_REQ [MSG_TYPE = 0x01, DN = DN1]               |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |              DELAY_RESP [MSG_TYPE = 0x09, DN = DN1, |           #
#           |                                        UN_FLAG = 0] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     UN_FLAG   = UnicastFlag                                                 #
#     PRT1      = Priority1                                                   #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Transmission Type as Multicast.                          #
#     v.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                  UnicastFlag         = 0                                    #
#                                                                             #
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z+1                                        #
#                                                                             #
#                                                                             #
# Step 5a: Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 0                                    #
#                                                                             #
# Step 5b:If clock is two step clock, observe that DUT transmits FOLLOW_UP    #
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 0                                    #
#                                                                             #
# Step 6 : Send DELAY_REQ message on port T1 with following parameters.       #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 7 : Verify that DUT transmits DELAY_RESP message on the port P1 with   #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x09                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 0                                    #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx            Initial                         #
###############################################################################
# Copyright (C) 2013 Veryx Technologies(P) Ltd.                               #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def            "Verify that the boundary clock configured for multicast\
                       transmission in Master has the unicastFlag field as\
                       FALSE in Delay Request-Response messages."


########################### START - INITIALIZATION ############################
set tee_port_1		 $::tee_port_num_1
set tee_session_id 	 $::session_id
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set clock_step           $::ptp_clock_step
set pdelay_mac           $::ptp_delay_mac
set sequence_ann         1
set sequence_sync        16385
set sequence_foll        16385
set sequence_del         32768 
set unicast_flag         0
set src_port_general     "320"
set dst_port_general     "320"
set src_port_event       "319"
set dst_port_event       "319"
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set dut_ip             $::dut_test_port_ip
set tee_ip             $::tee_test_port_ip


set tee_dest_ip                     $::multicast_dest_ip
set dut_dest_ip                     $::multicast_dest_ip
set tee_dest_mac       $::dest_mac
set dut_dest_mac       $::dest_mac
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set arp_timeout        $::ARP_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout       $::ptp_timeout
set timeout           [expr $ptp_timeout + $user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout [expr  int(pow(2,( $::ptp_sync_interval + 5))) +$::ptp_user_delay] 
set pdelay_timeout [expr int(pow(2,($::ptp_pdelay_req_interval)))+ $user_delay]

if {$::ptp_communication_model == "Unicast"} {

	LOG -level 0 -msg   "$tee_log_msg(MC)"

         TC_ABORT   -reason "$tee_log_msg(MC)"\
                              -tc_def $tc_def

       return 0

}

########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Transmission Type as Multicast.                          #
#     v.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

	       ERRLOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################
STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1


if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

###############################################################################
#                                                                             #
#	PRE - REQUISITE CONFIGURATION & ASSIGNING VARIABLES                   #
#                                                                             #
###############################################################################

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                  UnicastFlag         = 0                                    #
###############################################################################
### (Part1) ###
sleep $::wait_time
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_UC_FALSE_OBS)"


if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
			   -ucast_flag              ucast_flag\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -priority1		    priority1\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {

# To clear the periodic send / recieve function	
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }
LOG -level 0 -msg   "$tee_log_msg(TAB)"
LOG -level 0 -msg   "$tee_log_msg(ANN_RX_PRIOR1)"
set recv_priority1 [expr $priority1 +1 ]
LOG -level 0 -msg   "$tee_log_msg(TAB)"
LOG -level 0 -msg   "$tee_log_msg(ANN_RX_UC):$ucast_flag"
if { $ucast_flag != 0 } {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_UC_P1_FALSE_F):$ucast_flag"\
                            -tc_def $tc_def

       return 0 
       }

###############################################################################
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = Z+1                                        #
###############################################################################

STEP 4
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"
if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -message_length          $ann_len\
                           -control_field           $ctrl_announce\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $recv_priority1]} {
	            TEE_CLEANUP	
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ANN_TX_T1_F)"\
                                         -tc_def $tc_def

		return 0
	}

###############################################################################
# Step 5a: Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 0                                    #
###############################################################################
STEP 5a
LOG -level 0 -msg "$tee_log_msg(SYN_RX_UC_FLSE_OBS)"
if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
	                   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip   	            $tee_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
			   -reset_count		    $reset_count_off\
			   -ucast_flag              ucast_flag\
                           -message_type            $msg_sync\
                           -domain_number           $domain_no\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function	
           
	  TEE_CLEANUP	
           TC_CLEAN_AND_ABORT  -reason "$tee_log_msg(SYN_RX_P1_F)."\
                              -tc_def $tc_def

       return 0
       }

LOG -level 0 -msg   "$tee_log_msg(TAB)"
LOG -level 0 -msg   "$tee_log_msg(SYN_RX_UC):$ucast_flag"
if { $ucast_flag != 0 } {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(SYN_RX_UC_P1_FLSE_F):$ucast_flag."\
                              -tc_def $tc_def

       return 0 
       }

###############################################################################
# Step 5b:If clock is two step clock, observe that DUT transmits FOLLOW_UP    #
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 0                                    #
###############################################################################

if { $clock_step == "Two Step" } {
STEP 5b
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_UC_FLSE_OBS)"

if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac		    $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
	                   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip   	            $tee_dest_ip\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
			   -ucast_flag              ucast_flag\
                           -message_type            $msg_followup\
                           -domain_number           $domain_no\
                           -timeout                 $sync_timeout\
			   -reset_count		    $reset_count_off\
                           -filter_id               $filter_id]} {

# To clear the periodic send / recieve function	
          
	  TEE_CLEANUP	
          TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_RX_P1_F)."\
                              -tc_def $tc_def

       return 0

       }

LOG -level 0 -msg   "$tee_log_msg(TAB)"
LOG -level 0 -msg   "$tee_log_msg(FOLL_RX_UC):$ucast_flag"
if { $ucast_flag != 0 } {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(FOLL_RX_UC_P1_FLSE_F):$ucast_flag."\
                              -tc_def $tc_def

       return 0 
       }

}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################       
# Step 6 : Send DELAY_REQ message on port T1 with following parameters.       #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
###############################################################################
STEP 6 
LOG -level 0 -msg "$tee_log_msg(DREQ_TX_T1)"
if {![ptp::send_delay_req\
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $tee_mac\
                        -dest_mac             $dut_dest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $tee_ip\
                        -dest_ip              $dut_dest_ip\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -unicast_flag         $unicast_flag\
                        -sequence_id          $sequence_del\
                        -message_type         $msg_delayreq\
                        -domain_number        $domain_no\
                        -control_field        $ctrl_delayreq]} {
	            TEE_CLEANUP	
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(DREQ_TX_T1_F)"\
                                         -tc_def $tc_def
         
       return 0

       }

###############################################################################
# Step 7 : Verify that DUT transmits DELAY_RESP message on the port P1 with   #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type        = 0x09                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 0                                    #
###############################################################################
STEP 7 
LOG -level 0 -msg "$tee_log_msg(DRESP_RX_UC_FALSE_V)"
if {![ptp::recv_delayresp \
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $dut_mac\
                        -dest_mac             $tee_dest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $dut_ip\
                        -dest_ip              $tee_dest_ip\
                        -src_port             $src_port_general\
                        -dest_port            $dst_port_general\
                        -message_type         $msg_delayresp\
                        -domain_number        $domain_no\
                        -timeout              $timeout\
                        -filter_id            $filter_id\
			-ucast_flag           ucast_flag]} {
                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(DRESP_RX_P1_F)."\
                              -tc_def $tc_def

       return 0 
       } else {
LOG -level 0 -msg   "$tee_log_msg(TAB)"
LOG -level 0 -msg   "$tee_log_msg(DRESP_RX_UC):$ucast_flag"

if { $ucast_flag != 0 } {
		                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(DRESP_RX_UC_FLSE_P1_F)."\
                            -tc_def $tc_def

       return 0 
       }

TEE_CLEANUP	
TC_CLEAN_AND_PASS -reason "$tee_log_msg(DRESP_RX_UC_FLSE_P1)."\
                  -tc_def $tc_def

return 1
}
			      
	      


