#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_pcg_012                                  #
# Test Case Version : 1.0                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Configuration Group (PCG)                           #
#                                                                             #
# Title             : Configuration support for priority2                     #
#                                                                             #
# Purpose           : Verify that the boundary clock supports to configure    #
#                     priority2 to any value in range 0-255.                  #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 7.6.2.3 Page 54                   #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                                                     |           #
#           |                         <Configure priority2 =  X1> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                                         PRI2 =  X1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                         <Configure priority2 =  X2> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                                         PRI2 =  X2] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                         <Configure priority2 =  X3> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                                         PRI2 =  X3] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                        !<Configure priority2 =  N1> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                                         PRI2 =  N1] |           #
#        T1 |                         XX----------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                        !<Configure priority2 =  N2> |           #
#           |                                                     |           #
#           |                ANNOUNCE [MSG_TYPE = 0x0B, DN = DN1, |           #
#           |                                         PRI2 =  N2] |           #
#        T1 |                         XX----------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     PRI2      = priority2                                                   #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Configured priority2:                                                   #
#        0 <= Xn <= 255                                                       #
#        Nn < 0 or Nn > 255                                                   #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part I)                                                                    #
#                                                                             #
# Step 3 : Configure priority2 as X1 on Port P1 in DUT. (X1 = 0)              #
#                                                                             #
# Step 4 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  priority2        = X1                                      #
#                                                                             #
# Step 5 : Configure priority2 as X2 on Port P1 in DUT. (X2 = 50)             #
#                                                                             #
# Step 6 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  priority2        = X2                                      #
#                                                                             #
# Step 7 : Configure priority2 as X3 on Port P1 in DUT. (X3 = 255)            #
#                                                                             #
# Step 8 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  priority2        = X3                                      #
#                                                                             #
# Step 9a: Verify that DUT does not allows to configure priority2 as N1 on    #
#           Port P1 in DUT. (N1 = -1)                                         #
#                                                                             #
# Step 9b: Verify that DUT does not transmit ANNOUNCE message on the port P1  #
#           with priority2 same as the configured priority2(N1).              #
#                                                                             #
# Step 10a: Verify that DUT does not allows to configure priority2 as N2 on   #
#           Port P1 in DUT. (N2 = 256)                                        #
#                                                                             #
# Step 10b: Verify that DUT does not transmit ANNOUNCE message on the port P1 #
#           with priority2 same as the configured priority2(N2).              #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx            Initial                         #
# 1.1          Jan/2017      Veryx            Updated Ladder & relevant steps #
###############################################################################
# Copyright (C) 2013 Veryx Technologies(P) Ltd.                               #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock supports to configure\
		priority2 to any  the values in range 0 - 255."


########################### START - INITIALIZATION ############################
set tee_port_1		 $::tee_port_num_1
set tee_session_id 	 $::session_id
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set clock_mode		 "BC"
set unicast_flag         0
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set dut_ip             $::dut_test_port_ip
set tee_ip             $::tee_test_port_ip

set dut_dest_ip        $::dut_test_port_ip
set tee_dest_ip        $::tee_test_port_ip

set src_port_general   "320"
set dst_port_general   "320"
set tee_dest_mac       $::dest_mac
set dut_dest_mac       $::dest_mac
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set arp_timeout        $::ARP_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout       $::ptp_timeout
set timeout           [expr $ptp_timeout + $user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout [expr  int(pow(2,( $::ptp_sync_interval +5))) +$::ptp_user_delay] 
set pdelay_timeout [expr int(pow(2,($::ptp_pdelay_req_interval)))+ $user_delay]
set priority2_0 "0"
set priority2_158 "158"
set priority2_255 "255"
set priority2_minus1 "-1"
set priority2_256 "256"

ptp::dut_Initialization
set dut_session_id       $::dut_session_handler
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_001
set result [ptp::dut_set_priority2\
		-session_handler $::dut_session_handler\
		-clock_mode      $::ptp_dut_clock_mode\
		-priority2       $::ptp_dut_clock_priority2]

if { $result == 0 } {
           
          LOG -level 0 -msg "Unable to set priority2 ($::ptp_dut_clock_priority2)"
	
	}

}       
##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

	       ERRLOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}

###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization


set tee_mac           [string toupper  $::TEEMAC_1]

### (Part1) ###
### Creating filter - Receive packet ###
if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }

set filter_id [lindex $id1 end]


if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg   "$tee_log_msg(ARP_F1)"

           TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip


}
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 3 : Configure priority2 as X1 on Port P1 in DUT. (X1 = 0)              #
###############################################################################

sleep $::wait_time
STEP 3 
LOG -level 0 -msg "$dut_log_msg(PRIOR2_EQU_0)" 


set result [ptp::dut_set_priority2\
		-session_handler $dut_session_id\
		-clock_mode      $clock_mode\
		-priority2       $priority2_0]
if { $result == 0 } {
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$dut_log_msg(PRIOR2_EQU_0_F)"\
                              -tc_def $tc_def
	   return 0
	}

pltLib::clear_filter_pkt_dump\
		 -session_id		$::session_id\
		 -port_num		$tee_port_1


 if {![pltLib::clear_rx_info \
		-session_id              $session_id\
		-port_num                $tee_port_1]} {

		return 0 
	}
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 4 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  priority2        = X1                                      #
###############################################################################
STEP 4
LOG -level 0 -msg "$tee_log_msg(ANN_RX_PRIOR2_VER)"
 
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -gmpriority2		    $priority2_0\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }



###############################################################################
# Step 5 : Configure priority2 as X2 on Port P1 in DUT. (X2 = 158)            #
###############################################################################

STEP 5
LOG -level 0 -msg "$dut_log_msg(PRIOR2_EQU_158)"

set result [ptp::dut_set_priority2\
		-session_handler $dut_session_id\
		-clock_mode      $clock_mode\
		-priority2       $priority2_158]

if { $result == 0 } {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$dut_log_msg(PRIOR2_EQU_158_F)"\
                              -tc_def $tc_def
	   return 0
	}
pltLib::clear_filter_pkt_dump\
		 -session_id		$::session_id\
		 -port_num		$tee_port_1


 if {![pltLib::clear_rx_info \
		-session_id              $session_id\
		-port_num                $tee_port_1]} {

		return 0 
	}
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 6 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  priority2        = X3                                      #
###############################################################################
STEP 6 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_PRIOR2_VER)"
 
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -gmpriority2		    $priority2_158\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }



###############################################################################
# Step 7 : Configure priority2 as X3 on Port P1 in DUT. (X3 = 255)            #
###############################################################################

STEP 7
LOG -level 0 -msg "$dut_log_msg(PRIOR2_EQU_255)"

set result [ptp::dut_set_priority2\
		-session_handler $dut_session_id\
		-clock_mode      $clock_mode\
		-priority2       $priority2_255]

if { $result == 0 } {

           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$dut_log_msg(PRIOR2_EQU_255_F)"\
                              -tc_def $tc_def
           return 0
	
	}
pltLib::clear_filter_pkt_dump\
		 -session_id		$::session_id\
		 -port_num		$tee_port_1


 if {![pltLib::clear_rx_info \
		-session_id              $session_id\
		-port_num                $tee_port_1]} {

		return 0 
	}
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 8 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type     = 0x0B                                    #
#                  Domain Number    = DN1                                     #
#                  priority2        = X3                                      #
###############################################################################
STEP 8
LOG -level 0 -msg "$tee_log_msg(ANN_RX_PRIOR2_VER)"
 
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -gmpriority2		    $priority2_255\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }


###############################################################################
# Step 9a:  Verify that DUT does not allows to configure priority2 as N1 on   #
#           Port P1 in DUT. (N1 = -1)                                         #
###############################################################################

STEP 9a 
LOG -level 0 -msg "$dut_log_msg(PRIOR2_MINUS1_V)"

set result [ptp::dut_set_priority2\
		-session_handler $dut_session_id\
		-clock_mode      $clock_mode\
		-priority2       $priority2_minus1]

if { $result == 0 } {
LOG -level 0 -msg "$dut_log_msg(PRIOR2_EQU_MINUS1_F)"
} else {
            LOG -level 0 -msg "$dut_log_msg(PRIOR2_EQU_MINUS1_S)"\

	}

###############################################################################
# Step 9b : Verify that DUT does not transmit ANNOUNCE message on the port P1 #
#           with priority2 same as the configured priority2(N1).              #
###############################################################################
STEP 9b 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_PRIOR2_MINUS1_VER)"
 
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -gmpriority2		    $priority2_255\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_PRIOR2_S)"
###############################################################################
# Step 10a: Verify that DUT does not allows to configure priority2 as N2 on   #
#           Port P1 in DUT. (N2 = 256)                                        #
###############################################################################

STEP 10a 
LOG -level 0 -msg "$dut_log_msg(PRIOR2_256_V)"

set result [ptp::dut_set_priority2\
		-session_handler $dut_session_id\
		-clock_mode      $clock_mode\
		-priority2       $priority2_256]
if { $result == 0 } {
LOG -level 0 -msg "$dut_log_msg(PRIOR2_EQU_256_F)"
} else {
           LOG -level 0 -msg "$dut_log_msg(PRIOR2_EQU_256_S)"\
}
###############################################################################
# Step 10b: Verify that DUT does not transmit ANNOUNCE message on the port P1 #
#           with priority2 same as the configured priority2(N2).              #
###############################################################################
STEP 10b  
LOG -level 0 -msg "$tee_log_msg(ANN_RX_PRIOR2_256_VER)"
 
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -gmpriority2		    $priority2_255\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }



TEE_CLEANUP
TC_CLEAN_AND_PASS -reason "$tee_log_msg(ANN_RX_P1_PRIOR2_S)"\
                  -tc_def $tc_def
return 1


