#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_pcg_013                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Configuration Group (PCG)                           #
#                                                                             #
# Title             : Configuration support of Slave-only clocks              #
#                                                                             #
# Purpose           : Verify that the boundary clock does not allows to       #
#                     configure it as a Slave-only clock.                     #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.2.3 Page 76                     #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                                                     |           #
#           |                    <!Configure as Slave-only clock> |           #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
#                                                                             #    
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Verify that DUT does not allows to configure as Slave-only clock.  #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2013      Veryx            Initial                         #
# 1.1          Jan/2017      Veryx            Updated steps and script        #
###############################################################################
# Copyright (C) 2013 - 2017 Veryx Technologies(P) Ltd.                        #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################  

set tc_def      "Verify that the boundary clock does not allows to configure\
				 it as a Slave-only clock."


########################### START - INITIALIZATION ############################
set tee_port_1		 $::tee_port_num_1
set tee_session_id 	 $::session_id
set dut_port_1		 $::dut_port_num_1
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set clock_mode		 "BC"

set vlan_id $::ptp_vlan_id
set dut_port_list	 $::dut_port_num_1
set dut_port_num_1       [lindex $dut_port_list 0]

set clock_mode  	       $::ptp_dut_clock_mode
ptp::dut_Initialization
set dut_session_id       $::dut_session_handler
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_001
		  set result [ptp::dut_disable_slave_only\
							  -session_handler         $::dut_session_handler\
							  -port_num                $::dut_port_num_1\
							  -clock_mode              $::ptp_dut_clock_mode\
							  -slave_only              "false"]

if { $result == 0 } {

		   LOG -level 0 -msg "Unable to set slave only as false"

		}
}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


	if {![ptp::dut_configure_setup_001 \
					   -port_list	   $::dut_port_num_1\
					   -vm_mac             $::vm1_mac]} {

		   ERRLOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add ports T1 and T2 at TEE.                                        #
###############################################################################

STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

### (Part1) ###
###############################################################################
# Step 3 : Verify that DUT does not allows to configure as Slave-only clock.  #
#                                                                             #
###############################################################################
STEP 3
LOG -level 0 -msg "$dut_log_msg(SLV_CLKONLY_F_VER)"


if {[ptp::dut_enable_slave_only\
			 -session_handler          $dut_session_id\
				  -port_num                 $dut_port_1\
					   -clock_mode               $clock_mode\
					  -slave_only               "true"]} {


		   TEE_CLEANUP
		   TC_CLEAN_AND_FAIL -reason "$dut_log_msg(SLV_CLKONLY)"\
							  -tc_def $tc_def
	   return 0 
		
	} else {

		   TC_CLEAN_AND_PASS -reason "$dut_log_msg(SLV_CLKONLY_F)"\
							  -tc_def $tc_def
	   return 1

	}

