#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_pcg_014                                  #
# Test Case Version : 1.0                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Configuration Group (PCG)                           #
#                                                                             #
# Title             : Announce Message log Interval                           #
#                                                                             #
# Purpose           : Verify that the DUT transmits announce message with     #
#                     logMessageInterval field which matches it's configured  #
#                     Announce Interval.                                      #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 7.7.2.2 Page 62 and 8.2.5.4.1     #
#                     Page 73, 13.3.2.11 Table 24 Page 128                    #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                                                     |           #
#           |                  <Configure announce interval = X1> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                            DN = DN1, MSG_INTV = X1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                  <Configure announce interval = X2> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                            DN = DN1, MSG_INTV = X2] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                  <Configure announce interval = X3> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                            DN = DN1, MSG_INTV = X3] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                  <Configure announce interval = M1> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                            DN = DN1, MSG_INTV = M1] |           #
#        T1 |                      XX-------------------------<<--| P1        #
#           |                                                     |           #
#           |                  <Configure announce interval = M2> |           #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                            DN = DN1, MSG_INTV = M2] |           #
#        T1 |                      XX-------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     MSG_INTV  = LogMessageInterval                                          #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Configured Announce Interval:                                           #
#        0 <= Xn <= 4                                                         #
#        Mn < 0 or Mn > 4                                                     #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Configure Announce Interval as X1 on Port P1 in DUT. (X1 = 0)      #
#                                                                             #
# Step 4 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          logMessageInteraval same as the configured Announce Interval.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = X1                                    #
#                                                                             #
# Step 5 : Configure Announce Interval as X2 on Port P1 in DUT. (X2 = 2)      #
#                                                                             #
# Step 6 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          logMessageInteraval same as the configured Announce Interval.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = X2                                    #
#                                                                             #
# Step 7 : Configure Announce Interval as X3 on Port P1 in DUT. (X3 = 4)      #
#                                                                             #
# Step 8 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          logMessageInteraval same as the configured Announce Interval.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = X3                                    #
#                                                                             #
# Step 9a: Verify that DUT does not allow to configure announce interval      #
#           as M1 on Port P1. (M1 = -1)                                       #
#                                                                             #
# Step 9b: Verify that DUT  does not transmit ANNOUNCE message on the port P1 #
#          with logMessageInteraval same as the configured Announce Interval. #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = M1                                    #
#                                                                             #
#Step 10a: Verify that DUT does not allow to configure announce interval      #
#           as M2 on Port P1. (M2 = 5)                                        #
#                                                                             #
#Step 10b: Verify that DUT  does not transmit ANNOUNCE message on the port P1 #
#          with logMessageInteraval same as the configured Announce Interval. #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = M2                                    #
###############################################################################

#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx            Initial                         #
###############################################################################
# Copyright (C) 2013 Veryx Technologies(P) Ltd.                               #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id

####################### LOCAL VARIABLES ARE DECLARED HERE #####################



set tc_def      " Verify that the DUT transmits announce message with\
                  logMessageInterval field which matches it's configured\
                  Announce Interval."

#ptp::Initialization
########################### START - INITIALIZATION ############################

set tee_port_1		 $::tee_port_num_1
set tee_port_2		 $::tee_port_num_2
set tee_session_id 	 $::session_id
set dut_port_list	 $::dut_port_num_1
set dut_port_num_1       [lindex $dut_port_list 0]
set vlan_id		 $::ptp_vlan_id
set eth_type1		 $::eth_type1
set eth_type2		 $::eth_type2
set dest_mac             $::dest_mac
set clock_step           $::ptp_clock_step
set pdelay_mac           $::ptp_delay_mac
set dut_port_1		 $::dut_port_num_1
set sequence_ann         1
set sequence_sync        16385
set sequence_foll        16385
set sequence_del         32768 
set unicast_flag         0
set src_port_general     "320"
set dst_port_general     "320"
set src_port_event       "319"
set dst_port_event       "319"
#message Type
set msg_sync             $::msg_sync
set msg_delayreq         $::msg_delayreq
set msg_pdelayreq        $::msg_pdelayreq
set msg_pdelayresp       $::msg_pdelayresp
set msg_followup         $::msg_followup
set msg_delayresp        $::msg_delayresp
set msg_pdelayrespfoll   $::msg_pdelayrespfoll
set msg_announce         $::msg_announce
#Control Field
set ctrl_sync            $::ctrl_sync
set ctrl_delayreq        $::ctrl_delayreq
set ctrl_followup        $::ctrl_followup
set ctrl_delayresp       $::ctrl_delayresp
set ctrl_management      $::ctrl_management
set ctrl_announce        $::ctrl_announce
set ctrl_pdelayreq       $::ctrl_pdelayreq
set ctrl_pdelayresp      $::ctrl_pdelayresp
set ctrl_pdelayrespfoll  $::ctrl_pdelayrespfoll
#message length
set ann_len              $::ann_len
set sync_len             $::sync_len
set followup_len         $::followup_len
set delayreq_len         $::delayreq_len
set delayresp_len        $::delayresp_len
set pdelayreq_len        $::pdelayreq_len
set pdelayresp_len       $::pdelayresp_len
set pdelayrespfoll_len   $::pdelayrespfoll_len

set gmid                 $::gmid
set domain_no            $::default_domain
set reset_count_on       $::reset_count_on
set reset_count_off      $::reset_count_off
set ptp_ethtype          $::ptp_ethtype
set dut_ip             $::dut_test_port_ip
set tee_ip             $::tee_test_port_ip

set dut_dest_ip        $::dut_test_port_ip
set tee_dest_ip        $::tee_test_port_ip

set tee_dest_mac       $::dest_mac
set dut_dest_mac       $::dest_mac
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set arp_timeout        $::ARP_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout       $::ptp_timeout
set timeout           [expr $ptp_timeout + $user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout [expr  int(pow(2,( $::ptp_sync_interval + 5))) +$::ptp_user_delay] 
set pdelay_timeout [expr int(pow(2,($::ptp_pdelay_req_interval)))+ $user_delay]
set announce_interval_0 "0"
set announce_interval_2 "2"
set announce_interval_4 "4"
set announce_interval_minus1 "-1"
set announce_interval_5 "5"

########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_001
set result [ptp::dut_set_announce_interval\
                -session_handler     $::dut_session_handler\
                -clock_mode          $::ptp_dut_clock_mode\
                -announce_interval   $::ptp_announce_interval\
                -port_num            $::dut_port_num_1]

if { $result == 0 } {

           LOG -level 0 -msg "Unable to set announce interval ($::ptp_announce_interval)"

        }

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

	       ERRLOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################
STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1


if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }

set filter_id [lindex $id1 end]

###############################################################################
#                                                                             #
#	PRE - REQUISITE CONFIGURATION & ASSIGNING VARIABLES                   #
#                                                                             #
###############################################################################

if {$::ptp_communication_model == "Unicast"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}

set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip

}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1



set session_handler $::dut_session_handler
      

set clock_mode             $::ptp_dut_clock_mode


###############################################################################
# Step 3 : Configure Announce Interval as X1 on Port P1 in DUT. (X1 = 0)      #
###############################################################################
STEP 3
LOG -level 0 -msg "$dut_log_msg(ANN_INTVL_0)"
set result [ptp::dut_set_announce_interval \
                             -session_handler     $session_handler \
                             -clock_mode          $clock_mode\
                             -announce_interval   $announce_interval_0\
                             -port_num            $dut_port_num_1]
if { $result == 0 } {
        TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$dut_log_msg(ANN_INTVL_0_F)"\
                              -tc_def $tc_def
           
           return 0
      }
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
set announceReceiptTimeoutInterval_0   [expr (int(pow(2,$announce_interval_0)) * $announce_timeout) + $user_delay]

###############################################################################
# Step 4 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          logMessageInteraval same as the configured Announce Interval.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = X1                                    #
###############################################################################
 
sleep $::wait_time
STEP 4 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_V)"
if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval_0\
			   -priority1		    priority1\
			   -logmsg_interval         $announce_interval_0\
                           -logmsginterval          logmsginterval\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

         LOG -level 0 -msg   "DUT transmits Announce message with logMessageInterval value $logmsginterval\
	                      which matches the configured Announce interval $announce_interval_0"
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 5 : Configure Announce Interval as X2 on Port P1 in DUT. (X2 = 2)      #
###############################################################################
STEP 5 
LOG -level 0 -msg "$dut_log_msg(ANN_INTVL_2)"

set result [ptp::dut_set_announce_interval \
                             -session_handler     $session_handler \
                             -clock_mode          $clock_mode\
                             -announce_interval   $announce_interval_2\
                             -port_num            $dut_port_num_1]
if { $result == 0 } {
        TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$dut_log_msg(ANN_INTVL_2_F)"\
                              -tc_def $tc_def
           
           return 0
      }

set announceReceiptTimeoutInterval_2   [expr (int(pow(2,$announce_interval_2)) * $announce_timeout) + $user_delay]
###############################################################################
# Step 6 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          logMessageInteraval same as the configured Announce Interval.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = X2                                    #
###############################################################################
 
STEP 6 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_V)"

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval_2\
			   -priority1		    priority1\
			   -logmsg_interval         $announce_interval_2\
                           -logmsginterval          logmsginterval\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
# To clear the periodic send / recieve function	
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

         LOG -level 0 -msg   "DUT transmits Announce message with logMessageInterval value $logmsginterval\
	                      which matches the configured announce interval $announce_interval_2"
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 7 : Configure Announce Interval as X3 on Port P1 in DUT. (X3 = 4)      #
###############################################################################
STEP 7 
LOG -level 0 -msg "$dut_log_msg(ANN_INTVL_4)"

set result [ptp::dut_set_announce_interval \
                             -session_handler     $session_handler \
                             -clock_mode          $clock_mode\
                             -announce_interval   $announce_interval_4\
                             -port_num            $dut_port_num_1]
if { $result == 0 } {
        TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$dut_log_msg(ANN_INTVL_4_F)"\
                              -tc_def $tc_def
           
           return 0
      }
set announceReceiptTimeoutInterval_4   [expr (int(pow(2,$announce_interval_4)) * $announce_timeout) + $user_delay]
###############################################################################
# Step 8 : Verify that DUT transmits ANNOUNCE message on the port P1 with     #
#          logMessageInteraval same as the configured Announce Interval.      #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = X3                                    #
###############################################################################
 
STEP 8 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_P1_V)"

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval_4\
			   -priority1		    priority1\
			   -logmsg_interval         $announce_interval_4\
                           -logmsginterval          logmsginterval\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }

         LOG -level 0 -msg   "DUT transmits Announce message with logMessageInterval value $logmsginterval which matches the configured Announce interval  $announce_interval_4 "

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 9a: Verify that DUT does not allow to configure announce interval      #
#           as M1 on Port P1. (M1 = -1)                                       #
###############################################################################
STEP 9a 
LOG -level 0 -msg "$dut_log_msg(ANN_INTVL_MINUS1_NOT)"

set result [ptp::dut_set_announce_interval \
                             -session_handler     $session_handler \
                             -clock_mode          $clock_mode\
                             -announce_interval   $announce_interval_minus1\
                             -port_num            $dut_port_num_1]

if { $result == 0 } {          
           LOG -level 0 -msg "$dut_log_msg(ANN_INTVL_MINUS1_F)"
      } else {
           LOG -level 0 -msg "$dut_log_msg(ANN_INTVL_MINUS1_S)"
}
      
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 9b: Verify that DUT  does not transmit ANNOUNCE message on the port P1 #
#          with logMessageInteraval same as the configured Announce Interval. #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = M1                                    #
###############################################################################
 
STEP 9b 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_MINUS1_NOT)"

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval_4\
			   -priority1		    priority1\
			   -logmsg_interval         $announce_interval_4\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {
           
           
        TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                             -tc_def $tc_def

return 0
}
  LOG -level 0 -msg   "DUT transmits Announce message with logMessageInterval value $announce_interval_4 which matches the previously configured Announce Interval $announce_interval_4"
   

###############################################################################
#Step 10a: Verify that DUT does not allow to configure announce interval      #
#           as M2 on Port P1. (M2 = 5)                                        #
###############################################################################
STEP 10a 
LOG -level 0 -msg "$dut_log_msg(ANN_INTVL_PLUS5_NOT)"

set result [ptp::dut_set_announce_interval \
                             -session_handler     $session_handler \
                             -clock_mode          $clock_mode\
                             -announce_interval   $announce_interval_5\
                             -port_num            $dut_port_num_1]
if { $result == 0 } {
          
           LOG -level 0 -msg "$dut_log_msg(ANN_INTVL_PLUS5_F)"
      } else {
           LOG -level 0 -msg "$dut_log_msg(ANN_INTVL_PLUS5_S)"
}
      
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
set announceReceiptTimeoutInterval_5   [expr (int(pow(2,$announce_interval_5)) * $announce_timeout) + $user_delay]
###############################################################################
#Step 10b: Verify that DUT  does not transmit ANNOUNCE message on the port P1 #
#          with logMessageInteraval same as the configured Announce Interval. #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type       = 0x0B                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = M2                                    #
###############################################################################
 
STEP 10b
LOG -level 0 -msg "$tee_log_msg(ANN_RX_PLUS5_NOT)"

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval_5\
			   -priority1		    priority1\
			   -logmsg_interval         $announce_interval_4\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {

        TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(ANN_RX_P1_F)"\
                             -tc_def $tc_def

return 0
}

           TEE_CLEANUP
           TC_CLEAN_AND_PASS -reason   "DUT transmits Announce message with logMessageInterval value $announce_interval_4 which matches the previously configured Announce Interval $announce_interval_4"\
                              -tc_def $tc_def
   
      return 1

