#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_pcg_016                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Configuration Group (PCG)                           #
#                                                                             #
# Title             : Sync Message log Interval - Unicast transmission        #
#                                                                             #
# Purpose           : Verify that the DUT transmits unicast sync/ sync and    #
#                     follow_up message with logMessageInterval field as 7F.  #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 13.3.2.11 Table 24 Page 128       #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                       <Transmission Type - Unicast> |           #
#           |                                                     |           #
#           |                                                     |           #
#           |          ANNOUNCE [DEST_MAC = UC1, MSG_TYPE = 0x0B, |           #
#           |                                  DN = DN1,PRT1 = Z] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [DEST_MAC = UC2, MSG_TYPE = 0x0B,          |           #
#           |            DN = DN_1,PRT1=Z+1]                      |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                      <Configure sync interval = X1> |           #
#           |                                                     |           #
#           |               SYNC [DEST_MAC = UC1,MSG_TYPE = 0x00, |           #
#           |                            DN = DN1, MSG_INTV = 7F] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                          <If Clock Step = Two Step, |           #
#           |                             then receive FOLLOW_UP> |           #
#           |                                                     |           #
#           |          FOLLOW_UP [DEST_MAC = UC1,MSG_TYPE = 0x08, |           #
#           |                            DN = DN1, MSG_INTV = 7F] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                                                     |           #
#           |                                                     |           #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     MSG_INTV  = LogMessageInterval                                          #
#     PRT1      = Priority1                                                   #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Configured Sync Interval:                                               #
#        -1 <= Xn <= 1                                                        #
#        Mn < -1 or Mn > 1                                                    #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Transmission Type as Unicast.                            #
#     v.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Destination MAC = UC1                                      #
#                  Message Type    = 0x0B                                     #
#                  Domain Number   = DN1                                      #
#                  Priority1       = Z                                        #
#                                                                             #
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Destination MAC = UC2                                      #
#                  Message Type    = 0x0B                                     #
#                  Domain Number   = DN1                                      #
#                  Priority1       = Z+1                                      #
#                                                                             #
# Step 5 : Configure Sync Interval as X1 on Port P1 in DUT. (X1 = 1)          #
#                                                                             #
# Step 6a: Verify that DUT transmits SYNC message on the port P1 with         #
#          logMessageInteraval same as the configured Sync Interval.          #
#                                                                             #
#              PTP Header                                                     #
#                  Destination MAC    = UC1                                   #
#                  Message Type       = 0x00                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = 7F (in hex)                           #
#                                                                             #
# Step 6b:If the clock is two step clock, Verify that DUT transmits FOLLOW_UP #
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              PTP Header                                                     #
#                  Destination MAC    = UC1                                   #
#                  Message Type       = 0x08                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = 7F (in hex)                           #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx         Initial                            #
#                                                                             #
# 1.1          Jun/2017      Veryx         Handled logMessageinterval for     #
#                                          unicast (CR ID - 2447).            #
###############################################################################
# Copyright (C) 2017 Veryx Technologies(P) Ltd.                               #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the DUT transmits unicast sync/ sync and\
                 follow_up message with logMessageInterval field as 7F."

if { $::ptp_communication_model != "Unicast" } {

	LOG -level 0 -msg   "$tee_log_msg(UC)"
         TC_ABORT   -reason "$tee_log_msg(UC)"\
                           -tc_def $tc_def
	return 0 
	}

########################### START - INITIALIZATION ############################

set tee_port_1		            $::tee_port_num_1
set tee_session_id 	            $::session_id
set dut_port_list	            $::dut_port_num_1
set dut_port_num_1                  [lindex $dut_port_list 0]
set dut_port_1		            $::dut_port_num_1
set vlan_id		            $::ptp_vlan_id
set eth_type1		            $::eth_type1
set eth_type2		            $::eth_type2
set dest_mac                        $::dest_mac
set ann_interval                    $::ptp_announce_interval
set clock_step                      $::ptp_clock_step
set pdelay_mac                      $::ptp_delay_mac
set sequence_ann                    1
set correction_field                0
set sequence_sync                   16385
set sequence_del                    32768
set sequence_pdel                   49151
set src_port_general                "320"
set dst_port_general                "320"
set src_port_event                  "319"
set dst_port_event                  "319"
set unicast_flag                    1
set sync_int1                       1
set unilogmsgint                    [hex2dec 7f]

set dut_ip                          $::dut_test_port_ip
set tee_ip                          $::tee_test_port_ip

set dut_dest_ip                     $::dut_test_port_ip
set tee_dest_ip                     $::tee_test_port_ip

set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval +5))) +$::ptp_user_delay]
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set arp_timeout                    $::ARP_timeout

########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_001

}

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Transmission Type as Unicast.                            #
#     v.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list          $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

	       ERRLOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
                                   -tc_def $tc_def
                return 0
        }


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $::ptp_ethtype\
			   -vlan_id                 $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

if {![pltLib::clear_rx_info \
                -session_id              $session_id\
                -port_num                $tee_port_1]} {
                return 0
        }

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0



}

set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set tee_pdest_ip   $::tee_test_port_ip
set pdelay_mac     "dont_care" 
set unicast_flag   1
set session_handler $::dut_session_handler
      

set clock_mode             $::ptp_dut_clock_mode

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
sleep $::wait_time
###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Destination MAC     = UC1                                  #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
###############################################################################

STEP 3
LOG -level 0 -msg "$tee_log_msg(ANN_RX_UC_TRUE_OBS)"
### Creating filter - Receive packet ###

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_ip\
                           -unicast_flag            $unicast_flag\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_announce\
                           -domain_number           $::default_domain\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
                           -priority1               priority1\
                           -dut_mac                 dut_mac]} {

          TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }


set gmpriority1 [ expr $priority1 + 1 ]

###############################################################################
# Step 4 : Send ANNOUNCE message on port T1 with following parameters:        #
#                                                                             #
#              PTP Header                                                     #
#                  Destination MAC = UC2                                      #
#                  Message Type    = 0x0B                                     #
#                  Domain Number   = DN1                                      #
#                  Priority1       = Z+1                                      #
###############################################################################
STEP 4
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"
	if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
			   -dest_mac                $dut_mac\
			   -gmpriority1             $gmpriority1\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
                           -unicast_flag            $unicast_flag\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
			   -domain_number           $::default_domain\
                           -message_type            $::msg_announce\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmid                    $::gmid]} {

          TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def
		return 0
	}

###############################################################################
# Step 5 : Configure Sync Interval as X1 on Port P1 in DUT. (X1 = 1)          #
###############################################################################
STEP 5: "Configure Sync Interval as X1 on Port P1 in DUT. (X1 = 1)"
if {![ptp::dut_set_sync_interval\
                      -session_handler     $session_handler\
		      -port_num            $dut_port_num_1\
                      -clock_mode          $clock_mode\
                      -sync_interval       $sync_int1 ]} {
                  
                  TEE_CLEANUP
	          TC_CLEAN_AND_ABORT -reason "DUT does not allow to configure $sync_int1 as sync interval"\
                                     -tc_def $tc_def
		  return 0
   } else {
                  LOG -level 0 -msg "DUT allows to configure $sync_int1 as sync interval"
}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 6a: Verify that DUT transmits SYNC message on the port P1 with         #
#          logMessageInteraval same as the configured Sync Interval.          #
#                                                                             #
#              PTP Header                                                     #
#                  Destination MAC    = UC1                                   #
#                  Message Type       = 0x00                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = 7F (in hex)                           #
###############################################################################
STEP 6a
LOG -level 0 -msg "$tee_log_msg(SYN_RX_UC_TRUE_OBS)"
if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip		    $dut_ip\
			   -dest_ip		    $tee_ip\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -unicast_flag            $unicast_flag\
                           -logmsg_interval         $unilogmsgint\
                           -filter_id               $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_FAIL  -reason "$tee_log_msg(SYN_RX_P1_F)"\
                              -tc_def $tc_def
       return 0
       }

	 if { $clock_step == "One Step" } {
          TEE_CLEANUP
         TC_CLEAN_AND_PASS    -reason "DUT transmits Unicast Sync message with logMessageInterval field as 7F."\
                              -tc_def $tc_def
       return 1
       } else {
       LOG -level 0 -msg "DUT transmits Unicast Sync message with logMessageInterval field as 7F."
       }

###############################################################################
# Step 6b:If the clock is two step clock, Verify that DUT transmits FOLLOW_UP #
#         message on the port P1 with following parameters.                   #
#                                                                             #
#              PTP Header                                                     #
#                  Destination MAC    = UC1                                   #
#                  Message Type       = 0x08                                  #
#                  Domain Number      = DN1                                   #
#                  LogMessageInterval = 7F (in hex)                           #
###############################################################################

if { $clock_step == "Two Step" } {
STEP 6b
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_UC_TRUE_OBS)"
if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip		    $dut_ip\
			   -dest_ip		    $tee_ip\
                           -unicast_flag            $unicast_flag\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_followup\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -logmsg_interval         $unilogmsgint\
                           -filter_id               $filter_id]} {

          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(FOLL_RX_P1_F)"\
                              -tc_def $tc_def
       return 0
       }

         TEE_CLEANUP
         TC_CLEAN_AND_PASS    -reason "DUT transmits Unicast Sync and Follow_up message\
                                       with logMessageInterval field as 7F."\
                              -tc_def $tc_def

       return 1
}

