#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_pcg_017                                  #
# Test Case Version : 1.0                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP Configuration Group (PCG)                           #
#                                                                             #
# Title             : unicastFlag TRUE - Delay Request-Response - Slave       #
#                                                                             #
# Purpose           : Verify that the boundary clock configured for unicast   #
#                     transmission in Slave has the unicastFlag field as TRUE #
#                     in Delay Request message.                               #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 7.3.8.1 Page 46                   #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                   <Clock step - one step/ two-step> |           #
#           |                       <Transmission Type - Unicast> |           #
#           |                                                     |           #
#           |          ANNOUNCE [DEST_MAC = UC1, MSG_TYPE = 0x0B, |           #
#           |                      DN = DN1, PRT1=Z, UN_FLAG = 1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | ANNOUNCE [DEST_MAC = UC2, MSG_TYPE = 0x0B,          |           #
#           |           DN = DN_1,PRT1=Z-1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |<Send Sync/ Sync & Follow-up msgs                    |           #
#           |based on clock step>                                 |           #
#           |                                                     |           #
#           |         DELAY_REQ [DEST_MAC = UC1, MSG_TYPE = 0x01, |           #
#           |                             DN = DN_1, UN_FLAG = 1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     UN_FLAG   = UnicastFlag                                                 #
#     PRT1      = Priority1                                                   #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Transmission Type as Unicast.                            #
#     v.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                  UnicastFlag         = 1                                    #
#                                                                             #
# Step 4 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z-1                                  #
#                                                                             #
# Step 5a: Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 5b: If the clock is two step clock, send periodic FOLLOW_UP message    #
#          on port T1 with following parameters:                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
#                                                                             #
# Step 6 : Verify that DUT transmits DELAY_REQ message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx            Initial                         #
###############################################################################
# Copyright (C) 2017 Veryx Technologies(P) Ltd.                               #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id
####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock configured for unicast\
                 transmission in Slave has the unicastFlag field as TRUE\
                 in Delay Request message."

if { $::ptp_communication_model != "Unicast" } {


	LOG -level 0 -msg   "$tee_log_msg(UC)"
        TC_ABORT   -reason "$tee_log_msg(UC)"\
                           -tc_def $tc_def
	return 0 
	}

########################### START - INITIALIZATION ############################

set tee_port_1		            $::tee_port_num_1
set tee_session_id 	            $::session_id
set dut_port_1		            $::dut_port_num_1
set vlan_id		            $::ptp_vlan_id
set eth_type1		            $::eth_type1
set eth_type2		            $::eth_type2
set dest_mac                        $::dest_mac
set ann_interval                    $::ptp_announce_interval
set clock_step                      $::ptp_clock_step
set pdelay_mac                      $::ptp_delay_mac
set sequence_ann                    1
set correction_field                0
set sequence_sync                   16385
set sequence_del                    32768
set sequence_pdel                   49151
set src_port_general                "320"
set dst_port_general                "320"
set src_port_event                  "319"
set dst_port_event                  "319"
set unicast_flag                    1

set dut_ip                          $::dut_test_port_ip
set tee_ip                          $::tee_test_port_ip

set dut_dest_ip                     $::dut_test_port_ip
set tee_dest_ip                     $::tee_test_port_ip

set announceReceiptTimeoutInterval [expr (int(pow(2,$::ptp_announce_interval)) * $::ptp_announce_timeout ) + $::ptp_user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_req_timeout              [expr int(pow(2,( $::ptp_sync_interval +1))) +$::ptp_user_delay]
set response_timeout               [expr $::ptp_user_delay + $::ptp_timeout]
set arp_timeout                    $::ARP_timeout

########################### END - INITIALIZATION ##############################
ptp::dut_Initialization

proc cleanup { } {

ptp::dut_cleanup_setup_001

}

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock step as One-step/ two-step.                        #
#    iv.   Configure Transmission Type as Unicast.                            #
#     v.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"


    if {![ptp::dut_configure_setup_001 \
                       -port_list          $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

	       ERRLOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
                                   -tc_def $tc_def
                return 0
        }


###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2 
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $::ptp_ethtype\
			   -vlan_id                 $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]

if {![pltLib::clear_rx_info \
                -session_id              $session_id\
                -port_num                $tee_port_1]} {
                return 0
        }

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg   "$tee_log_msg(ARP_F1)"
        TEE_CLEANUP
        TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0



}

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 3 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC1                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z                                    #
#                  UnicastFlag         = 1                                    #
###############################################################################

sleep $::wait_time
STEP 3
LOG -level 0 -msg "$tee_log_msg(ANN_RX_UC_TRUE_OBS)"
### Creating filter - Receive packet ###

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -unicast_flag            $unicast_flag\
                           -reset_count             $::reset_count_off\
                           -message_type            $::msg_announce\
                           -domain_number           $::default_domain\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
                           -priority1               priority1\
                           -dut_mac                 dut_mac]} {
            TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }


set gmpriority1 [ expr $priority1 - 1 ]

###############################################################################
# Step 4 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x0B                                 #
#                  Domain Number       = DN1                                  #
#                  Priority1           = Z-1                                  #
###############################################################################

STEP 4
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"
	
      if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
			   -dest_mac                $dut_mac\
			   -gmpriority1             $gmpriority1\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
                           -unicast_flag            $unicast_flag\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
			   -domain_number           $::default_domain\
                           -message_type            $::msg_announce\
                           -message_length          $::ann_len\
                           -sequence_id             $sequence_ann\
                           -gmid                    $::gmid]} {

            TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_TX_T1_F)"\
                              -tc_def $tc_def
		return 0
	}


pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 5a: Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x00                                 #
#                  Domain Number       = DN1                                  #
###############################################################################

STEP 5a
LOG -level 0 -msg "$tee_log_msg(SYN_TX_PERIOD_T1)"

if {![ptp::send_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_mac\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
                           -twostep_flag            $::twostep_flag\
		  	   -unicast_flag            $unicast_flag\
                           -message_length          $::sync_len\
			   -domain_number           $::default_domain\
                           -control_field           $::ctrl_sync\
                           -sequence_id             $sequence_sync]} {

            TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_TX_T1_F)"\
                              -tc_def $tc_def				   

		return 0
	}


###############################################################################
# Step 5b: If the clock is two step clock, send periodic FOLLOW_UP message    #
#          on port T1 with following parameters:                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x08                                 #
#                  Domain Number       = DN1                                  #
###############################################################################
	
if { $clock_step == "Two Step" } {
STEP 5b
LOG -level 0 -msg "$tee_log_msg(FOLL_TX_PERIOD_T1)"

        if {![ptp::send_followup\
			 -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_mac\
                         -eth_type                $eth_type1\
			 -vlan_id		  $vlan_id\
			 -type 		  	  $eth_type2\
		         -source_ip               $tee_ip\
		         -dest_ip	          $dut_dest_ip\
			 -src_port		  $src_port_general\
			 -dest_port		  $dst_port_general\
		  	 -unicast_flag            $unicast_flag\
                         -message_length          $::followup_len\
			 -domain_number           $::default_domain\
                         -control_field           $::ctrl_followup\
                         -sequence_id             $sequence_sync]} {

            TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_TX_T1_F)"\
                              -tc_def $tc_def				 

		return 0 
	
		}
}

###############################################################################
# Step 6 : Verify that DUT transmits DELAY_REQ message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              Ethernet Header                                                #
#                  Destination MAC     = UC2                                  #
#              PTP Header                                                     #
#                  Message Type        = 0x01                                 #
#                  Domain Number       = DN1                                  #
#                  UnicastFlag         = 1                                    #
###############################################################################

STEP 6
LOG -level 0 -msg "$tee_log_msg(DREQ_RX_UC_TRUE_V)"

	if {![ptp::recv_delayreq \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $dut_mac\
                           -dest_mac                $tee_mac\
                           -eth_type                $eth_type1\
  			   -vlan_id		    $vlan_id\
		    	   -type 		    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
			   -reset_count		    $::reset_count_on\
			   -ucast_flag              ucast_flag\
                           -message_type            $::msg_delayreq\
                           -domain_number           $::default_domain\
			   -timeout		    $delay_req_timeout\
                           -filter_id               $filter_id]} {
 
	       TEE_CLEANUP
               TC_CLEAN_AND_FAIL -reason "$tee_log_msg(DREQ_RX_P1_F)"\
                                  -tc_def $tc_def

return 0
}
if { $ucast_flag == 1 } {
        TEE_CLEANUP
        TC_CLEAN_AND_PASS -reason  "$tee_log_msg(DREQ_RX_UC_TRUE_P1)"\
                                  -tc_def $tc_def
        return 1
} else {
        TEE_CLEANUP
        TC_CLEAN_AND_FAIL -reason  "$tee_log_msg(DREQ_RX_UC_FLSE_P1)"\
                                  -tc_def $tc_def
        return 1
}

