#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_smg_002                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP State Machine Group (SMG)                           #
#                                                                             #
# Title             : PTP portState - DISABLED                                #
#                                                                             #
# Purpose           : Verify that the boundary clock does not transmit any PTP#
#                     messages on its port in DISABLED state.                 #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.2.5 Table 10 Page 77            #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                     <Clock mode -one-step/two-step> |           #
#           |                                                     |           #
#           |                                        <Enable PTP> | P1        #
#           |                                                     |           #
#           |        <Wait for Announce Receipt Timeout Interval> |           #
#           |                                                     |           #
#           |                    <Check PTP port status - MASTER> | P1        #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                       <Disable PTP> | P1        #
#           |                                                     |           #
#           |                  <Check PTP port status - DISABLED> | P1        #
#           |                                                     |           #
#           |                       PTP MSG [IEEE 802.3/ UDP-IPv4]|           #
#        T1 |                      XX-------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock mode as One-step/ two-Step.                        #
#    iv.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Wait for expiry of announce receipt timeout interval.              #
#                                                                             #
# Step 4 : Check whether DUT's PTP port status P1 is in master state.         #
#                                                                             #
# Step 5 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 6 : Disable PTP on DUT's port P1.                                      #
#                                                                             #
# Step 7 : Verify that the DUT's port P1 is in DISABLED state.                #
#                                                                             #
# Step 8 : Verify that DUT does not transmit PTP message on Port P1.          #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx            Initial                         #
#                                                                             #
# 1.1          Jun/2017      Veryx            1.Ladder updated (CR ID - 2451).#
#                                             2.The Unwanted display of LOG   #
#                                               print is cleared(CR ID - 2431)#
###############################################################################
# Copyright (C) 2017 Veryx Technologies(P) Ltd.                               #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id
#######################  LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock does not transmit any PTP\
                     messages on its port in DISABLED state."


########################### START - INITIALIZATION ############################
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout       $::ptp_timeout
set timeout           [expr $ptp_timeout + $user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout [expr  int(pow(2,( $::ptp_sync_interval +1))) +$::ptp_user_delay] 
set port_list              $::dut_port_num_1
set clock_mode             $::ptp_dut_clock_mode
set domain                 $::ptp_dut_clock_domain
set priority1              $::ptp_dut_clock_priority1
set delay_mechanism        $::ptp_dut_delay_mechanism
set clock_step             $::ptp_dut_clock_step
set tee_port_1		   $::tee_port_num_1
set tee_session_id 	   $::session_id
set dut_port_1		   $::dut_port_num_1
set vlan_id		   $::ptp_vlan_id
set eth_type1		   $::eth_type1
set eth_type2		   $::eth_type2
set dest_mac               $::dest_mac
set dut_ip                 $::dut_test_port_ip
set tee_ip                 $::tee_test_port_ip
set unicast_flag           0
set src_port_general       "320"
set dst_port_general       "320"
set src_port_event         "319"
set dst_port_event         "319"
set arp_timeout            $::ARP_timeout
set dut_dest_ip            $::dut_test_port_ip
set tee_dest_ip            $::tee_test_port_ip
#message Type
set msg_sync               $::msg_sync
set msg_delayreq           $::msg_delayreq
set msg_pdelayreq          $::msg_pdelayreq
set msg_pdelayresp         $::msg_pdelayresp
set msg_followup           $::msg_followup
set msg_delayresp          $::msg_delayresp
set msg_pdelayrespfoll     $::msg_pdelayrespfoll
set msg_announce           $::msg_announce
#Control Field
set ctrl_sync              $::ctrl_sync
set ctrl_delayreq          $::ctrl_delayreq
set ctrl_followup          $::ctrl_followup
set ctrl_delayresp         $::ctrl_delayresp
set ctrl_management        $::ctrl_management
set ctrl_announce          $::ctrl_announce
set ctrl_pdelayreq         $::ctrl_pdelayreq
set ctrl_pdelayresp        $::ctrl_pdelayresp
set ctrl_pdelayrespfoll    $::ctrl_pdelayrespfoll
#message length
set ann_len                $::ann_len
set sync_len               $::sync_len
set followup_len           $::followup_len
set delayreq_len           $::delayreq_len
set delayresp_len          $::delayresp_len
set pdelayreq_len          $::pdelayreq_len
set pdelayresp_len         $::pdelayresp_len
set pdelayrespfoll_len     $::pdelayrespfoll_len

set gmid                   $::gmid
set domain_no              $::default_domain
set reset_count_on         $::reset_count_on
set reset_count_off        $::reset_count_off
set ptp_ethtype            $::ptp_ethtype
set tee_dest_mac           $::dest_mac
set dut_dest_mac           $::dest_mac
set Disable                $::DUT_PTP_STATE_DISABLED
set Master                 $::DUT_PTP_STATE_MASTER
ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock mode as One-step/ two-Step.                        #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"

      if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


    

###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]


if {$::ptp_communication_model == "Unicast" && $::ptp_transport_type == "UDP/IPv4"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg "$tee_log_msg(ARP_F1)"
         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set tee_dest_ip    $tee_ip
set dut_dest_ip    $dut_ip
set unicast_flag   1

} else {

set dut_mac "dont_care"
set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
}

set session_handler $::dut_session_handler
      
sleep $::wait_time

###############################################################################
# Step 3 : Wait for expiry of announce receipt timeout interval.              #
###############################################################################
STEP 3 
LOG -level 0 -msg "$tee_log_msg(WAIT_ANN_RX_INTR)"

sleep $announceReceiptTimeoutInterval


###############################################################################	
# Step 4 : Check whether DUT's PTP port status P1 is in master state.         #
###############################################################################
STEP 4 
LOG -level 0 -msg "Check whether DUT's PTP  Port status  P1  is in Master state"
if {![ptp::dut_check_state \
                    -session_handler         $session_id\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $Master] } {

        TEE_CLEANUP
	TC_CLEAN_AND_ABORT -reason "DUT Port P1  is not in Master state"\
			   -tc_def $tc_def
return 0 
}

LOG -level 0 -msg "DUT Port P1 is in master state"
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 5 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
###############################################################################           
STEP 5 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"

if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {

# To clear the periodic send / recieve function	
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }




###############################################################################
# Step 6 : Disable PTP on DUT's port P1.                                      #
###############################################################################
STEP 6
LOG -level 0 -msg "Disable PTP on DUT's port P1\n"

if {![ptp::dut_port_ptp_disable\
                      -session_handler     $session_handler\
                      -port_list           $dut_port_1]} {

                  TEE_CLEANUP
                  TC_CLEAN_AND_ABORT   -reason "Unable to Disable PTP on DUT port P1."\
                                       -tc_def $tc_def

           return 0
   } 
         

###############################################################################	
# Step 7 : Verify that the DUT's port P1 is in DISABLED state.                #
###############################################################################
STEP 7
LOG -level 0 -msg "Verify that the DUT's port P1 is in DISABLED state\n"

if {![ptp::dut_check_state \
                    -session_handler         $session_handler\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $Disable] } {

        TEE_CLEANUP
	TC_CLEAN_AND_ABORT -reason "DUT Port P1  is not in DISABLE state"\
			   -tc_def $tc_def
return 0 
}
LOG -level 0 -msg "DUT Port P1 is in DISABLE state"
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 8 : Verify that DUT does not transmit PTP message on Port P1.          #
###############################################################################           

STEP 8
LOG -level 0 -msg "Verify that DUT does not transmit PTP  message on the port P1\n"

# To clear the periodic send / recieve function	
set ::semlock "LOCKED"
if {![ptp::recv_ptpmessage \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -filter_id               $filter_id]} {
set ::semlock "UNLOCKED"
           TEE_CLEANUP
           TC_CLEAN_AND_PASS -reason "DUT does not transmit any PTP  messages in DISABLED STATE"\
                              -tc_def $tc_def

       return 1
       }
set ::semlock "UNLOCKED"
TEE_CLEANUP
TC_CLEAN_AND_FAIL -reason "DUT transmits PTP  messages in DISABLED STATE "\
                  -tc_def $tc_def

return 0
