#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_smg_003                                  #
# Test Case Version : 1.0                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP State Machine Group (SMG)                           #
#                                                                             #
# Title             : PTP portState - DISABLED                                #
#                                                                             #
# Purpose           : Verify that the boundary clock discards any PTP messages#
#                     that are received on its port in DISABLED state.        #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.2.5 Table 10 Page 77            #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                                                     |           #
#           |                                        <Enable PTP> | P1        #
#           |                                                     |           #
#           |        <Wait for Announce Receipt Timeout Interval> |           #
#           |                                                     |           #
#           |                 <Check  PTP's port status = MASTER> | P1        #
#           |                                                     |           #
#           |                          ANNOUNCE [MSG_TYPE = 0x0B, |           #
#           |                                          DN = DN1]  |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                              SYNC [MSG_TYPE = 0x00, |           #
#           |                               DN = DN1, SEQ_ID = Y] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                         FOLLOW_UP [MSG_TYPE = 0x08, |           #
#           |                               DN = DN1, SEQ_ID = Y] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                       <Disable PTP> | P1        #
#           |                                                     |           #
#           |               <Check  PTP's port status = DISABLED> | P1        #
#           |                                                     |           #
#           | DELAY_REQ [MSG_TYPE = 0x01,                         |           #
#           |            DN = DN1, SEQ_ID = C]                    |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                        DELAY_RESP [MSG_TYPE = 0x09, |           #
#           |                               DN = DN1, SEQ_ID = C] |           #
#        T1 |                      XX-------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Wait for expiry of announce receipt timeout interval.              #
#                                                                             #
# Step 4 : Check whether DUT's PTP port P1 status is in MASTER state.         #
#                                                                             #
# Step 5 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 6a: Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                                                                             #
# Step 6b: If the clock is two step,observe that DUT transmits FOLLOW_UP      #
#          message on the port P1 with following parameters:                  #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
#                                                                             #
# Step 7 : Disable PTP on port P1.                                            #
#                                                                             #
# Step 8 : Check whether DUT's PTP port P1 status is in DISABLED state.       #
#                                                                             #
# Step 9 : Send DELAY_REQ message on port T1 with following parameters.       #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x01                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = C                                          #
#                                                                             #
# Step 10: Verify that DUT does not transmit DELAY_RESP message on port P1    #
#          with following parameters:                                         #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x01                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = C                                          #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx            Initial                         #
###############################################################################
# Copyright (C) 2017 Veryx Technologies(P) Ltd.                               #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id
#######################  LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock discards any PTP messages\
                     that are received on its port in DISABLED state."


########################### START - INITIALIZATION ############################
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout       $::ptp_timeout
set response_timeout           [expr $ptp_timeout + $user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout [expr  int(pow(2,( $::ptp_sync_interval +1))) +$::ptp_user_delay] 

set port_list              $::dut_port_num_1
set clock_mode             $::ptp_dut_clock_mode
set domain                 $::ptp_dut_clock_domain
set priority1              $::ptp_dut_clock_priority1
set delay_mechanism        $::ptp_dut_delay_mechanism
set clock_step             $::ptp_dut_clock_step
set tee_port_1		   $::tee_port_num_1
set tee_session_id 	   $::session_id
set dut_port_1		   $::dut_port_num_1
set vlan_id		   $::ptp_vlan_id
set eth_type1		   $::eth_type1
set eth_type2		   $::eth_type2
set dest_mac               $::dest_mac
set dut_ip                 $::dut_test_port_ip
set tee_ip                 $::tee_test_port_ip
set unicast_flag           0
set src_port_general       "320"
set dst_port_general       "320"
set src_port_event         "319"
set dst_port_event         "319"
set sequence_ann           1
set sequence_sync          16385
set sequence_foll          16385
set sequence_del           49151
set arp_timeout            $::ARP_timeout
set dut_dest_ip            $::dut_test_port_ip
set tee_dest_ip            $::tee_test_port_ip
#message Type
set msg_sync               $::msg_sync
set msg_delayreq           $::msg_delayreq
set msg_pdelayreq          $::msg_pdelayreq
set msg_pdelayresp         $::msg_pdelayresp
set msg_followup           $::msg_followup
set msg_delayresp          $::msg_delayresp
set msg_pdelayrespfoll     $::msg_pdelayrespfoll
set msg_announce           $::msg_announce
#Control Field
set ctrl_sync              $::ctrl_sync
set ctrl_delayreq          $::ctrl_delayreq
set ctrl_followup          $::ctrl_followup
set ctrl_delayresp         $::ctrl_delayresp
set ctrl_management        $::ctrl_management
set ctrl_announce          $::ctrl_announce
set ctrl_pdelayreq         $::ctrl_pdelayreq
set ctrl_pdelayresp        $::ctrl_pdelayresp
set ctrl_pdelayrespfoll    $::ctrl_pdelayrespfoll
#message length
set ann_len                $::ann_len
set sync_len               $::sync_len
set followup_len           $::followup_len
set delayreq_len           $::delayreq_len
set delayresp_len          $::delayresp_len
set pdelayreq_len          $::pdelayreq_len
set pdelayresp_len         $::pdelayresp_len
set pdelayrespfoll_len     $::pdelayrespfoll_len

set gmid                   $::gmid
set domain_no              $::default_domain
set reset_count_on         $::reset_count_on
set reset_count_off        $::reset_count_off
set ptp_ethtype            $::ptp_ethtype
set tee_dest_mac           $::dest_mac
set dut_dest_mac           $::dest_mac
set Disable                $::DUT_PTP_STATE_DISABLED
set Master                 $::DUT_PTP_STATE_MASTER
ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"

      if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


    

###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]


if {$::ptp_communication_model == "Unicast" && $::ptp_transport_type == "UDP/IPv4"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg "$tee_log_msg(ARP_F1)"
         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

set dut_mac "dont_care"
set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
}

if {[info exists param(::dut_session_handler)]} {
          set session_handler $::dut_session_handler
      } else {
          set session_handler $::dut_session_handler
      }


sleep $::wait_time
###############################################################################
# Step 3 : Wait for expiry of announce receipt timeout interval.              #
###############################################################################
STEP 3 
LOG -level 0 -msg "$tee_log_msg(WAIT_ANN_RX_INTR)"

sleep $announceReceiptTimeoutInterval

###############################################################################	
# Step 4 : Check whether DUT's PTP port P1 status is in MASTER state.         #
###############################################################################
STEP 4
LOG -level 0 -msg "Check  whether  DUT's port P1  is in Master state"

if {![ptp::dut_check_state \
                    -session_handler         $session_id\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $Master] } {
        TEE_CLEANUP
	TC_CLEAN_AND_ABORT -reason "DUT Port P1  is not in Master state"\
			   -tc_def $tc_def
return 0 
}  
LOG -level 0 -msg "DUT Port P1  is in Master state"

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################
# Step 5 : Observe that DUT transmits ANNOUNCE message on the port P1 with    #
#          following parameters:                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
###############################################################################           
STEP 5 
LOG -level 0 -msg "$tee_log_msg(ANN_RX_OBS)"
 if {![ptp::recv_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip		    $tee_dest_ip\
			   -src_port		    $src_port_general\
			   -dest_port               $dst_port_general\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_announce\
                           -domain_number           $domain_no\
                           -timeout                 $announceReceiptTimeoutInterval\
			   -priority1		    priority1\
                           -filter_id               $filter_id\
                           -dut_mac                 dut_mac]} {

# To clear the periodic send / recieve function	
           TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANN_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }
###############################################################################
# Step 6a: Observe that DUT transmits SYNC message on the port P1 with        #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
###############################################################################
STEP 6a 
LOG -level 0 -msg "$tee_log_msg(SYN_RX_OBS)"
if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac              $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
	                   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip   	            $tee_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
			   -reset_count		    $reset_count_off\
                           -message_type            $msg_sync\
                           -domain_number           $domain_no\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {
# To clear the periodic send / recieve function	
           
        TEE_CLEANUP
           TC_CLEAN_AND_ABORT  -reason "$tee_log_msg(SYN_RX_P1_F)"\
                               -tc_def $tc_def

       return 0
       }


###############################################################################
# Step 6b: If the clock is two step,observe that DUT transmits FOLLOW_UP      #
#          message on the port P1 with following parameters:                  #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = Y                                          #
###############################################################################

if { $clock_step == "Two Step" } {
STEP 6b 
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_P1_OBS)"

      if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
			   -source_mac		    $dut_mac\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
	                   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $dut_ip\
			   -dest_ip   	            $tee_dest_ip\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_followup\
                           -domain_number           $domain_no\
                           -timeout                 $sync_timeout\
			   -reset_count		    $reset_count_off\
                           -filter_id               $filter_id]} {
# To clear the periodic send / recieve function	
          
        TEE_CLEANUP
          TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }
}

###############################################################################
# Step 7 : Disable PTP on port P1.                                            #
###############################################################################
STEP 7
LOG -level 0 -msg "Disable PTP on DUT's port P1"

if {![ptp::dut_port_ptp_disable\
                      -session_handler     $session_handler\
                      -port_list           $port_list]} {

        TEE_CLEANUP
                  TC_CLEAN_AND_ABORT   -reason "Unable to disable PTP on DUT port P1 ($::dut_port_num_1)"\
                                       -tc_def $tc_def

           return 0
   }

###############################################################################	
# Step 8 : Check whether DUT's PTP port P1 status is in DISABLED state.       #
###############################################################################
STEP 8
LOG -level 0 -msg "Check whether DUT's PTP port P1 is in DISABLED state"

if {![ptp::dut_check_state \
                    -session_handler         $session_id\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $Disable] } {
        TEE_CLEANUP
        TC_CLEAN_AND_FAIL -reason "DUT Port P1  is not in disable state"\
			   -tc_def $tc_def
return 0 
}
LOG -level 0 -msg "DUT Port P1  is in DISABLE state"
 
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
###############################################################################       
# Step 9 : Send DELAY_REQ message on port T1 with following parameters.       #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x01                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = C                                          #
###############################################################################
STEP 9
LOG -level 0 -msg "$tee_log_msg(DREQ_TX_T1)"
 
if {![ptp::send_delay_req\
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $tee_mac\
                        -dest_mac             $dut_dest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $tee_ip\
                        -dest_ip              $dut_dest_ip\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -unicast_flag         $unicast_flag\
                        -sequence_id          $sequence_del\
                        -message_type         $msg_delayreq\
                        -domain_number        $domain_no\
                        -control_field        $ctrl_delayreq]} {
                    TEE_CLEANUP 
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(DREQ_TX_T1_F)"\
                                         -tc_def $tc_def
       
       return 0

       }
###############################################################################
# Step 10: Verify that DUT does not transmit DELAY_RESP message on port P1    #
#          with following parameters:                                         #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x01                                       #
#                  Domain Number = DN1                                        #
#                  Sequence ID   = C                                          #
###############################################################################

STEP 10 
LOG -level 0 -msg "$tee_log_msg(DRESP_RX_P1_VER_NOT)"
if {![ptp::recv_delayresp \
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $dut_mac\
                        -dest_mac             $tee_dest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $dut_ip\
                        -dest_ip              $tee_dest_ip\
                        -src_port             $src_port_general\
                        -dest_port            $dst_port_general\
                        -message_type         $msg_delayresp\
			-message_length       $delayresp_len\
                        -domain_number        $domain_no\
                        -sequence_id          $sequence_del\
                        -reset_count          $reset_count_on\
                        -timeout              $response_timeout\
                        -filter_id            $filter_id\
			-control_field        $ctrl_delayresp]} {
			                        
	  TEE_CLEANUP	
          TC_CLEAN_AND_PASS -reason "DUT discards any PTP messages that are received on its port in DISABLED state"\
                              -tc_def $tc_def

       return 1 
       } else {

          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "DUT does not discard PTP messages in DISABLED state"\
		            -tc_def $tc_def
 
              
     return 0
   }

