#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_smg_004                                  #
# Test Case Version : 1.2                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP State Machine Group (SMG)                           #
#                                                                             #
# Title             : PTP portState - LISTENING                               #
#                                                                             #
# Purpose           : Verify that the boundary clock port in LISTENING state  #
#                     shall wait for the Announce receipt timeout to expire   #
#                     and transit to MASTER state.                            #
#                                                                             #
# Reference         : IEEE 1588-2008 Clause 9.2.5 Table 10 Page 77, Figure 23 #
#                     Page 78                                                 #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                                                     |           #
#           |            <Configure Announce Receipt Timeout = X> | P1        #
#           |                                                     |           #
#           |                                        <Enable PTP> | P1        #
#           |                                                     |           #
#           |                     <Check port status = LISTENING> | P1        #
#           |                                                     |           #
#           |        <Wait for Announce Receipt Timeout Interval> |           #
#           |                                                     |           #
#           |                        <Check port status = MASTER> | P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure AnnounceReceiptTimeout as X.                             #
#     v.   Configure PTP on port P1.                                          #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
#                                                                             #
# Step 3 : Check whether the port status of port P1 is in LISTENING.          #
#                                                                             #
# Step 4 : Wait for expiry of announce receipt timeout interval.              #
#                                                                             #
# Step 5 : Verify whether the DUT port P1 is in MASTER state.                 #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jan/2017      Veryx         Initial                            #
#                                                                             #
# 1.1          Jun/2017      Veryx         Replaced wait_time with            #
#                                          listening_wait (CR ID -2433).      #
#                                                                             #
# 1.2          Oct/2018      Veryx         Removed the wait before expecting  #
#                                          the PTP state to be 'LISTENING'    #
#                                          (CR: 18268).                       #
###############################################################################
# Copyright (C) 2017-2018 Veryx Technologies(P) Ltd.                          #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id
#######################  LOCAL VARIABLES ARE DECLARED HERE #####################
set tc_def      "Verify that the boundary clock port in LISTENING state\
                 shall wait for the Announce receipt timeout to expire\
                 and transit to MASTER state."


########################### START - INITIALIZATION ############################
set user_delay            $::ptp_user_delay
set announce_timeout      $::ptp_announce_timeout
set announce_interval     $::ptp_announce_interval
set announce_interval     [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout           $::ptp_timeout
set timeout               [expr $ptp_timeout + $user_delay]
set sync_timeout          [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout         [expr  int(pow(2,( $::ptp_sync_interval +1))) +$::ptp_user_delay] 

set port_list              $::dut_port_num_1
set clock_mode             $::ptp_dut_clock_mode
set domain                 $::ptp_dut_clock_domain
set priority1              $::ptp_dut_clock_priority1
set announce_interval      $::ptp_announce_interval
set announce_timeout       $::ptp_announce_timeout
set delay_mechanism        $::ptp_dut_delay_mechanism
set clock_step             $::ptp_dut_clock_step
set tee_port_1		   $::tee_port_num_1
set tee_session_id 	   $::session_id
set dut_port_1		   $::dut_port_num_1
set vlan_id		   $::ptp_vlan_id
set eth_type1		   $::eth_type1
set eth_type2		   $::eth_type2
set dest_mac               $::dest_mac
set dut_ip                 $::dut_test_port_ip
set tee_ip                 $::tee_test_port_ip
set arp_timeout            $::ARP_timeout
set dut_dest_ip            $::dut_test_port_ip
set tee_dest_ip            $::tee_test_port_ip
#message Type
set msg_sync               $::msg_sync
set msg_delayreq           $::msg_delayreq
set msg_pdelayreq          $::msg_pdelayreq
set msg_pdelayresp         $::msg_pdelayresp
set msg_followup           $::msg_followup
set msg_delayresp          $::msg_delayresp
set msg_pdelayrespfoll     $::msg_pdelayrespfoll
set msg_announce           $::msg_announce
#Control Field
set ctrl_sync              $::ctrl_sync
set ctrl_delayreq          $::ctrl_delayreq
set ctrl_followup          $::ctrl_followup
set ctrl_delayresp         $::ctrl_delayresp
set ctrl_management        $::ctrl_management
set ctrl_announce          $::ctrl_announce
set ctrl_pdelayreq         $::ctrl_pdelayreq
set ctrl_pdelayresp        $::ctrl_pdelayresp
set ctrl_pdelayrespfoll    $::ctrl_pdelayrespfoll
#message length
set ann_len                $::ann_len
set sync_len               $::sync_len
set followup_len           $::followup_len
set delayreq_len           $::delayreq_len
set delayresp_len          $::delayresp_len
set pdelayreq_len          $::pdelayreq_len
set pdelayresp_len         $::pdelayresp_len
set pdelayrespfoll_len     $::pdelayrespfoll_len

set gmid                   $::gmid
set domain_no              $::default_domain
set reset_count_on         $::reset_count_on
set reset_count_off        $::reset_count_off
set ptp_ethtype            $::ptp_ethtype

set tee_dest_mac           $::dest_mac
set dut_dest_mac           $::dest_mac
set LISTENING              $::DUT_PTP_STATE_LISTENING
set Master                 $::DUT_PTP_STATE_MASTER
ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as Boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure AnnounceReceiptTimeout as X.                             #
#     v.   Configure PTP on port P1.                                          #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"

      if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


    

###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]


if {$::ptp_communication_model == "Unicast" && $::ptp_transport_type == "UDP/IPv4"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg "$tee_log_msg(ARP_F1)"
         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
}

###############################################################################	
# Step 3 : Check whether the port status of port P1 is in LISTENING.          #
###############################################################################
STEP 3
LOG -level 0 -msg "Check the DUT Port P1 is in LISTENING state\n"

if {![ptp::dut_check_state \
                    -session_handler         $session_id\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $LISTENING] } {
        TEE_CLEANUP
	TC_CLEAN_AND_ABORT -reason "DUT's Port P1 is not in Listening state"\
			   -tc_def $tc_def
return 0 
}

LOG -level 0 -msg "DUT Port P1 is in LISTENING state"

###############################################################################
# Step 4 : Wait for expiry of announce receipt timeout interval.              #
###############################################################################
STEP 4 
LOG -level 0 -msg "$tee_log_msg(WAIT_ANN_RX_INTR)"

sleep $announceReceiptTimeoutInterval


###############################################################################	
# Step 5 : Verify whether the DUT port P1 is in MASTER state.                 #
###############################################################################
STEP 5 
LOG -level 0 -msg "Verify whether the DUT port P1 is in MASTER state\n"

if {![ptp::dut_check_state \
                    -session_handler         $session_id\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $Master] } {
        TEE_CLEANUP
        TC_CLEAN_AND_FAIL -reason "DUT's Port P1 does not move from LISTENING state to MASTER state after the expiry of Announce receipt timeout interval"\
			   -tc_def $tc_def
return 0 
}

TEE_CLEANUP
TC_CLEAN_AND_PASS -reason "DUT's Port P1 moves from LISTENING state to MASTER state after the expiry of Announce receipt timeout interval"\
                  -tc_def $tc_def

return 1
