#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_smg_014                                  #
# Test Case Version : 1.0                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP State Machine Group (SMG)                           #
#                                                                             #
# Title             : PTPPortState - MASTER - DISABLED                        #
#                                                                             #
# Purpose           : Verify that the boundary clock in MASTER state moves to #
#                     DISABLED state when PTP is disabled at port.            #
#                                                                             #
# Reference         : IEEE 1588-2008 Figure 23 Page 78                        #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                           <Configure Priority1 = X> |           #
#           |                                        <Enable PTP> | P1        #
#           |                                                     |           #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=X+1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                        <Check port status = MASTER> | P1        #
#           |                                                     |           #
#           |                              SYNC [MSG_TYPE = 0x00, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                          <If Clock Step = Two Step, |           #
#           |                             then receive FOLLOW_UP> |           #
#           |                                                     |           #
#           |                         FOLLOW_UP [MSG_TYPE = 0x08, |           #
#           |                                           DN = DN1] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |                                       <Disable PTP> | P1        #
#           |                                                     |           #
#           |                      <Check port status = DISABLED> | P1        #
#           |                                                     |           #
#           |                       PTP MSG [IEEE 802.3/ UDP-IPv4]|           #
#        T1 |                      XX-------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     PRT1      = Priority1                                                   #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure Priority1 as X (Default : 128).                          #
#     v.   Enable PTP on port P1.                                             #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = X+1                                        #
#                                                                             #
# Step 4 : Check whether the DUT port P1 is in MASTER state.                  #
#                                                                             #
# Step 5a: Verify that DUT transmits SYNC message on the port P1 with         #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 5b: If the clock is two step, verify that DUT transmits FOLLOW_UP      #
#          message on the port P1 with following parameters:                  #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 6 : Disable PTP on port P1.                                            #
#                                                                             #
# Step 7 : Verify whether the DUT port P1 is in DISABLED state.               #
#                                                                             #
# Step 8 : Verify that DUT does not transmit any PTP message on port P1.      #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jun/2017    Veryx            Initial                           #
###############################################################################
# Copyright (C) 2017 Veryx Technologies(P) Ltd.                               #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id
#######################  LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def      "Verify that the boundary clock in MASTER state moves to\
                 DISABLED state when PTP is disabled at port."

########################### START - INITIALIZATION ############################
set user_delay         $::ptp_user_delay
set announce_timeout   $::ptp_announce_timeout
set announce_interval  $::ptp_announce_interval
set announce_interval  [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout       $::ptp_timeout
set timeout           [expr $ptp_timeout + $user_delay]
set sync_timeout                   [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout [expr  int(pow(2,( $::ptp_sync_interval +5))) +$::ptp_user_delay] 
set port_list              $::dut_port_num_1
set clock_mode             $::ptp_dut_clock_mode
set domain                 $::ptp_dut_clock_domain
set priority1              $::ptp_dut_clock_priority1
set delay_mechanism        $::ptp_dut_delay_mechanism
set clock_step             $::ptp_dut_clock_step
set tee_port_1		   $::tee_port_num_1
set tee_session_id 	   $::session_id
set dut_port_1		   $::dut_port_num_1
set vlan_id		   $::ptp_vlan_id
set eth_type1		   $::eth_type1
set eth_type2		   $::eth_type2
set dest_mac               $::dest_mac
set dut_ip                 $::dut_test_port_ip
set tee_ip                 $::tee_test_port_ip
set unicast_flag           0
set src_port_general       "320"
set dst_port_general       "320"
set src_port_event         "319"
set dst_port_event         "319"
set sequence_ann           1
set sequence_sync          16385
set sequence_foll          16385
set sequence_del           49151
set arp_timeout            $::ARP_timeout
set dut_dest_ip            $::dut_test_port_ip
set tee_dest_ip            $::tee_test_port_ip
#message Type
set msg_sync               $::msg_sync
set msg_delayreq           $::msg_delayreq
set msg_pdelayreq          $::msg_pdelayreq
set msg_pdelayresp         $::msg_pdelayresp
set msg_followup           $::msg_followup
set msg_delayresp          $::msg_delayresp
set msg_pdelayrespfoll     $::msg_pdelayrespfoll
set msg_announce           $::msg_announce
#Control Field
set ctrl_sync              $::ctrl_sync
set ctrl_delayreq          $::ctrl_delayreq
set ctrl_followup          $::ctrl_followup
set ctrl_delayresp         $::ctrl_delayresp
set ctrl_management        $::ctrl_management
set ctrl_announce          $::ctrl_announce
set ctrl_pdelayreq         $::ctrl_pdelayreq
set ctrl_pdelayresp        $::ctrl_pdelayresp
set ctrl_pdelayrespfoll    $::ctrl_pdelayrespfoll
#message length
set ann_len                $::ann_len
set sync_len               $::sync_len
set followup_len           $::followup_len
set delayreq_len           $::delayreq_len
set delayresp_len          $::delayresp_len
set pdelayreq_len          $::pdelayreq_len
set pdelayresp_len         $::pdelayresp_len
set pdelayrespfoll_len     $::pdelayrespfoll_len

set gmid                   $::gmid
set domain_no              $::default_domain
set reset_count_on         $::reset_count_on
set reset_count_off        $::reset_count_off
set ptp_ethtype            $::ptp_ethtype
set tee_dest_mac           $::dest_mac
set dut_dest_mac           $::dest_mac
set Disable                $::DUT_PTP_STATE_DISABLED
set Master                 $::DUT_PTP_STATE_MASTER
ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_001

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure Priority1 as X (Default : 128).                          #
#     v.   Enable PTP on port P1.                                             #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"

      if {![ptp::dut_configure_setup_001 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
				   -tc_def $tc_def
		return 0
	}


    

###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]


if {$::ptp_communication_model == "Unicast" && $::ptp_transport_type == "UDP/IPv4"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg "$tee_log_msg(ARP_F1)"
         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set tee_dest_ip    $tee_ip
set dut_dest_ip    $dut_ip
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
}

set session_handler $::dut_session_handler
sleep $::wait_time
set gm_priority1 [expr $::ptp_dut_clock_priority1 + 1 ]
###############################################################################	
# Step 3 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = X+1                                        #
###############################################################################
	
STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"
if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
                           -domain_number           $::default_domain\
                           -message_length          $ann_len\
                           -gmid                    $::gmid\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $gm_priority1]} {
	            TEE_CLEANUP	
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ANN_TX_T1_F)"\
                                         -tc_def $tc_def

		return 0
	}
sleep $announceReceiptTimeoutInterval
###############################################################################	
# Step 4 : Check whether the DUT port P1 is in MASTER state.                  #
###############################################################################
STEP 4 
LOG -level 0 -msg "$dut_log_msg(CHK_PTP_PORT_MAS_STATE)"
if {![ptp::dut_check_state \
                    -session_handler         $session_id\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $Master] } {

        TEE_CLEANUP
	TC_CLEAN_AND_ABORT -reason "$dut_log_msg(PTP_PORT_MAS_STATE_F)"\
			   -tc_def $tc_def
return 0 
}

LOG -level 0 -msg "$dut_log_msg(PTP_PORT_MAS_STATE_P)"
if { $clock_step == "One Step" } {
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

}
###############################################################################
# Step 5a: Verify that DUT transmits SYNC message on the port P1 with         #
#          following parameters.                                              #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
###############################################################################

STEP 5a 
LOG -level 0 -msg "$tee_log_msg(SYN_RX_VER)"

if {![ptp::recv_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_event\
                           -dest_port               $dst_port_event\
                           -reset_count             $::reset_count_on\
                           -message_type            $::msg_sync\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

           TEE_CLEANUP
           TC_CLEAN_AND_FAIL -reason "$tee_log_msg(SYN_RX_P1_F)"\
                             -tc_def $tc_def

       return 0
       }
###############################################################################
# Step 5b: If the clock is two step, verify that DUT transmits FOLLOW_UP      #
#          message on the port P1 with following parameters:                  #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
###############################################################################


if { $clock_step == "Two Step" } {
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

STEP 5b 
LOG -level 0 -msg "$tee_log_msg(FOLL_RX_P1_VER)"

if {![ptp::recv_followup \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -dest_mac                $tee_dest_mac\
                           -eth_type                $eth_type1\
                           -vlan_id                 $vlan_id\
                           -type                    $eth_type2\
                           -source_ip               $dut_ip\
                           -dest_ip                 $tee_dest_ip\
                           -src_port                $src_port_general\
                           -dest_port               $dst_port_general\
                           -reset_count             $::reset_count_on\
                           -message_type            $::msg_followup\
                           -domain_number           $::default_domain\
                           -timeout                 $sync_timeout\
                           -filter_id               $filter_id]} {

          TEE_CLEANUP
          TC_CLEAN_AND_FAIL -reason "$tee_log_msg(FOLL_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }
}

###############################################################################
# Step 6 : Disable PTP on port P1.                                            #
###############################################################################
STEP 6
LOG -level 0 -msg "$dut_log_msg(PTP_DISABLE)\n"

if {![ptp::dut_port_ptp_disable\
                      -session_handler     $session_handler\
                      -port_list           $dut_port_1]} {

                  TEE_CLEANUP
                  TC_CLEAN_AND_ABORT   -reason "$dut_log_msg(PTP_DISABLE_F)."\
                                       -tc_def $tc_def

           return 0
   } 
         

###############################################################################	
# Step 7 : Verify whether the DUT port P1 is in DISABLED state.               #
###############################################################################
STEP 7
LOG -level 0 -msg "$dut_log_msg(CHK_PTP_PORT_DIS_STATE)\n"

if {![ptp::dut_check_state \
                    -session_handler         $session_handler\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $Disable] } {

        TEE_CLEANUP
	TC_CLEAN_AND_FAIL  -reason "$dut_log_msg(DUT_PRT_DIS_F)"\
			   -tc_def $tc_def
return 0 
}
LOG -level 0 -msg "$dut_log_msg(DUT_PRT_DIS)"
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################
# Step 8 : Verify that DUT does not transmit any PTP message on port P1.      #
###############################################################################           

STEP 8
LOG -level 0 -msg "$tee_log_msg(NO_PTP_MSG_V)"
sleep $announceReceiptTimeoutInterval
set ::semlock "LOCKED"
if {![ptp::recv_ptpmessage \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -filter_id               $filter_id]} {
set ::semlock "UNLOCKED"
           LOG -level 0 -msg "DUT does not transmit any PTP  messages on port P1."
           TEE_CLEANUP
           TC_CLEAN_AND_PASS -reason "DUT port P1  in MASTER state moves to DISABLED state when PTP is disabled at port and does not transmit any PTP messages on Port P1."\
                             -tc_def $tc_def

       return 1
       }
set ::semlock "UNLOCKED"
LOG -level 0 -msg "DUT transmits PTP  messages on port P1."
TEE_CLEANUP
TC_CLEAN_AND_FAIL -reason "DUT port P1 in MASTER state moves to DISABLED state when PTP is disabled at port but transmits PTP messages on Port P1."\
                  -tc_def $tc_def

return 0
