#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_ptp-bc_smg_018                                  #
# Test Case Version : 1.1                                                     #
# Component Name    : VERYX ATTEST PTP-BC CONFORMANCE TEST SUITE              #
# Module Name       : PTP State Machine Group (SMG)                           #
#                                                                             #
# Title             : PTPPortState - FAULTY - DISABLED                        #
#                                                                             #
# Purpose           : Verify that the boundary clock in FAULTY state moves to #
#                     DISABLED state when PTP is disabled at port.            #
#                                                                             #
# Reference         : IEEE 1588-2008 Figure 23 Page 78                        #
#                                                                             #
# Conformance Type  : SHALL                                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 002                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#           |                                                     |           #
#           |                    <Enable PTP with Boundary Clock> |           #
#           |                    <Clock step - one-step/two-step> |           #
#           |                           <Configure Priority1 = X> |           #
#           |                 <Mechanism - Peer Delay on Port P1> | P1        #
#           |                                        <Enable PTP> | P1        #
#           |                                                     |           #
#           |                                                     |           #
#           | ANNOUNCE [MSG_TYPE = 0x0B,                          |           #
#           |            DN = DN1,PRT1=X-1]                       |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |<Send Sync/ Sync & Follow-up msgs                    |           #
#           |based on clock step>                                 |           #
#           |                                                     |           #
#           |               PDELAY_REQ [MSG_TYPE = 0x02, DN = DN1,|           #
#           |                         SRC_PRT_ID = E, SEQ_ID = D] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | PDELAY_RESP [SRC_MAC = SRC1,CLK_ID = CLK1,          |           #
#           | REQ_PRT_ID = E, SEQ_ID = D                          |           #
#           | MSG_TYPE = 0x03, DN = DN1]                          |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           | PDELAY_RESP [SRC_MAC = SRC2,CLK_ID = CLK2,          |           #
#           | REQ_PRT_ID = E, SEQ_ID = D                          |           #
#           | MSG_TYPE = 0x03, DN = DN1]                          |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                        <Check port status = FAULTY> | P1        #
#           |                                                     |           #
#           |                                       <Disable PTP> | P1        #
#           |                                                     |           #
#           |               <Check  PTP's port status = DISABLED> | P1        #
#           |                                                     |           #
#           |                                                     |           #
#           |                       PTP MSG [IEEE 802.3/ UDP-IPv4]|           #
#        T1 |                      XX-------------------------<<--| P1        #
#           |                                                     |           #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     MSG_TYPE  = Message Type                                                #
#     DN        = Domain Number                                               #
#     SEQ_ID    = Sequence ID                                                 #
#     PRT1      = Priority1                                                   #
#     SRC_MAC   = Source mac address                                          #
#     CLK_ID    = Clock Identity                                              #
#     SRC_PRT_ID= Source Port Identity                                        #
#     REQ_PRT_ID= Requesting Port Identity                                    #
#                                                                             #
# NOTE :                                                                      #
#                                                                             #
#  1. Sequence ID pool:                                                       #
#        A = 1     - 16384 (ANNOUNCE)                                         #
#        B = 16385 - 32767 (SYNC)                                             #
#        C = 32768 - 49150 (DELAY_REQ)                                        #
#        D = 49151 - 65535 (PDELAY_REQ)                                       #
#                                                                             #
#  2. This Objective is verified using PTP profile. The profile specifies a   #
#     selection of PTP options and attributes that makes a device to work     #
#     without requiring user configuration.                                   #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure Priority1 as X (Default : 128).                          #
#     v.   Enable PTP on port P1.                                             #
#    vi.   Configure mechanism as peer delay mechanism on port P1.            #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = X-1                                        #
#                                                                             #
# Step 4a: Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 4b: If the clock is two step clock, send periodic FOLLOW_UP message on #
#          port T1 with following parameters:                                 #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
#                                                                             #
# Step 5 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters :                                             #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type              = 0x02                           #
#                  Domain Number             = DN1                            #
#                  Sequence ID               = D                              #
#                  Source Port Identity      = E                              #
#                                                                             #
# Step 6 : Send PDELAY_RESP on the port P1 and with following parameters:     #
#                                                                             #
#              PTP Header                                                     #
#                  Source Mac               = SRC1                            #
#                  Clock ID                 = CLK1                            #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
#                  Sequence Id              = D                               #
#                  Requesting Port Identity = E                               #
#                                                                             #
# Step 7 : Send PDELAY_RESP on the port P1 and with following parameters:     #
#                                                                             #
#              PTP Header                                                     #
#                  Source Mac               = SRC2                            #
#                  Clock ID                 = CLK2                            #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
#                  Sequence Id              = D                               #
#                  Requesting Port Identity = E                               #
#                                                                             #
# Step 8 : Check whether the DUT port P1 is in FAULTY state.                  #
#                                                                             #
# Step 9 : Disable PTP on port P1.                                            #
#                                                                             #
# Step 10: Verify whether the DUT port P1 is in DISABLED state.               #
#                                                                             #
# Step 11: Verify that DUT does not transmit any PTP message on port P1.      #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Jun/2017    Veryx            Initial                           #
# 1.1          Nov/2018    Veryx            Test case updated to be applicable#
#                                           only if the implementation        #
#                                           moves to FAULTY state on receiving#
#                                           multiple pDelay Resp messages.    #
#                                           Updated the wait timeout for      #
#                                           pDelay request (CR: 18275).       #
###############################################################################
# Copyright (C) 2013 - 2018 Veryx Technologies(P) Ltd.                        #
###############################################################################
global dut_log_msg
global tee_log_msg
global session_id
########################  LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def    "Verify that the boundary clock in FAULTY state moves to\
               DISABLED state when PTP is disabled at port."

########################### START - INITIALIZATION ############################
set user_delay             $::ptp_user_delay
set announce_timeout       $::ptp_announce_timeout
set announce_interval      $::ptp_announce_interval
set announce_interval      [expr int(pow(2,$announce_interval))]
set announceReceiptTimeoutInterval   [expr ($announce_interval * $announce_timeout) + $user_delay]
set ptp_timeout            $::ptp_timeout
set response_timeout       [expr $ptp_timeout + $user_delay]
set sync_timeout           [expr (pow(2,$::ptp_sync_interval)) + $::ptp_user_delay]
set delay_timeout          [expr  int(pow(2,( $::ptp_sync_interval +1))) +$::ptp_user_delay] 
set pdelay_timeout         [expr (2 * int(pow(2,($::ptp_pdelay_req_interval)))) + $::ptp_user_delay] 
set port_list              $::dut_port_num_1
set clock_mode             $::ptp_dut_clock_mode
set domain                 $::ptp_dut_clock_domain
set priority1              $::ptp_dut_clock_priority1
set delay_mechanism        $::ptp_dut_delay_mechanism
set clock_step             $::ptp_dut_clock_step
set tee_port_1		   $::tee_port_num_1
set tee_session_id 	   $::session_id
set dut_port_1		   $::dut_port_num_1
set vlan_id		   $::ptp_vlan_id
set eth_type1		   $::eth_type1
set eth_type2		   $::eth_type2
set dest_mac               $::dest_mac
set dut_ip                 $::dut_test_port_ip
set tee_ip                 $::tee_test_port_ip
set unicast_flag           0
set src_port_general       "320"
set dst_port_general       "320"
set src_port_event         "319"
set dst_port_event         "319"
set sequence_ann           1
set sequence_sync          16385
set sequence_foll          16385
set sequence_del           49151
set arp_timeout            $::ARP_timeout
set dut_dest_ip            $::dut_test_port_ip
set tee_dest_ip            $::tee_test_port_ip
#message Type
set msg_sync               $::msg_sync
set msg_delayreq           $::msg_delayreq
set msg_pdelayreq          $::msg_pdelayreq
set msg_pdelayresp         $::msg_pdelayresp
set msg_followup           $::msg_followup
set msg_delayresp          $::msg_delayresp
set msg_pdelayrespfoll     $::msg_pdelayrespfoll
set msg_announce           $::msg_announce
#Control Field
set ctrl_sync              $::ctrl_sync
set ctrl_delayreq          $::ctrl_delayreq
set ctrl_followup          $::ctrl_followup
set ctrl_delayresp         $::ctrl_delayresp
set ctrl_management        $::ctrl_management
set ctrl_announce          $::ctrl_announce
set ctrl_pdelayreq         $::ctrl_pdelayreq
set ctrl_pdelayresp        $::ctrl_pdelayresp
set ctrl_pdelayrespfoll    $::ctrl_pdelayrespfoll
#message length
set ann_len                $::ann_len
set sync_len               $::sync_len
set followup_len           $::followup_len
set delayreq_len           $::delayreq_len
set delayresp_len          $::delayresp_len
set pdelayreq_len          $::pdelayreq_len
set pdelayresp_len         $::pdelayresp_len
set pdelayrespfoll_len     $::pdelayrespfoll_len
set dut_ip                 $::dut_test_port_ip
set tee_ip                 $::tee_test_port_ip

set tee_pdest_mac          $::ptp_delay_mac
set dut_pdest_mac          $::ptp_delay_mac
set dut_dest_ip            $::dut_test_port_ip
set tee_dest_ip            $::tee_test_port_ip

set tee_dest_mac           $::dest_mac
set dut_dest_mac           $::dest_mac

set gmid                   $::gmid
set domain_no              $::default_domain
set reset_count_on         $::reset_count_on
set reset_count_off        $::reset_count_off
set ptp_ethtype            $::ptp_ethtype
set tee_dest_mac           $::dest_mac
set dut_dest_mac           $::dest_mac
set Uncalibrated           $::DUT_PTP_STATE_UNCALIBRATED
set Slave                  $::DUT_PTP_STATE_SLAVE
set Listen                 $::DUT_PTP_STATE_LISTENING
set Initialize             $::DUT_PTP_STATE_INITIALIZING
set Faulty                 $::DUT_PTP_STATE_FAULTY
set Disable                $::DUT_PTP_STATE_DISABLED

if { $::faulty_trig_by_mul_pdelresp != "true" } {

       LOG -level 0 -msg   "$tee_log_msg(FAULTY_IMP)"

       TC_ABORT -reason "$tee_log_msg(FAULTY_IMP)"
                -tc_def $tc_def
       return 0 
}

ptp::dut_Initialization
########################### END - INITIALIZATION ##############################
proc cleanup { } {

ptp::dut_cleanup_setup_002

}       

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Ensure that DUT port P1 is operational.                            #
#    ii.   Enable PTP globally with Device Type as boundary clock.            #
#   iii.   Configure clock mode as one-step/ two-Step.                        #
#    iv.   Configure Priority1 as X (Default : 128).                          #
#     v.   Enable PTP on port P1.                                             #
#    vi.   Configure mechanism as peer delay mechanism on port P1.            #
###############################################################################

STEP 1 
LOG -level 0 -msg "$dut_log_msg(INIT_SETUP2)"

      if {![ptp::dut_configure_setup_002 \
                       -port_list	   $dut_port_1\
                       -vm_mac             $::vm1_mac]} {

		LOG -msg "$dut_log_msg(INIT_SETUP2_F)"

		TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP2_F)"\
				   -tc_def $tc_def
		return 0
	}


    

###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP 2
LOG -level 0 -msg "$tee_log_msg(INIT)"

ptp::Initialization

set tee_mac            $::TEEMAC_1

if {![pltLib::create_filter_ptp_pkt\
                           -port_num                $tee_port_1\
                           -ether_type              $ptp_ethtype\
			   -vlan_id		    $vlan_id\
                           -ids                     id1]} {
        return 0
        }


set filter_id [lindex $id1 end]


if {$::ptp_communication_model == "Unicast" && $::ptp_transport_type == "UDP/IPv4"} {

set result [UNICAST_PRE_REQUISITE_CONFIGURATION\
		-session_id             $session_id\
		-port_num		$tee_port_1\
		-tee_mac		$tee_mac\
		-tee_ip			$tee_ip\
		-dut_ip			$dut_ip\
		-ethertype		$eth_type1\
		-vlan_id		$vlan_id\
		-type		        $eth_type2\
		-filter_id		$filter_id\
		-dut_mac		dut_mac\
		-timeout		$arp_timeout]

if { $result == 0 } {

	ERRLOG -msg "$tee_log_msg(ARP_F1)"
         TEE_CLEANUP
         TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ARP_F2)"\
                              -tc_def $tc_def

       return 0

}
set tee_pdest_mac  $tee_mac
set dut_pdest_mac  $dut_mac
set tee_dest_mac   $tee_mac
set dut_dest_mac   $dut_mac
set tee_pdest_ip   $::tee_test_port_ip
set dut_pdest_ip   $::dut_test_port_ip
set unicast_flag   1

} else {

set tee_dest_ip  $::multicast_dest_ip
set dut_dest_ip  $::multicast_dest_ip
set tee_pdest_ip $::multicast_pdest_ip
set dut_pdest_ip $::multicast_pdest_ip
set unicast_flag   0

}

set session_handler $::dut_session_handler
      


sleep $::wait_time

set gm_priority1 [expr $::ptp_dut_clock_priority1 - 1 ]
###############################################################################
# Step 3 : Send periodic ANNOUNCE message on port T1 with following parameters#
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x0B                                       #
#                  Domain Number = DN1                                        #
#                  Priority1     = X-1                                        #
###############################################################################

STEP 3 
LOG -level 0 -msg "$tee_log_msg(ANN_TX_PERIOD_T1)"
if {![ptp::send_announce \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type   	            $eth_type1\
			   -vlan_id                 $vlan_id\
			   -type                    $eth_type2\
			   -source_ip               $tee_ip\
			   -dest_ip   	            $dut_dest_ip\
			   -unicast_flag            $unicast_flag\
			   -src_port                $src_port_general\
			   -dest_port               $dst_port_general\
                           -message_type            $msg_announce\
			   -domain_number           $::default_domain\
                           -message_length          $ann_len\
                           -gmid                    $::gmid\
                           -control_field           $ctrl_announce\
                           -sequence_id             $sequence_ann\
                           -gmpriority1             $gm_priority1]} {
	            TEE_CLEANUP	
                    TC_CLEAN_AND_ABORT   -reason "$tee_log_msg(ANN_TX_T1_F)"\
                                         -tc_def $tc_def

		return 0
	}


###############################################################################
# Step 4a: Send periodic SYNC message on port T1 with following parameters.   #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x00                                       #
#                  Domain Number = DN1                                        #
###############################################################################
	
STEP 4a 
LOG -level 0 -msg "$tee_log_msg(SYN_TX_PERIOD_T1)"

if {![ptp::send_sync \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -source_mac              $tee_mac\
                           -dest_mac                $dut_dest_mac\
                           -eth_type                $eth_type1\
			   -vlan_id		    $vlan_id\
			   -type 		    $eth_type2\
		           -source_ip               $tee_ip\
		           -dest_ip	            $dut_dest_ip\
			   -src_port		    $src_port_event\
			   -dest_port		    $dst_port_event\
		  	   -unicast_flag            $unicast_flag\
			   -twostep_flag            $::twostep_flag\
                           -message_type            $msg_sync\
                           -message_length          $::sync_len\
			   -domain_number           $::default_domain\
                           -control_field           $::ctrl_sync\
                           -sequence_id             $sequence_sync]} {

           TEE_CLEANUP 
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(SYN_TX_T1_F)"\
                              -tc_def $tc_def				   

		return 0
	}
if { $clock_step == "One Step" } {
pltLib::reset_all_stats\
        -session_id         $::session_id\
        -port_num           $tee_port_1
}
###############################################################################
# Step 4b: If the clock is two step clock, send FOLLOW_UP message on port T1  #
#          with following parameters:                                         #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type  = 0x08                                       #
#                  Domain Number = DN1                                        #
###############################################################################

if { $clock_step == "Two Step" } {

pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1
STEP 4b 
LOG -level 0 -msg "$tee_log_msg(FOLL_TX_PERIOD_T1)"

        if {![ptp::send_followup\
			 -session_id              $::session_id\
                         -port_num                $tee_port_1\
                         -source_mac              $tee_mac\
                         -dest_mac                $dut_dest_mac\
                         -eth_type                $eth_type1\
			 -vlan_id		  $vlan_id\
			 -type 		  	  $eth_type2\
		         -source_ip               $tee_ip\
		         -dest_ip	          $dut_dest_ip\
			 -src_port		  $src_port_general\
			 -dest_port		  $dst_port_general\
		  	 -unicast_flag            $unicast_flag\
                         -message_length          $::followup_len\
                         -message_type            $msg_followup\
			 -domain_number           $::default_domain\
                         -control_field           $::ctrl_followup\
                         -sequence_id             $sequence_sync]} {

           TEE_CLEANUP 
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FOLL_TX_T1_F)"\
                              -tc_def $tc_def				 

		return 0 
	
		}
}


###############################################################################
# Step 5 : Observe that DUT transmits PDELAY_REQ message on the port P1 with  #
#          following parameters :                                             #
#                                                                             #
#              PTP Header                                                     #
#                  Message Type              = 0x02                           #
#                  Domain Number             = DN1                            #
#                  Sequence ID               = D                              #
#                  Source Port Identity      = E                              #
###############################################################################

STEP 5 
LOG -level 0 -msg "$tee_log_msg(PDREQ_RX_OBS)"

if {![ptp::recv_pdelayreq \
                           -session_id                 $::session_id\
                           -port_num                   $tee_port_1\
                           -dest_mac                   $tee_pdest_mac\
                           -eth_type   	               $eth_type1\
			   -vlan_id                    $vlan_id\
			   -type                       $eth_type2\
			   -source_ip                  $dut_ip\
			   -dest_ip   	               $tee_pdest_ip\
			   -src_port		       $src_port_event\
			   -dest_port                  $dst_port_event\
			   -reset_count		       $reset_count_off\
                           -message_type               $msg_pdelayreq\
			   -domain_number              $domain_no\
			   -timeout                    $pdelay_timeout\
                           -filter_id                  $filter_id\
                           -sportid                    sportid\
                           -clock_id                   clock_id\
                           -seq_id                     seq_id]} {

          TEE_CLEANUP
           TC_CLEAN_AND_ABORT -reason "$tee_log_msg(PDREQ_RX_P1_F)"\
                              -tc_def $tc_def

       return 0
       }
set seq_id1 $seq_id
set reqsrcPortID $sportid
set reqclockIdentity $clock_id
set tee_mac1 $::TEEMAC_1
if { $::clock_Identity_format == "EUI-64 constructed from a EUI-48" } {
set clockid1 [clock_identity $tee_mac1]
} else {
set clockid1 [clock_identity_noneui $tee_mac1]
}
set srcPortID1 5

###############################################################################
# Step 6 : Send PDELAY_RESP on the port P1 and with following parameters:     #
#                                                                             #
#              PTP Header                                                     #
#                  Source Mac               = SRC1                            #
#                  Clock ID                 = CLK1                            #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
#                  Sequence Id              = D                               #
#                  Requesting Port Identity = E                               #
###############################################################################
STEP 6 
LOG -level 0 -msg "$tee_log_msg(PDRESP_TX_T1)"

if {![ptp::send_pdelayresp\
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $tee_mac1\
                        -dest_mac             $dut_pdest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $tee_ip\
                        -dest_ip              $dut_pdest_ip\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -unicast_flag         $unicast_flag\
                        -message_type         $msg_pdelayresp\
			-twostep_flag         $::twostep_flag\
                        -domain_number        $domain_no\
                        -sequence_id          $seq_id1\
                        -clockIdentity        $clockid1\
                        -src_port_identity    $srcPortID1\
                        -reqngport_idnty      $reqclockIdentity\
                        -reqngport_id         $reqsrcPortID]} {
          
            TEE_CLEANUP
            TC_CLEAN_AND_ABORT -reason "$tee_log_msg(PDRESP_TX_T1_F)"\
                               -tc_def $tc_def
						
		return 0
       }
set tee_mac2 "000a0b0c0d1e"
if { $::clock_Identity_format == "EUI-64 constructed from a EUI-48" } {
set clockid2 [clock_identity $tee_mac2]
} else {
set clockid2 [clock_identity_noneui $tee_mac2]
}
set tee_mac2 "00:0a:0b:0c:0d:1e"
set srcPortID2 6
###############################################################################
# Step 7 : Send PDELAY_RESP on the port P1 and with following parameters:     #
#                                                                             #
#              PTP Header                                                     #
#                  Source Mac               = SRC2                            #
#                  Clock ID                 = CLK2                            #
#                  Message Type             = 0x03                            #
#                  Domain Number            = DN1                             #
#                  Sequence Id              = D                               #
#                  Requesting Port Identity = E                               #
###############################################################################
STEP 7 
LOG -level 0 -msg "$tee_log_msg(PDRESP_TX_T1)"

if {![ptp::send_pdelayresp\
                        -session_id           $::session_id\
                        -port_num             $tee_port_1\
                        -source_mac           $tee_mac2\
                        -dest_mac             $dut_pdest_mac\
                        -eth_type             $eth_type1\
                        -vlan_id              $vlan_id\
                        -type                 $eth_type2\
                        -source_ip            $tee_ip\
                        -dest_ip              $dut_pdest_ip\
                        -src_port             $src_port_event\
                        -dest_port            $dst_port_event\
                        -unicast_flag         $unicast_flag\
                        -message_type         $msg_pdelayresp\
			-twostep_flag         $::twostep_flag\
                        -domain_number        $domain_no\
                        -sequence_id          $seq_id1\
                        -clockIdentity        $clockid2\
                        -src_port_identity    $srcPortID2\
                        -reqngport_idnty      $reqclockIdentity\
                        -reqngport_id         $reqsrcPortID]} {
          
            TEE_CLEANUP
            TC_CLEAN_AND_ABORT -reason "$tee_log_msg(PDRESP_TX_T1_F)"\
                               -tc_def $tc_def
						
		return 0
       }
sleep $::wait_time
###############################################################################
# Step 8 : Check whether the DUT port P1 is in FAULTY state.                  #
###############################################################################

STEP 8
LOG -level 0 -msg "$dut_log_msg(CHK_PTP_PORT_FAULT_STATE)"

if {![ptp::dut_check_state \
                    -session_handler         $session_id\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $Faulty] } {
        TEE_CLEANUP
	TC_CLEAN_AND_ABORT -reason "$dut_log_msg(CHK_PTP_PORT_FAULT_STATE_F)."\
			   -tc_def $tc_def
return 0 
}
LOG -level 0 -msg "$dut_log_msg(CHK_PTP_PORT_FAULT_STATE_P)"
###############################################################################
# Step 9 : Disable PTP on port P1.                                            #
###############################################################################
STEP 9
LOG -level 0 -msg "$dut_log_msg(PTP_DISABLE)"

if {![ptp::dut_port_ptp_disable\
                      -session_handler     $session_handler\
                      -port_list           $port_list]} {

                  TEE_CLEANUP
                  TC_CLEAN_AND_ABORT   -reason "$dut_log_msg(PTP_DISABLE_F)"\
                                       -tc_def $tc_def

           return 0
   }
pltLib::reset_all_stats\
	-session_id         $::session_id\
	-port_num	    $tee_port_1

###############################################################################	
# Step 10: Verify whether the DUT port P1 is in DISABLED state.               #
###############################################################################
STEP 10
LOG -level 0 -msg "$dut_log_msg(PTP_PORT_DIS_STATE_V)"

if {![ptp::dut_check_state \
                    -session_handler         $session_id\
	            -port_num		     $dut_port_1\
		    -clock_mode              $clock_mode\
		    -port_state              $Disable] } {
        TEE_CLEANUP
        TC_CLEAN_AND_FAIL -reason "$dut_log_msg(DUT_PRT_DIS_F)"\
			   -tc_def $tc_def
return 0 
}
LOG -level 0 -msg "$dut_log_msg(DUT_PRT_DIS)"

###############################################################################
# Step 11: Verify that DUT does not transmit any PTP message on port P1.      #
###############################################################################
STEP 11
LOG -level 0 -msg "$tee_log_msg(NO_PTP_MSG_V)"
sleep $announceReceiptTimeoutInterval
set ::semlock "LOCKED"
if {![ptp::recv_ptpmessage \
                           -session_id              $::session_id\
                           -port_num                $tee_port_1\
                           -filter_id               $filter_id]} {
set ::semlock "UNLOCKED"
          TEE_CLEANUP
          TC_CLEAN_AND_PASS  -reason "DUT does not transmit any PTP messages on port P1."\
                             -tc_def $tc_def
          return 1
  } else {

set ::semlock "UNLOCKED"
TEE_CLEANUP
TC_CLEAN_AND_FAIL  -reason "DUT transmits PTP  messages on port P1."\
                   -tc_def $tc_def
return 0
}
